/*
 *	elong
 *
 *	Given the right ascension and declination of two objects, find the
 *	angle between them.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
extern void exit(int);

#define	PI	3.141592653589793

char *Prog;

double get_ra(char *ra_string);
double get_dec(char *dec_string);
char *get_dms(double angle);

int
main(int argc, char **argv)
{
	/*
	 * Each variable name says 1 for object 1, or 2 for object 2.  The
	 * suffix is hr for hours, deg for degrees, or rad for radians.
	 */
	double ra1_hr;
	double dec1_deg;
	double ra2_hr;
	double dec2_deg;
	double ra1_rad;
	double dec1_rad;
	double ra2_rad;
	double dec2_rad;

	/*
	 * Pretending that the celestial sphere is of radius 1, these are the
	 * x, y, z coordinate of the objects.  The X axis points at the
	 * vernal equinox, the Z axis points at the north celestial pole, and
	 * the Y axis is as it needs to be to make a normal right handed
	 * coordinate system: RA 6 hours, dec 0.
	 */
	double obj1_x, obj1_y, obj1_z;
	double obj2_x, obj2_y, obj2_z;

	double dx, dy, dz; /* distances between the objects for each coord */

	double dist;		/* direct distance between the objects */
	double cosangle;	/* cosine of the angle between them */
	double angle_deg;	/* the angle between them in degrees */


	Prog = argv[0];

	if (argc != 5) {
		fprintf(stderr, "Usage:  %s ra1 dec1 ra2 dec2\n", Prog);
		fprintf(stderr, "\tra  format: Hh or HhMm or HhMmSs (any can be decimals)\n");
		fprintf(stderr, "\tdec format: Do or DoM' or DoM'S\" (any can be decimals)\n");
		fprintf(stderr, "\t\twith optional + or - before the degs\n");
		exit(1);
	}

	/* get RA and declination from the command line */
	ra1_hr = get_ra(argv[1]);
	dec1_deg = get_dec(argv[2]);
	ra2_hr = get_ra(argv[3]);
	dec2_deg = get_dec(argv[4]);

#ifdef DEBUG
	printf("ra1_hr=%lf dec1_deg=%lf ra2_hr=%lf dec2_deg=%lf\n", 
		ra1_hr, dec1_deg, ra2_hr, dec2_deg);
#endif

	/* convert RA and declination to radians */
	ra1_rad = ra1_hr * PI / 12.0;
	ra2_rad = ra2_hr * PI / 12.0;
	dec1_rad = dec1_deg * PI / 180.0;
	dec2_rad = dec2_deg * PI / 180.0;

	/* convert to x, y, z coords */
	obj1_x = cos(dec1_rad) * cos(ra1_rad);
	obj1_y = cos(dec1_rad) * sin(ra1_rad);
	obj1_z = sin(dec1_rad);
	obj2_x = cos(dec2_rad) * cos(ra2_rad);
	obj2_y = cos(dec2_rad) * sin(ra2_rad);
	obj2_z = sin(dec2_rad);

#ifdef DEBUG
	printf("Object 1:  (%lf, %lf, %lf)\n", obj1_x, obj1_y, obj1_z);
	printf("Object 2:  (%lf, %lf, %lf)\n", obj2_x, obj2_y, obj2_z);
#endif

	/* get distance between objects, using 3D pythagorean theorem */
	dx = obj1_x - obj2_x;
	dy = obj1_y - obj2_y;
	dz = obj1_z - obj2_z;
	dist = sqrt(dx * dx + dy * dy + dz * dz);

	/* use law of cosines to get the cosine of the angle */
	cosangle = (2.0 - dist * dist) / 2.0;
#ifdef DEBUG
	printf("cos(angle) = %lf\n", cosangle);
#endif

	/* guard against roundoff error giving illegal cosines */
	if (cosangle > 1.0) {
		cosangle = 1.0;
	}
	if (cosangle < -1.0) {
		cosangle = -1.0;
	}

	/* get the angle in degrees, and print */
	angle_deg = acos(cosangle) * 180.0 / PI;
	printf("angle = %lf degrees = %s\n", angle_deg, get_dms(angle_deg));

	return 0;
}

/*
 * Convert an RA input string to decimal hours.  Make a half hearted attempt
 * to enforce the correct syntax.
 */
double
get_ra(char *ra_string)
{
	int num_fields;
	double hr, min, sec;

	if (strspn(ra_string, "0123456789.hms") != strlen(ra_string)) {
		fprintf(stderr, "%s: invalid right ascension string \"%s\"\n",
				Prog, ra_string);
		exit(1);
	}
	num_fields = sscanf(ra_string, "%lfh%lfm%lf", &hr, &min, &sec);
	if (num_fields == 0) {
		fprintf(stderr, "%s: invalid right ascension string \"%s\"\n",
				Prog, ra_string);
		exit(1);
	}
	if (num_fields >= 2) {
		hr += min / 60.0;
	}
	if (num_fields == 3) {
		hr += sec / 3600.0;
	}
	return hr;
}

/*
 * Convert a declination input string to decimal hours.  Make a half hearted
 * attempt to enforce the correct syntax.
 */
double
get_dec(char *dec_string)
{
	int num_fields;
	double deg, min, sec;
	char *num_ptr;
	int sign;

	num_ptr = dec_string;
	sign = 1;
	if (dec_string[0] == '+' || dec_string[0] == '-') {
		num_ptr++;
		if (dec_string[0] == '-') {
			sign = -1;
		}
	}
	if (strspn(num_ptr, "0123456789.o'\"") != strlen(num_ptr)) {
		fprintf(stderr, "%s: invalid declination string \"%s\"\n",
				Prog, dec_string);
		exit(1);
	}
	num_fields = sscanf(num_ptr, "%lfo%lf'%lf", &deg, &min, &sec);
	if (num_fields == 0) {
		fprintf(stderr, "%s: invalid declination string \"%s\"\n",
				Prog, dec_string);
		exit(1);
	}
	if (num_fields >= 2) {
		deg += min / 60.0;
	}
	if (num_fields == 3) {
		deg += sec / 3600.0;
	}
	if (sign == -1) {
		deg = -deg;
	}
	return deg;
}

/*
 * Convert an angle in degrees to a string formatted degree, minutes, seconds.
 */
char *
get_dms(double angle)
{
	int deg_int;
	int min_int;
	int sec_int;
	static char buf[30];

	angle += 0.5 / 3600.0;	/* add 1/2 second for rounding */
	deg_int = angle;
	angle -= deg_int;
	angle *= 60.0;
	min_int = angle;
	angle -= min_int;
	angle *= 60.0;
	sec_int = angle;
	sprintf(buf, "%do %d' %d\"", deg_int, min_int, sec_int);
	return buf;
}
