/*	lunsol
**	15 Aug 85
**
**	Add moon information to sun program.
**	To compile: cc -O -o lunsol lunsol.c -lm
**
**	lunsol <options>
**		-t hh:mm:ss	time (default is current system time)
**		-d mm/dd/yy	date (default is current system date)
**		-a lat		decimal latitude (default = 45.5333)
**		-o lon		decimal longitude (default = 122.8333) 
*		-z tz		timezone (default = 8, pst)
**		-p		show position of sun (azimuth)
*        
*        All output is to standard io.  
*
*	 Note that the latitude, longitude, time zone correction and
*	 time zone string are all defaulted in the global variable section.
*
*	 This program was inspired by, and derived from, a program
*	 that provided simular information about the sun.  Written
*	 by Robert Bond - Beaverton Oregon.
*
*	 Most of the code in this program is adapted from algorithms
*	 presented in "Practical Astronomy With Your Calculator" by
*	 Peter Duffet-Smith.
*
*	 The GST and ALT-AZIMUTH algorithms are from Sky and Telescope,
*	 June, 1984 by Roger W. Sinnott
*
*	1.35 850820 in lon_to_ed: /cos(lambda) not cepsilon
*	1.36 850821 fix pnt?ms when deg or hr 0 but neg. redo disp
*	1.37 850821 Add age and size and moon name, renam lunsol
*	1.40 850822 Add sunrise/sunset info, nflag
*	1.43 851018 catch -t with no arg problems
*	1.44 851220 show month name not number  "original version"
*	1.45 130527 tweaks; change -a and -o to work like "planet"
*
*/


#define VERNUM "1.44"

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define PI       3.141592653589793

  /* this is the 1980 value. Should use same as EPOCH */
#define	OBLIQ_ECLIP	23.441884

	/* these are 1980 values */
#define EPOCH   1980
#define JDE	 2444238.5	/* Julian date of EPOCH (1980)	*/
#define EPSILONG 278.833540     /* solar ecliptic long at EPOCH */
#define RHOG     282.596403     /* solar ecliptic long of perigee  */
#define ESOL       0.016718     /* solar orbit eccentricity */
#define LZERO     64.975464     /* lunar mean long at EPOCH */
#define PZERO    349.383063     /* lunar mean long of perigee at EPOCH */
#define NZERO    151.950429     /* lunar mean long of node at EPOCH */
#define LUNI	   5.145396	/* inclination of lunar orbit	*/
#define LUNE	   0.054900	/* eccentricity of lunar orbit	*/
#define LUNA  384401.0		/* semi-major axis (km)		*/


double addtime();
double dtor();
double rtod();
double adj360();
double adj24();
double julian_date();
double hms_to_dh();
double acos_deg();
double asin_deg();
double atan_q_deg();
double atan_deg();
double sin_deg();
double cos_deg();
double tan_deg();
double gmst();

int th;
int tm;
int ts;
int mo;				/* month num Jan = 1	*/
int day;			/* day of month		*/
int yr;				/* four digits		*/
int tz=6;			/* Default time zone	*/
char *tzs = "(CST)";		/* Default time zone string */
char *dtzs = "(CDT)";		/* Default daylight savings time string */
int popt = 0;			/* true if -p option requested */
int nflag = 0;			/* on for debug printfs		*/

double lat = 42.033;		/* Default latitude (Elgin Ill.) */
double lon = 88.267;		/* Default Longitude (Degrees west) */ 

/* these are global so moonloc() can use them */
double msol;	/* solar mean anomaly	*/
double lambdasol;  /* solar ecliptic long */

/* indian names for the various moons */
/* the name is chosen by the month in which the full moon occurs */
char *Mntbl[] = {
	"dummy",		/* so array index == month	*/
	"Wolf",			/* Jan.				*/
	"Snow",			/* Feb.				*/
	"Worm",			/* Mar.				*/
	"Pink",			/* Apr. also Grass, Egg, Fish	*/
	"Flower",		/* May.				*/
	"Strawberry", 		/* June. also Rose		*/
	"Thunder",		/* July. also Buck 		*/
	"Corn",			/* Aug. also Red, Sturgeon	*/
	"Harvest",		/* Sept.			*/
	"Hunters",		/* Oct.				*/
	"Beaver",		/* Nov.				*/
	"Cold",			/* Dec.				*/
	"Blue"			/* have 13 moons every 6 yrs	*/
};

char	*mtab[] = {
	"dummy", "January", "February", "March", "April", "May", "June",
	"July", "August", "September", "October", "November", "December",
	};



main(argc,argv)
int	argc;
char	*argv[];
{
	double ed, jd;
	double alpha1, delta1, alpha2, delta2, st1r, st1s, st2r, st2s;
	double a1r, a1s, a2r, a2s, dt, dh, x, y;
	double trise, tset, ar, as, alpha, delta, tri, da;
	double lambda1, lambda2;
	double alt, az, gst, m1, lst;
	double hsm, ratio;
	double strise, stset, mtrise, mtset;	/* rise and set times */
	double ra, dec;		/* moons location */
	double melat, melon;	/* lunar ecliptic location	*/
	double age, size;
	int mnidx;		/* index in moon name table	*/
	time_t sec_1970;
	int h, m;
	struct tm *pt;

	sec_1970 = time((time_t *)0);  
	pt = localtime(&sec_1970);  

	th = pt->tm_hour;
	tm = pt->tm_min;
	ts = pt->tm_sec;
	yr = pt->tm_year + 1900;	/* unix uses last two digits	*/
	mo = pt->tm_mon + 1;		/* unix uses Jan = 0	*/
	day = pt->tm_mday;

	/* convert tz to daylight savings time */
	if (pt->tm_isdst) {
		tz--;
		tzs = dtzs;	
	}

	initopts(argc,argv);

	jd = julian_date(mo,day,yr);
	ed = jd - JDE;

	printf("lunsol.c version %s \n", VERNUM);
	if (nflag)  {
		printf("For julian day %f \n", jd);
		printf("jd-JDE= %f \n", ed);
	}

	/* find suns location for today and tomorrow */
	/* Note: msol ignored now. Reuse later	*/
	solar_loc(ed, &msol, &lambda1);
	solar_loc(ed + 1.0, &msol, &lambda2);
	if (nflag)
		printf("solar_loc OK \n");
	lon_to_eq(lambda1, 0.0, &alpha1, &delta1);
	lon_to_eq(lambda2, 0.0, &alpha2, &delta2);
	if (nflag)
		printf("lon_to_eq  OK \n");

	rise_set(alpha1, delta1, &st1r, &st1s, &a1r, &a1s);
	rise_set(alpha2, delta2, &st2r, &st2s, &a2r, &a2s);
	if (nflag)
		printf("rise_set  OK \n");

	/* lst midnight */
	m1 = adj24(gmst(jd - 0.5, 0.5 + tz / 24.0) - lon / 15);
	hsm = adj24(st1r - m1);
	ratio = hsm / 24.07;

	if (fabs(st2r - st1r) > 1.0) {
		st2r += 24.0;
	}

	trise = adj24((1.0 - ratio) * st1r + ratio * st2r);

	hsm = adj24(st1s - m1);
	ratio = hsm / 24.07;

	if (fabs(st2s - st1s) > 1.0) {
		st2s += 24.0;
	}

	tset = adj24((1.0 - ratio) * st1s + ratio * st2s);

	ar = a1r * 360.0 / (360.0 + a1r - a2r);
	as = a1s * 360.0 / (360.0 + a1s - a2s);
	if (nflag)
		printf("stage 2  OK \n");

	delta = (delta1 + delta2) / 2.0;
	tri = acos_deg(sin_deg(lat)/cos_deg(delta));
	if (nflag)
		printf("stage 3  OK \n");

	/* correction for refraction, parallax, size of sun */
	x = 0.835608;
	y = asin_deg(sin_deg(x)/sin_deg(tri));
	if (nflag)
		printf("stage 4  OK \n");

	da = asin_deg(tan_deg(x)/tan_deg(tri));
	dt = 240.0 * y / cos_deg(delta) / 3600;
	if (nflag)
		printf("stage 5  OK \n");

	/* Start displaying the data */
/*	printf("\nOn %d/%02d/%d \n", mo, day, yr);
*/
	/* show month name, not number */
	if ( (mo < 1) || (mo > 12) ) {
		printf("\nInvalid month (%d) \n", mo);
		usage();
		exit(1);
	}
	printf("\nFor: %s %d, %d \n", mtab[mo], day, yr);


	lst_to_lct(trise - dt, jd, &strise);
	printf("Sunrise:");
	pnthms(strise);
        printf(" %s",tzs);

	if (popt) {
	        printf("\t\tAzimuth: ");
		pntdms(ar - da);
	}
	putchar('\n');

	lst_to_lct(tset + dt, jd, &stset);
	printf("Sunset: ");
	pnthms(stset);
        printf(" %s",tzs);

	if (popt) {
	        printf("\t\tAzimuth: ");
		pntdms(as + da);
	}
	putchar('\n');

	/* add hours and minutes into day to ed */
	ed = addtime(ed);	/* add time into day */

	/* find sun's true anomaly and ecliptic long */
	solar_loc(ed, &msol, &lambdasol);
	if (nflag)
		printf("Sun MA %f, long %f \n", msol, lambdasol);

	/* conv ecliptic to equatorial for sun */
	lon_to_eq(lambdasol, 0.0, &alpha, &delta);
	if (nflag) 
		printf("Solar ra %f, dec %f \n", alpha, delta);

	/* find the current lunar ecliptic long and lat */
	moonloc(ed, &melon, &melat, &age, &size);
	if (nflag) 
		printf("  melat %f, melon %f \n", melat, melon);

	/* conv ecliptic to equatorial */
	lon_to_eq(melon, melat, &ra, &dec);
	if (nflag) 
		printf("ra %f, dec %f \n", ra, dec);

	/* if p option not given, exit here... */
	if (popt == 0)
		exit (0);	/* POSSABLE EXIT */

	/* else show position information */
	printf("\nAt %02d:%02d:%02d %s\n", th, tm, ts, tzs);

	dh = (hms_to_dh(th, tm, ts) + tz) / 24.0;
	if (dh > 0.5) {		/* correct for jd starts at noon */
	    dh -= 0.5;
	    jd += 0.5;
	} else {
	    dh += 0.5;
	    jd -= 0.5;
	}

	gst = gmst(jd, dh);

	/* alpha is RA in hours, delta is dec in deg */
	eq_to_altaz(alpha, delta, gst, &alt, &az);

	printf("The sun is located at: \n");
	printf("   RA:  ");
	pnthms(alpha);
	printf("   \t\tAltitude: ");
	pntdms(alt);
	putchar('\n');
	printf("   Dec: ");
	pntdms(delta);
	printf("   \tAzimuth:  ");
	pntdms(az);
	putchar('\n');

	/* alpha is RA in hours, delta is dec in deg */
	alpha = ra;
	delta = dec;
	eq_to_altaz(alpha, delta, gst, &alt, &az);

	/*
	** The first full moon in Jan is the Wolf moon.
	** Later figger out how to do this right...
	** but for now just use this...
	** This will in and after a month that has two moons.
	** ie: year has 13 moons (every six years)
	*/
	mnidx = (age < (double)day) ? mo : mo -1;
	if (mnidx < 1)
		mnidx += 12;

	printf("The %s moon is %.1f days old and %.1f%% of full.\n",
		Mntbl[mnidx], age, size);
	printf("   RA:  ");
	pnthms(alpha);
	printf("   \t\tAltitude: ");
	pntdms(alt);
	putchar('\n');
	printf("   Dec: ");
	pntdms(delta);
	printf("   \tAzimuth:  ");
	pntdms(az);
	putchar('\n');

	  /* convert GST to LST and display */
	lst = gst - lon / 15.0;
	if (lst < 0)
		lst += 24.0;
	printf("LST: ");
	pnthms(lst);
	putchar('\n');
	putchar('\n');
}

double
addtime(ed)		/* add time into day to epic day */
double	ed;
{
	int	h;

	h = th + tz;
	if (h >= 24) {
		ed++;
		h -= 24;
	}
	ed += (double)h / 24.0;
	ed += (double)tm / (24.0 * 60.0);
	ed += (double)ts / (24.0 * 60.0 * 60.0);
	return (ed);	/* ed in UTC */
}


moonloc(ed, melon, melat, age, size)
double	ed;		/* time since epoch in decimal days */
double	*melon;		/* ecliptic long	*/
double	*melat;		/* ecliptic lat		*/
double	*age;		/* age in days		*/
double	*size;		/* percent illuminated	*/
{

	  /* for moon */
	double l;	/* mean long	*/
	double Mm;	/* MA		*/
	double Ev;	/* evection corr	*/
	double Ac;	/* annual corr		*/
	double A3;	/* another corr		*/
	double Ec;	/* equ or center corr	*/
	double Mmprime;	/* corrected anomaly	*/
	double A4;	/* another corr		*/
	double lprime;	/* corrected long	*/
	double V;	/* variation		*/
	double ldprime;	/* true orbital long	*/
	double Nprime;
	double y, x;
	double betam;	/* ecliptic lat		*/
	double lambdam;	/* ecliptic long	*/
	double D;
	double Nm;	/* mean long, ascending node	*/
	double lage, phase;


	if (nflag)
		printf("ed (D) = %f \n", ed);

	/* moon mean long */
	l = 13.1763966 * ed + LZERO;       /* sec 61 #4 */
	l = adj360(l);
	if (nflag)
		printf("l %f \n", l);
	/* moon mean anomaly */
	Mm = l - (0.1114041 * ed) - PZERO; /* sec 61 #5 */
	Mm = adj360(Mm);
	if (nflag)
		printf("Mm %f \n", Mm);
	/* mean long - ascending node */
	Nm = NZERO - (0.0529539 * ed);     /* sec 61 #6 */
	Nm = adj360(Nm);
	if (nflag)
		printf("Nm %f \n", Nm);
	/* eviction */
	Ev = 1.2739 * sin(dtor(2.0*(l - lambdasol) - Mm)); /* sec 61 #7 */

	/* annual equation */
	Ac = 0.1858 * sin(dtor(msol));       /* sec 61 #8 */
	A3 = 0.37 * sin(dtor(msol));

	/* corrected anomaly */
	Mmprime = Mm + Ev - Ac - A3;         /* sec 61 #9 */
	if (nflag)
		printf("Mmprime %f \n", Mmprime);
	/* equation of center correction */
	Ec = 6.2886 * sin(dtor(Mmprime));    /* sec 61 #10 */
	A4 = 0.214 * sin(dtor(2.0 * Mmprime)); /* sec 61 #11 */

	/* moons corrected long */
	lprime = l + Ev + Ec - Ac + A4;      /* sec 61 #12 */
	if (nflag)
		printf("lprime %f \n", lprime);
	/* variation correction */
	V = 0.6583 * sin(dtor(2.0 * (lprime - lambdasol))); /* sec 61 #13 */

	/* moons true orbital long */
	ldprime = lprime + V;                /* sec 61 #14 */
	if (nflag)
		printf("ldprime %f \n", ldprime);
	/* find ecliptic lat and long */
	Nprime = Nm - (0.16 * sin_deg(msol));	/* sec 61 #15 */
	if (nflag)
		printf("Nprime %f \n", Nprime);
	y = sin_deg(ldprime - Nprime) * cos_deg(LUNI);	/* sec 61 #16 */
	x = cos_deg(ldprime - Nprime);		/* sec 61 #17 */

	lambdam = atan_q_deg(y, x);		/* sec 61 #18 */
	lambdam += Nprime;			/* sec 61 #19 */
	betam = asin_deg(sin_deg(ldprime - Nprime) * sin_deg(LUNI));

	if (nflag)
		printf("lambdam %f, betam %f \n", lambdam, betam);

	*melon = lambdam;
	*melat = betam;


	/* calc and show age and phase */
	/* see #63 page 146 */
	lage = ldprime - lambdasol;
	phase = (1 - cos_deg(lage)) / 2;
	if (nflag)
		printf("age %f, phase %f \n", lage, phase);
	lage = adj360(lage);
	*age = (lage / 360.0) * 29.5306; /* days into synodic month */
	*size = phase * 100.0;		/* conv to percent	*/
	return;
}

/*
 * Function:	hms_str2float
 *
 * Description:	This function converts a string representing hours, minutes,
 *		and seconds to a floating point number in hours.  These
 *		formats are allowed:
 *
 * 			HOURS
 * 			HOURS h
 * 			HOURS h MINUTES
 * 			HOURS h MINUTES m
 * 			HOURS h MINUTES m SECONDS
 * 			HOURS h MINUTES m SECONDS s
 *
 *		where HOURS, MINUTES, and SECONDS are numbers having no sign
 *		and there are not actually any blanks between tokens.  All
 *		numbers may contain a decimal point, even though (for example)
 *		it would be strange to have minutes along with fractional hours.
 *		Whatever numbers are given must be in these ranges:
 *
 *			0 <= hours < 24
 *			0 <= minutes < 60
 *			0 <= seconds < 60
 *
 *		The sum must be less than 24.
 *
 * Returns:	0 if success, -1 if failure.
 */

int
hms_str2float(

char *str,		/* the string input */
float *ans_ptr		/* answer filled in, valid only on success */
)
{
	char *beg_ptr;		/* point at first char of a number */
	char *end_ptr;		/* point after last char of a number */
	float temp;		/* each number as parsed */


	/* only these characters are allowed in the string */
	if (str[0] == '\0' || strspn(str, "0123456789.hms") != strlen(str)) {
		 return -1;
	}

	beg_ptr = str;		/* start of hours number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 'h' && *end_ptr != '\0') {
		 return -1;	/* can't parse hours or junk after it */
	}
	*ans_ptr = temp;
	if (temp >= 24.0) {
		 return -1;
	}
	if (*end_ptr == '\0' || *end_ptr == 'h' && *(end_ptr + 1) == '\0') {
		return 0;	/* nothing, or just "h", after hours */
	}

	beg_ptr = end_ptr + 1;	/* start of minutes number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 'm' && *end_ptr != '\0') {
		 return -1;	/* can't parse minutes or junk after it */
	}
	*ans_ptr += temp / 60.0;
	if (temp >= 60.0 || *ans_ptr >= 24.0) {
		 return -1;
	}
	if (*end_ptr == '\0' || *end_ptr == 'm' && *(end_ptr + 1) == '\0') {
		return 0;	/* nothing, or just "m", after minutes */
	}

	beg_ptr = end_ptr + 1;	/* start of seconds number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 's' && *end_ptr != '\0') {
		 return -1;	/* can't parse seconds or junk after it */
	}
	*ans_ptr += temp / 3600.0;
	if (temp >= 60.0 || *ans_ptr >= 24.0) {
		 return -1;
	}
	if (*end_ptr == '\0' || *end_ptr == 's' && *(end_ptr + 1) == '\0') {
		return 0;	/* nothing, or just "s", after seconds */
	}

	 return -1;		/* extra junk after the end */
}



/*
 * Function:	dms_str2float
 *
 * Description:	This function converts a string representing degrees, minutes,
 *		and seconds to a floating point number in degrees.  These
 *		formats are allowed:
 *
 * 			DEGREES
 * 			DEGREES o
 * 			DEGREES o ARCMINUTES
 * 			DEGREES o ARCMINUTES '
 * 			DEGREES o ARCMINUTES ' ARCSECONDS
 * 			DEGREES o ARCMINUTES ' ARCSECONDS "
 *
 *		where DEGREES, ARCMINUTES, and ARCSECONDS are numbers,
 *		ARCMINUTES and ARCSECONDS having no sign, and DEGREES
 *		optionally having a + or - sign, and there are not actually any
 *		blanks between tokens.  All numbers may contain a decimal point,
 *		even though (for example) it would be strange to have
 *		arcminutes along with fractional degrees.  If DEGREES has a
 *		sign, it is assumed to apply to all the numbers.  Whatever
 *		numbers are given must be in these ranges (disregarding the
 *		sign):
 *
 *			0 <= degrees < 360
 *			0 <= arcminutes < 60
 *			0 <= arcseconds < 60
 *
 *		The sum must be less than 360.
 *
 * Returns:	0 if success, -1 if failure.
 */

int
dms_str2float(

char *str,		/* the string input */
float *ans_ptr		/* answer filled in, valid only on success */
)
{
	char *beg_ptr;		/* point at first char of a number */
	char *end_ptr;		/* point after last char of a number */
	float temp;		/* each number as parsed */
	int sign;		/* -1 if negative input, else 1 */


	/* set sign if present, and skip by it */
	if (str[0] == '-') {
		sign = -1;
		str++;
	} else if (str[0] == '+') {
		sign = 1;
		str++;
	} else {
		sign = 1;
	}

	/* only these characters are allowed in the rest of the string */
	if (str[0] == '\0' || strspn(str, "0123456789.o'\"") != strlen(str)) {
		 return -1;
	}

	beg_ptr = str;		/* start of degrees number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 'o' && *end_ptr != '\0') {
		 return -1;	/* can't parse hours or junk after it */
	}
	*ans_ptr = temp;
	if (temp > 360.0) {
		 return -1;
	}
	if (*end_ptr == '\0' || *end_ptr == 'o' && *(end_ptr + 1) == '\0') {
		*ans_ptr *= sign;
		return 0;	/* nothing, or just "o", after degrees */
	}

	beg_ptr = end_ptr + 1;	/* start of arcminutes number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != '\'' && *end_ptr != '\0') {
		 return -1;	/* can't parse arcminutes or junk after it */
	}
	*ans_ptr += temp / 60.0;
	if (temp >= 60.0 || *ans_ptr > 360.0) {
		 return -1;
	}
	if (*end_ptr == '\0' || *end_ptr == '\'' && *(end_ptr + 1) == '\0') {
		*ans_ptr *= sign;
		return 0;	/* nothing, or just "'", after arcminutes */
	}

	beg_ptr = end_ptr + 1;	/* start of arcseconds number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != '"' && *end_ptr != '\0') {
		 return -1;	/* can't parse arcseconds or junk after it */
	}
	*ans_ptr += temp / 3600.0;
	if (temp >= 60.0 || *ans_ptr > 360.0) {
		 return -1;
	}
	if (*end_ptr == '\0' || *end_ptr == '"' && *(end_ptr + 1) == '\0') {
		*ans_ptr *= sign;
		return 0;	/* nothing, or just "\"", after arcseconds */
	}

	 return -1;
}

void
float_to_3_longs(float f, long *a_ptr, long *b_ptr, long *c_ptr)
{
	int sign;

	/* remember the sign, and make f positive */
	if (f < 0.0) {
		sign = -1;
		f = -f;
	} else {
		sign = 1;
	}

	/* add half a second so that round off is correct */
	f += 0.5 / 3600.0;

	/* break it out into the 3 longs */
	*a_ptr = (long)f;
	*b_ptr = (long)(60 * (f - *a_ptr));
	*c_ptr = (long)(3600 * (f - *a_ptr - *b_ptr / 60.0));

	/* if it was negative, negate all three */
	if (sign == -1) {
		*a_ptr = -*a_ptr;
		*b_ptr = -*b_ptr;
		*c_ptr = -*c_ptr;
	}
}



initopts(argc,argv)
int argc;
char *argv[];
{
    int ai;
    char *ca;
    char *str;
    float temp;
    int ret;

    while (--argc) {
        if ((*++argv)[0] == '-') {
	    ca = *argv;
	    for(ai = 1; ca[ai] != '\0'; ai++)
                switch (ca[ai]) {
		case 'p':
		    popt++;	/* enable position display	*/
		    break;
		case 'n':
		    nflag++;	/* enable debug prints	*/
		    break;
                case 'a':
                    str = *++argv;
		    if (str == (char *)0)	usage();
		    if (dms_str2float(str, &temp) != 0) {
		    	usage();
		    }
		    if (temp < -90.0 || temp > 90.0) {
		    	usage();
		    }
		    lat = temp;
                    argc--;
                    break;
                case 'o':
                    str = *++argv;
		    if (str == (char *)0)	usage();
		    if (dms_str2float(str, &temp) != 0) {
		    	usage();
		    }
		    if (temp < -180.0 || temp > 180.0) {
		    	usage();
		    }
		    lon = temp;
                    argc--;
                    break;
                case 'z':
                    str = *++argv;
		    if (str == (char *)0)	usage();
		    if (sscanf(str, "%d", &tz) != 1)
			usage();
		    tzs = "   ";
                    argc--;
                    break;
                case 't':
                    str = *++argv;
		    if (str == (char *)0)	usage();
		    ret = sscanf(str, "%d:%d:%d", &th, &tm, &ts);
		    switch (ret) {
		    case 2:
			ts = 0;
			break;
		    case 3:
			break;
		    default:
			if (nflag)
			    printf("time given: %d:%d:%d\n", th, tm, ts);
			printf("Bad time string. \n");
			usage();
		    }
                    argc--;
                    break;
                case 'd':
if (nflag) printf("\nin d case \n");
                    str = *++argv;
		    if (str == (char *)0)	usage();
if (nflag) printf("\nstr not null \n");
		    if (sscanf(str, "%d/%d/%d", &mo, &day, &yr) != 3) {
			if (nflag) {
				printf("\nBad date string.... \n");
				printf(" mo %d, day %d, yr %d \n", 
					mo, day, yr);
			}
			printf("Bad date string \n");
			usage();
		    }
                    argc--;
                    break;
                default: usage();
                }
        } else usage();
    }
}

usage()
{
    printf("Usage: lunsol [-p] [-t hh:mm:ss] [-d mm/dd/yyyy] [-a lat] [-o lon] [-z tz]\n");
    exit(1);
}

double 
julian_date(m, d, y)
{
    int a, b;
    double jd;

    if (m == 1 || m == 2) {
	--y;
	m += 12;
    }
    if (y < 1583) {
	printf("Can't handle dates before 1583 \n");
	exit(1);
    }
    a = y/100;
    b = 2 - a + a/4;
    b += (int)((double)y * 365.25);
    b += (int)(30.6001 * ((double)m + 1.0));
    jd = (double)d + (double)b + 1720994.5;
    return(jd);
}


solar_loc(ed, ma, lsol)
double	ed;		/* time into epoch in days (epochdays) */
double	*ma;		/* return mean anomaly */
double	*lsol;		/* and solar long */
{
	double n, m, e, ect, errt, v;

	/* n = ma of mean sun since perigee */
	n = 360.0 * ed / 365.2422;
	n = adj360(n);
	/* correct for epoch */
	m = n + EPSILONG - RHOG;
	m = adj360(m);
	*ma = m;	/* mean anomaly */
	/* use Kepler's equ to find true anomaly (v) */
	m = dtor(m);	/* conv MA to rads */
	e = m;		/* initial quess */
	ect = ESOL;	/* eccentricity */
	while ((errt = e - ect * sin(e) - m) > 0.0000001) 
		e = e - errt / (1 - ect * cos(e));
/* fix later. calc const below using actual e. (eccentricity)
** see: M43 p84
*/
	v = 2 * atan(1.0168601 * tan(e/2));
	v = adj360(v * 180.0 / PI + RHOG);
	*lsol = v;	/* lambda sol */
	return;
}



/*
** ecliptic to equatorial conversion.
** ecliptic long and lat to RA and dec.
** sec #27 page 44 
*/
lon_to_eq(lambda, beta, alpha, delta)
double lambda;	/* ecliptic longitude	*/
double beta;	/* ecliptic latitude	*/
double *alpha;	/* return RA		*/
double *delta;	/* return declination	*/
{
    double slam, cepsilon, sepsilon;

    slam = sin_deg(lambda);
    cepsilon = cos(dtor((double)OBLIQ_ECLIP));
    sepsilon = sin(dtor((double)OBLIQ_ECLIP));

	/* get RA in hours */
    *alpha = atan_q_deg((slam*cepsilon)-(tan_deg(beta)*sepsilon),
		cos_deg(lambda))/15.0;
	/* and declination in degrees */
    *delta = asin_deg(sin_deg(beta)*cepsilon+cos_deg(beta)*sepsilon*slam);

}

/*
** find the local sidereal time when the altitude is zero.
*/
rise_set(alpha, delta, lstr, lsts, ar, as)
double	alpha;		/* RA				*/
double  delta;		/* Dec				*/
double  *lstr;		/* local sidereal time of rise	*/
double  *lsts;		/* local sidereal time of set	*/
double  *ar;		/* azimuth at rise		*/
double  *as;		/* azimuth at set		*/

{
    double tar;
    double h;

    tar = sin_deg(delta)/cos_deg(lat);
    if (tar < -1.0 || tar > 1.0) {
	printf("The object is circumpolar \n");
	exit (1);
    }
    *ar = acos_deg(tar);
    *as = 360.0 - *ar;

    h = acos_deg(-tan_deg(lat) * tan_deg(delta)) / 15.0;
    *lstr = 24.0 + alpha - h;
    if (*lstr > 24.0)
	*lstr -= 24.0;
    *lsts = alpha + h;
    if (*lsts > 24.0)
	*lsts -= 24.0;
}

/*
** Local sidereal time to local civil time.
** Uses globals: lon, tz, ?
**
*/
lst_to_lct(lst, jd, lct)
double lst;		/* local sidereal time in decimal hours	*/
double jd;		/* julian date				*/
double *lct;		/* return local civil time		*/
{
    double ed, gst, jzjd, t, r, b, t0, gmt;

    gst = lst + lon / 15.0;
    if (gst > 24.0)
	gst -= 24.0;
    jzjd = julian_date(1,0,yr);
    ed = jd-jzjd;
    t = (jzjd -2415020.0)/36525.0;
    r = 6.6460656+2400.05126*t+2.58E-05*t*t;
    b = 24-(r-24*(yr-1900));
    t0 = ed * 0.0657098 - b;
    if (t0 < 0.0)
	t0 += 24;
    gmt = gst-t0;
    if (gmt<0) 
	gmt += 24.0;
    gmt = gmt * 0.99727 - tz;;
    if (gmt < 0)
	gmt +=24.0;
    *lct = gmt;			/* fill in the result */
}

/* decimal hours to hours, minutes. Rounded to nearest minute */
dh_to_hm(dh, h, m)
double dh;
int *h, *m;
{
    double tempsec;

    *h = dh;
    *m = (dh - *h) * 60;
    tempsec = (dh - *h) * 60 - *m;
    tempsec = tempsec * 60 + 0.5;
    if (tempsec > 30)
	(*m)++;
    if (*m == 60) {
	*m = 0;
	(*h)++;
    }
}

/*
** Equatorial to horizon conversion
** sec #25 page 40
*/
eq_to_altaz(r, d, t, alt, az)
double r, d, t;			/* given: RA(hours), dec(deg), GST	*/
double *alt, *az;		/* ret: altitude, azimith	*/
{
    double p = (double)PI;
    double r1 = p / 180.0;
    double b = lat * r1;		/* b = rads north of observer	*/
    double l = (360 - lon) * r1;	/* l = rads east of observer	*/
    double t5, s1, c1, c2, s2, a, h;

    r = r * 15.0 * r1;
    d = d * r1;
    t = t * 15.0 * r1;
    t5 = t - r + l;	/* t5 = hour angle of obj from observer */
    s1 = sin(b) * sin(d) + cos(b) * cos(d) * cos(t5);	/*s1 = sin(alt) */
    c1 = 1 - s1 * s1;
    if (c1 > 0) {
	c1 = sqrt(c1);
	h = atan(s1 / c1);
    } else {
	h = (s1 / fabs(s1)) * (p / 2.0);
    }
    c2 = cos(b) * sin(d) - sin(b) * cos(d) * cos(t5);
    s2 = -cos(d) * sin(t5);
    if (c2 == 0) 
	a = (s2/fabs(s2)) * (p/2);
    else {
	a = atan(s2/c2);
	if (c2 < 0)
	    a=a+p;
    }
    if (a<0)
        a=a+2*p;
    *alt = h / r1;
    *az = a / r1;
}

double			/* return Greenwich sidereal time */
gmst(j, f)
double j;		/* given current julian day */
double	f;		/* and frac of day */
{
    double d, j0, t, t1, t2, s;

    d = j - 2451545.0;	/* since Jan 1.5 2000 */
    t = d / 36525.0;	/* t = years into this epoch */
    t1 = floor(t);
    j0 = t1 * 36525.0 + 2451545.0;
    t2 = (j - j0 + 0.5)/36525.0;
    s = 24110.54841 + 184.812866 * t1; 
    s += 8640184.812866 * t2;
    s += 0.093104 * t * t;
    s -= 0.0000062 * t * t * t;
    s /= 86400.0;
    s -= floor(s);
    s = 24 * (s + (f - 0.5) * 1.002737909);
    if (s < 0)
	s += 24.0;
    if (s > 24.0)
	s -= 24.0;
    return(s);
}
	
    
double 		/* angle in degrees */
acos_deg(x)
double x;
{
    return rtod(acos(x));
}

double  		/* angle in degrees */
asin_deg(x)
double x;
{
    return rtod(asin(x));
}

/*
**	Return (degrees)arctan of (y/x) correcting for quadrant
*/
double 
atan_q_deg(y,x)
double y,x;
{
    double rv;

    if (y == 0)			/* on x axis		*/
        rv = 0;
    else if (x == 0)		/* on y axis		*/
        rv = y>0 ? 90.0 : -90.0;
    else rv = atan_deg(y/x);

    if (x<0) return (rv + 180.0);	/* 2nd or 3rd quadrant	*/
    if (y<0) return (rv + 360.0);	/* 4th quadrant		*/
    return (rv);
}

double 		/* angle in degrees */
atan_deg(x)
double x;
{
    return rtod(atan(x));
}

double 
sin_deg(x)
double x; 		/* angle in degrees */
{
    return sin(dtor(x));
}

double 
cos_deg(x)
double x; 		/* angle in degrees */
{
    return cos(dtor(x));
}

double 
tan_deg(x)
double x; 		/* angle in degrees */
{
    return tan(dtor(x));
}


double
dtor(deg)
double deg;
{
return (deg * PI / 180.0);
}

double
rtod(deg)
double deg;
{
    return (deg * 180.0 / PI);
}


double 
adj360(deg)
double deg;
{
    while (deg < 0.0) 
	deg += 360.0;
    while (deg > 360.0)
	deg -= 360.0;
    return(deg);
}

double 
adj24(hrs)
double hrs;
{
    while (hrs < 0.0) 
	hrs += 24.0;
    while (hrs > 24.0)
	hrs -= 24.0;
    return(hrs);
}

double 
hms_to_dh(h, m, s)
{
    double rv;

    rv = h + m / 60.0 + s / 3600.0;
    return rv;
}

pnthms(dh)	/* print decimal hours in hours, min, sec */
double	dh;
{
	int h, m, s;

	if (dh < 0.0) {
		putchar('-');
		dh *= -1.0;	/* dont show rest as negitave */	
	}  else
		putchar(' ');	/* leave a space 	*/
	h = dh;
	dh = 60.0 * (dh - (double)h);
	m = dh;
	dh = 60.0 * (dh - (double)m);
	s = dh;
	printf("%02d:%02d:%02d", h, m, s);
}

pntdms(dd)	/* print decimal degrees in deg, min, sec */
double	dd;
{
	int	d,m,s;

	if (dd < 0.0) {
		putchar('-');
		dd *= -1.0;	/* dont show rest as negitave */
	} else
		putchar(' ');	/* leave a space	*/	
	d = dd;
	dd = 60.0 * (dd - (double)d);
	m = dd;
	dd = 60.0 * (dd - (double)m);
	s = dd;
	printf("%03d deg %02d\' %02d\'\'", d, m, s);

}


