/*
 *	starmap program:  for a description, see the usage function below, and
 *	the comment below it.
 */

#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <stdarg.h>

#define VERSION			"3.2"		/* program version number */

#define DEF_MAG_LIMIT		(5.0)		/* default magnitude cutoff */
#define MIN_MAG_LIMIT		(-2.0)		/* minimum magnitude cutoff */
#define MAX_MAG_LIMIT		(11.5)		/* maximum magnitude cutoff */
#define DEF_CEN_WID_DEG		(60.0)		/* 4 hours */
#define MAX_CEN_WID_DEG		(90.0)		/* 6 hours */
#define MIN_CEN_WID_DEG		(1.0)		/* 4 minutes */
#define DEF_ID_LIMIT		(4.5)		/* default mag limit for IDs */
#define MIN_ID_LIMIT		(-2.0)		/* default mag limit for IDs */
#define MAX_ID_LIMIT		(11.5)		/* default mag limit for IDs */
#define EPOCH			(2000)		/* for the RA and dec */

char *Prog;			/* program name */
void usage() { fprintf(stderr,

"starmap program, version %s\n"
"\n"
"%s -a additional_objects -d designations -g grid_level -i inhibits \\\n"
"        -l limiting_mag -m map -p precession_year -s label_mags -t title \\\n"
"        -v viewfields -z zenith -b -e -h -n -q -u -w -x\n"
"(all parameters are optional)\n"
"\n"
"Program to generate PostScript commands for printing a star map.  The map\n"
"can be centered on any point in the sky, and cover a specified field of\n"
"view.  Or, you can print the following six standard maps to cover the sky,\n"
"with some overlap:\n"
"        1 = north circumpolar\n"
"        2 = south circumpolar\n"
"        3 = midsection, right ascension 0 hours to beyond 7.5 hours\n"
"        4 = midsection, right ascension before 7.5 hours to 15 hours\n"
"        5 = midsection, right ascension 12 hours to beyond 19.5 hours\n"
"        6 = midsection, right ascension before 19.5 hours wrapped to 3 hours\n"
"The intent is that you would print maps 3 and 4 and splice them into\n"
"one wide map; and the same with 5 and 6.\n"
"\n"
"For the parameters below, whenever right ascension is stated, it can be in\n"
"any of these six formats:  H  Hh  HhM  HhMm  HhMmS  HhMmSs  where H is\n"
"hours, M is minutes, and S is seconds.  Whenever an angle is stated, for\n"
"declination or other things, it can be in any of these six formats:  D  Do\n"
"DoM  DoM'  DoM'S  DoM'S\"  where D is degrees, M is arcminutes, and S is\n"
"arcseconds.  For all twelve formats, only the last number given can have a\n"
"decimal point, and none can have a sign except for D.  All must be in the\n"
"appropriate range.  If D is negative, that sign applies to M and S also, if\n"
"present.\n"
"\n"
"The parameters that have operands are:\n"
"    -a: show additional objects.  The operand to -a is either \"file=\"\n"
"        followed by the name of a file containing a list of objects, one per\n"
"        line, or the objects can be listed on the command line, separated by\n"
"        colons (\":\").  Each object is given as 4 items:  right ascension,\n"
"        declination, magnitude, name.  If given in a file, the 4 items can\n"
"        be separated by either commas or spaces or tabs.  If given on the\n"
"        command line, they must be separated by commas.  The name is\n"
"        optional, and if omitted, the last separator may also be omitted.\n"
"    -d: designations to print for a star, whichever exist, as follows:\n"
"           b:   Bayer letter (the default choice)\n"
"           f:   Flamsteed number\n"
"           b-f: Bayer if it exists, otherwise Flamsteed\n"
"           b+f: Bayer preceded by Flamsteed\n"
"           n:   neither\n"
"    -g: level of coordinate grid lines to show, as follows:\n"
"           0: do not print (also inhibits the ecliptic and galactic equator)\n"
"           1: right ascension every hour, declination every 10 degrees\n"
"           2: right ascension every 30 minutes, declination every 5 degrees\n"
"           3: right ascension every 5 minutes, declination every degree\n"
"        The default is 1, except 2 for centered maps with screen width less\n"
"        than 60 degrees and 3 for centered maps of width less than 25 degrees.\n"
"        Near the poles, not as many right ascension lines are shown.\n"
"    -i: inhibit printing of each item in this operand.  The letters in the\n"
"        operand may be in any order, and are as follows:\n"
"           b: constellation boundaries\n"
"           n: constellation names\n"
"           o: constellation outlines\n"
"           s: star names and designations\n"
"    -l: limiting magnitude (faintest stars to print): default %.1f,\n"
"        minimum %.1f, maximum %.1f.  Star names etc. are placed to avoid\n"
"        colliding with stars down to %.1f, but fainter than that there will\n"
"        be collisions.\n"
"    -m: which map to print (default 1).  This can be a number from 1 to 6,\n"
"        for one of the standard maps; or it can be comma separated items\n"
"        specifying a map centered on some point in the sky.  In the latter\n"
"        case, the items can be right ascension, declination, and optionally\n"
"        the width to be shown as an angle from %.0f to %.0f degrees defaulting\n"
"        to %.0f; or they can be an ID and optionally the width.\n"
"        An ID can be a constellation, or an object given by its proper name\n"
"        or by its Bayer or Flamsteed designation.\n"
"        A Bayer designation is a letter, followed by a digit if needed,\n"
"        followed by a constellation.  The letter can be the name of a Greek\n"
"        letter spelled out, or a single Roman letter that represents a Greek\n"
"        letter; a for alpha, b for beta, etc.  The whole list of Roman\n"
"        letters for the Greek alphabet in Greek alphabetical order is:\n"
"           a b g d e z h q i k l m n x o p r s t u f c y w.\n"
"        A Flamsteed designation is a number followed by a constellation.\n"
"        In any of these contexts, for constellation you can use the standard\n"
"        name, or the possessive form, or the official three letter\n"
"        abbreviation (like Cetus, Ceti, Cet).  In any of the forms of ID,\n"
"        blanks and case of letters are ignored.\n"
"    -p: year to precess everything to, default %d.\n"
"    -s: limiting magnitudes for showing object names, Bayer letters, and\n"
"        Flamsteed numbers: default %.1f, minimum %.1f, maximum %.1f.  One or\n"
"        two numbers (comma separated) may be given.  If one number, it applies\n"
"        to all of these IDs.  If two, the first applies to Bayer and Flamsteed\n"
"        and the second applies to object names.  Object names for objects from\n"
"        the -a option are printed regardless of magnitude.  For standard maps,\n"
"        some names are never printed.  For centered maps, some names may or\n"
"        may not be printed, based on the screen width, to avoid collisions.\n"
"    -t: title to be printed above the map.  If it is multiple lines, end each\n"
"        line but the last with \\n.  The font size defaults to 14.  For a\n"
"        different size, begin the string with s(X) where X is the size.\n"
"    -v: view field circles to draw.  This is for centered maps only.  The\n"
"        items are colon separated.  Each item starts with an angle, which is\n"
"        the diameter of the view field, optionally followed (comma separated)\n"
"        by the right ascension and declination of where the view field is to\n"
"        be centered, or an ID in the same format as for the -m option.  If\n"
"        right ascension/declination and ID are not given, the view field will\n"
"        be centered where the map is centered.\n"
"    -z: right ascension and declination of the zenith; the horizon is drawn\n"
"        based on this.  They are comma separated.  This is for centered\n"
"        maps only.\n"
"\n"
"The parameters without operands are:\n"
"    -b: use black and white only in drawing the map.\n"
"    -e: print page edges (your paper should extend at least to there).\n"
"    -h: print this help message and exit.\n"
"    -n: show star number where the proper name would normally be.\n"
"    -q: quantize magnitudes to the nearest integer.\n"
"    -u: show magnitude times 10 where the proper name would normally be.\n"
"    -w: print on white background.\n"
"    -x: draw an X at the point the map is centered on (centered maps only).\n"
, VERSION, Prog, DEF_MAG_LIMIT, MIN_MAG_LIMIT, MAX_MAG_LIMIT, DEF_MAG_LIMIT,
MIN_CEN_WID_DEG, MAX_CEN_WID_DEG, DEF_CEN_WID_DEG,
EPOCH, DEF_ID_LIMIT, MIN_ID_LIMIT, MAX_ID_LIMIT
); exit(1); }
/*
 *
 *	The program reads six files, as follows.  It is not very defensive
 *	against formatting that violates what is stated below.
 *
 *	stardata
 *	This started as yale.star, taken from the starchart-3.2.2 program off
 *	the web and reformated for convenience.  The RA is given to the nearest
 *	second, and declination to the nearest arcminute.  But I compared it to
 *	catalog.dat of "Catalogue of the Brightest Stars (Ochsenbein+ 1988)".
 *	Mostly this just added additional info, like names or Bayer letters,
 *	but yale.star was mistakenly using "x", which is xi, where it should
 *	have used "c", chi.  A few other Bayer letters (actually numbers
 *	suffixed to letters) had to be changed.  A few magnitudes disagreed,
 *	typically variable or multiple stars.  I used Wil Tirion's atlas as
 *	the final authority.  I also added some proper names from other
 *	sources.  I also set a few stars to mag 99 because they were doubles
 *	landing on top of and obliterating the other stars; and I changed Mira.
 *	Then, I got the Tycho-2 data from the Hipparcos satellite.  Since it
 *	is not complete and the magnitudes not reliable much beyond magnitude
 *	11.5, I cut it off at that point.  The Tycho data doesn't include any
 *	names, so I needed to merge it with my previous data.  I compared the
 *	positions, matching stars that were within 1.5 arcminutes.  Some of the
 *	old stars matched multiple of the Tycho ones, probably due to double
 *	and multiple stars.  I kept the old magnitudes, since the new data
 *	seems to have some strange things, like it has Sirius at -1.088
 *	instead of the well known -1.46.  Next, I found the web site
 *	http://en.wikipedia.org/wiki/Category:Lists_of_stars_by_constellation.
 *	I matched all those stars, down to magnitude 7, to my existing data,
 *	considering it a match when stars were less than 1.5 arcminutes apart
 *	and differing by no more than one magnitude.  If a star in my original
 *	date matched zero or multiple stars in this new data, I didn't change
 *	it.  Otherwise, I changed its RA and declination, since the new data
 *	should be more accurate.  And I compared the Bayer letters.  When they
 *	differed, I went with whatever Wil Tirion's atlas said, except in
 *	a few cases where there were bright stars like a1 and a2 very close
 *	together, and both would be printed by default and there wouldn't be
 *	room (so in this example I would use just "a" on one of them).  And
 *	then, whenever Flamsteed was 0 in the old data, I put in the new data,
 *	if there was any.  The stars are in order by magnitude, except for the
 *	few I altered above (mag 99, and Mira, and Dschubba, which I altered
 *	because in recent years it has become brighter).  Next, I altered and
 *	added some proper names according to "IAU Catalog of Star Names
 *	(IAU-CSN)".
 *	Format: whitespace separated columns:
 *		right ascension in decimal hours
 *		declination in decimal degrees
 *		magnitude
 *	(plus the following down to mag 8.21 only)
 *		constellation abbreviation
 *		Bayer letter, "-" if none or unknown
 *		Flamsteed number, 0 if none or unknown
 *		"=", followed by proper name if any
 *
 *	boundary
 *	This came from "Constellation Boundary Data (Davenhall+ 1989)".
 *	Format: columns separate by whitespace, each line being a point on the
 *	boundary of some constellation.  All of a constellation's points are to
 *	be joined, in order.
 *		right ascension in decimal hours
 *		declination in decimal degrees
 *		constellation abbreviation
 *		"O" if original point, precessed from 1875, when everything
 *			aligned with the RA/dec coord system and those were the
 *			only points you needed; "I" for in between points added
 *			each degree, so everything could be precessed to 2000
 *			and still have almost exactly correct boundaries
 *
 *	connames
 *	This came from constlabel.dat from the Cartes du Ciel program, but
 *	altered by me.  I needed to allow for different positioning of the
 *	names depending on which map is being printed.
 *	Format: tab separated columns:
 *		right ascension in decimal hours for map 1 or 2
 *		declination in decimal degrees for map 1 or 2
 *		right ascension in decimal hours for map 3 or 4
 *		declination in decimal degrees for map 3 or 4
 *		right ascension in decimal hours for map 5 or 6
 *		declination in decimal degrees for map 5 or 6
 *		"=" followed by name.  For two word names, this may be followed
 *			by "/" and number(s).  If number X is there, it means
 *			the words are to be printed stacked up on map 2X+1 and
 *			2X+2.
 *		(Centered maps use one of the above three cases, depending on
 *			the location of each constellation.)
 *
 *	conlines
 *	I wrote this.  Each nonblank line gives the line numbers in stardata of
 *	two stars that are to be joined by a line, to show the outline of a
 *	constellation.
 *	Format: tab separated columns:
 *		first star
 *		second star
 *		for the first star of a constellation, the constellation name
 *
 *	bayerpos
 *	I wrote this.  Normally, I put proper star names to the right of the
 *	star.  I put Bayer letters to the right unless there is a proper name,
 *	in which case I put Bayer on the left.  If Bayer needs to be somewhere
 *	else, the bayerpos file gives the x and y point offsets from the
 *	center of the star where the letter should start.  There are pairs of
 *	offsets for map 1/2, 3/4, and 5/6.  Offsets of "0 0" mean don't do
 *	this, use the default positioning.  This file matches stardata line
 *	for line.
 *	Format: tab separated columns:
 *		x offset for map 1 or 2
 *		y offset for map 1 or 2
 *		x offset for map 3 or 4
 *		y offset for map 3 or 4
 *		x offset for map 5 or 6
 *		y offset for map 5 or 6
 *	Centered maps use one of these, depending on the location of each star.
 *
 *	proppos
 *	This is just like bayerpos but for proper names.
 */

/*
 * Versions:
 *
 * 1.0	Original program
 * 2.0	Added star data from Tycho-2
 *	Added centered maps
 *	Several other additions
 *	Cleaned up the code and comments
 * 2.1	Added the -v option
 * 2.2	Added the -t and -z options
 *	Stopped printing the whole usage message when bad options, added -h
 *	Fixed Pleiades name which sometimes was missing
 *	Fixed galactic equator for centered maps only a few degrees wide
 *	Enhanced -v to allow multiple viewfield circles
 *	Other minor cleanup
 * 2.3	Increased a buffer to allow more additional objects
 *	Reduced intensity of constellation outlines
 *	From Wikipedia added more Bayer and Flamsteed info, adjusted RA and dec
 *	Replaced -f option with -d option
 *	Improved error messages to say what the correct input is
 * 3.0	Expanded -m option to allow IDs instead of RA and dec
 *	Fixed bug where constellation names sometimes didn't print
 *	Replaced -b option (black background) with -w (white background)
 *	Replaced -c option (use color) with -b (black and white only)
 *	Changed -y option to -s
 *	Enhance -a option to allow giving a file
 *	Other minor changes
 * 3.1	Fix default in option switch
 *	Add alternate genitive forms of some constellation names
 *	Revise star names according to the "IAU Catalog of Star Names (IAU-CSN)"
 *	  (see http://www.pas.rochester.edu/~emamajek/WGSN/IAU-CSN.txt) and add
 *	  the other star names from there
 *	Enhance -s option to allow a separate number for proper names
 *	For -a option objects, print proper name regardless of magnitude
 *	Allow inhibiting proper names by stating screen width in stardata
 *	Alter constellation lines for Monoceros
 * 3.2	Alter constellation lines for Virgo
 *	Enhance -v to allow view fields to be centered at given places
 */

#ifndef PI
#define PI	3.14159265358979323846
#endif

/*
 * Limits of the PostScript coords that my printer can print.  Hopefully, if
 * your printer differs, you can just change these numbers and all the later
 * definitions based on these will adjust appropriately such that everything
 * will still print well.
 */
#define MIN_X	22.0
#define MAX_X	572.0
#define MIN_Y	40.0
#define MAX_Y	805.0

#define MARGIN	1500.0

/* the midpoints of the printable page */
#define MID_X	(((MIN_X) + (MAX_X)) / 2.0)
#define MID_Y	(((MIN_Y) + (MAX_Y)) / 2.0)

/* the width in points allowed for printing coordinates on the side of maps */
#define COORD_PTS	24.0

/*
 * The height in points of the area in which we print maps 3 to 6 and
 * centered maps: we don't use quite the whole printable area.
 */
#define MID_GRID_HEIGHT	668.0

/*
 * The width in points in which we print maps 3 to 6, not considering the RA
 * numbers or the fact that we actually allow it to flow into the other margin
 * to allow easier splicing of maps 3 & 4, 5 & 6.  The official "edge" of the
 * map (as printed by -e) includes the RA numbers but not the overflow.
 */
#define MID_GRID_WIDTH	((MAX_X) - (MIN_X) - (COORD_PTS))

/* find number of elements in an array */
#define NUMELEM(array)	(sizeof (array) / sizeof (array[0]))

/* define bool symbols */
typedef enum {
	false = 0,
	true = 1
} bool;

/*
 * These are used only for maps 3 to 6 and centered maps.  The x values do not
 * include coordinate numbers or the overflow of maps 3 to 6.  The y values do
 * not include coordinate numbers.
 */
float Min_x;
float Max_x;
float Min_y;
float Max_y;

/* conversions between radians, degrees, and hours */
#define RAD2DEG(x)	((x) * 180.0 / PI)
#define DEG2RAD(x)	((x) * PI / 180.0)
#define DEG2HOUR(x)	((x) / 15.0)
#define HOUR2DEG(x)	((x) * 15.0)
#define RAD2HOUR(x)	(DEG2HOUR(RAD2DEG(x)))
#define HOUR2RAD(x)	(DEG2RAD(HOUR2DEG(x)))

/* for maps 3 to 6, convert declination degrees to points */
#define DEC_DEG2Y(dec_deg)	(MID_Y + (dec_deg) * MID_GRID_HEIGHT / 130.0)

/* for maps 1 and 2, convert declination degrees to points from the center */
#define DEC_DEG2R(deg_deg)	((90.0 - fabs((float)(deg_deg))) * 4.8)

/* RA hours shown on each map 3 to 6 */
#define HOURSPERPAGE	7.5

/* assumes Ra_offset is set for a map 3 to 6, converts RA hours to x position */
#define RA_HOUR2X(ra)	(Max_x - ((ra) - Ra_offset) * (MID_GRID_WIDTH) \
			/ HOURSPERPAGE)

/* sine and cosine of the ecliptic's obliquity */
float Cos_epsilon, Sin_epsilon;

/* for maps 3 to 6, minimum right ascension in hours */
float Ra_offset;

/* if mag cutoff fainter than this, scale star radii to avoid too small */
#define MAG_SCALE_LIMIT		(DEF_MAG_LIMIT + 0.5)

#define MAX_PROPNAME	20	/* max length of a star's proper name */
#define MAX_M_OPT	100	/* max length of the -m option */
#define MAX_V_OPT	100	/* max length of the -v option */

/* the files we will read to get all the input data */
#define STARDATA_FILE 		"stardata"
#define BOUNDARY_FILE		"boundary"
#define CONST_NAMES_FILE	"connames"
#define CONST_LINES_FILE	"conlines"
#define BAYER_POS_FILE		"bayerpos"
#define PROPER_POS_FILE		"proppos"

/* PostScript gray scale number for various kinds of lines, etc. */
#define GRAY_GRID	(0.45)
#define	GRAY_BOUND	(0.75)
#define GRAY_CLOUD	(0.90)
#define GRAY_OUTLINE	(0.30)

/* distance in points from center of star where the printing should begin */
struct offset {
	float xoff;
	float yoff;
};

/* return values of center_xy() */
enum CENTERPOS {
	CP_ONMAP,		/* x,y is in the map area */
	CP_OFFMAP,		/* x,y is off the map area */
	CP_NOTSET		/* x,y is not set (wrong hemisphere) */
};

/* populated from stardata file and later calculations */
struct stardata {
	float ra_hour;			/* right ascension in hours */
	float dec_deg;			/* declination in degrees */
	float mag;			/* magnitude */
	float radius;			/* in points */
	float x, y;			/* position in points */
	enum CENTERPOS centerpos;	/* for centered map, where are x and y*/
	char bayer[3];			/* room for letter, number, \0 */
	char flamsteed[4];		/* flamsteed number, \0 */
	char const_abbrev[4];		/* constellation abbrev, all caps */
	char propname[MAX_PROPNAME + 1];/* proper name */
	float width_limit;		/* threshold for printing proper name */
	/* the following are indexed by map index, and are offsets from the star
	 * in points */
	struct offset bayerpos[3];
	struct offset properpos[3];
};

/* which star designations to print */
enum DESIGNATION {
	DES_BAYER,		/* Bayer only */
	DES_FLAM,		/* Flamsteed only */
	DES_BAYER_OR_FLAM,	/* Bayer if it exists, else Flamsteed */
	DES_BAYER_AND_FLAM,	/* Bayer preceded by Flamsteed */
	DES_NEITHER		/* neither */
};

/* index by the line number of the stardata file (so 0 is not used) */
#define MAX_OBJECTS	1392000
struct stardata Stardata[MAX_OBJECTS];
int Num_objects;		/* number of objects in Stardata */
int Num_stars;			/* number of real stars in Stardata */
int Last_bayer_idx;		/* index to the last star that has bayer */
int Last_flamsteed_idx;		/* index to the last star that has flamsteed */
int Last_proper_idx;		/* index to the last star that has propername */

/* flags defaulted here, can be changed by command line options */
bool Quantize = false;		/* quantize star magnitudes to integers? */
float Precess_year = EPOCH;	/* year to precess to */
float Mag_limit = DEF_MAG_LIMIT;/* limiting mag for printing stars */
float Bf_limit = DEF_ID_LIMIT;	/* limiting mag for printing bayer/flamsteed */
float Prop_limit = DEF_ID_LIMIT;/* limiting mag for printing proper names */
bool Designation = DES_BAYER;	/* print Bayer only */
bool Print_star_namelet = true;	/* print star names and bayer/flamsteed */
bool Print_mag = false;		/* don't show mag * 10 instead of proper name */
bool Print_starnum = false;	/* don't show star num instead of proper name */
bool White_background = false;	/* don't print on a white background */
bool Black_and_white_map = false; /* don't use black and white (use color) */
int Grid_level = 0;		/* how finely to print coord grids (default) */

/* reverse gray value if black background is set */
#define GETGRAY(x)	(White_background ? (x) : 1 - (x))

/* map 1 and 2 give index 0, 3 and 4 give 1, 5 and 6 give 2 */
#define MIDX 	((Map - 1) / 2)

int Map = 1;			/* map number 1 to 6, or 0 for centered */

/* info for centered maps */
float Cen_ra_hour;		/* center point right ascension in hours */
float Cen_dec_deg;		/* center point declination in degrees */
float Cen_wid_deg;		/* width of map in degrees */
float Zenith_ra_hour;		/* zenith right ascension in hours */
float Zenith_dec_deg;		/* zenith declination in degrees */

/* the places where each view field is centered, the centers, and how many */
float Vf_ra_hour[MAX_V_OPT + 1];
float Vf_dec_deg[MAX_V_OPT + 1];
float Vf_diam_deg[MAX_V_OPT + 1];
int Num_view_fields;

#define SCALE			(Cen_wid_deg / DEF_CEN_WID_DEG)

/* convenient macros for maps */
#define NORTH		(Map == 1)
#define SOUTH		(Map == 2)
#define CIRCUMPOLAR	(NORTH || SOUTH)
#define MIDSECTION	(Map >= 3)
#define CENTER		(Map == 0)

/* PostScript width of a thin line */
#define THIN		0.2


/* function templates */

void get_centered_args(char *args);
int get_loc_from_id(char *obj, float *ra_hour, float *dec_deg);
void compress_lower(char *out_str, char *in_str);
char *get_const_abbrev(char *const_string);
char get_roman_letter(char *greek_name, int *len_ptr);
int get_bayer(char *lowerobj, bool single_letter, float *ra_hour,
		float *dec_deg);
void load_stardata(char *addobj);
void get_additional_obj(char *args, float *ra_hour_ptr, float *dec_deg_ptr,
		float *mag_ptr, char *name_ptr);
void calc_stardata(void);
void parse_viewfields(char *args);
void get_zenith(char *args);
void set_star_coords(int sidx);
void output_prolog(void);
void output_postlog(void);
void print_grid(int call);
void print_grid_mid(void);
void print_grid_polar(void);
void print_grid_center(int call);
void print_grid_center_raline(int call, float ra_hour, float declim, int level);
void print_grid_center_decline(int call, float dec_deg, int level);
void set_grid_appearance(int level);
void print_grid_numbers(void);
void print_grid_numbers_mid(void);
void print_grid_numbers_polar(void);
void print_ecliptic(void);
void print_ecliptic_mid(void);
void print_ecliptic_center(void);
void print_galactic(void);
void print_galactic_piece(float rastart_deg, int horz, int vert);
void print_stars(void);
void print_star_polar(int sidx);
void print_star_mid(int sidx);
void print_star_center(int sidx);
void print_star_itself(int sidx, float xpos, float ypos);
void print_star_bayer_flam(int sidx, float star_x, float star_y);
void print_star_proper(int sidx, float star_x, float star_y);
char *proper_print(int sidx);
void print_boundaries(void);
void print_boundaries_polar(FILE *bd_p);
void print_boundaries_mid(FILE *bd_p);
void print_boundaries_center(FILE *bd_p);
void print_const_outlines(void);
void print_const_names(void);
void print_const_names_polar(float ra_hour, float dec_deg, char *con,
		bool split);
void print_const_names_mid(float ra_hour, float dec_deg, char *con, bool split);
void print_const_names_center(float ra_hour, float dec_deg, char *con,
		bool split);
void print_const_names_common(char *con, bool split, float x, float y);
void print_misc(int call);
void print_misc_polar(int call);
void print_misc_mid(int call);
void print_misc_center(int call);
void print_title(char *title);
void print_view_fields(void);
void print_one_view_field(int vf_num);
void print_horizon(void);
void whiteout(void);
void whiteout_polar(void);
void whiteout_mid(void);
void whiteout_center(void);
float star_radius(float mag);
int star_midx(int sidx);
int best_midx(float ra_hour, float dec_deg);
float quantize(float f);
bool cross_left(float x1, float y1, float x2, float y2, float *y_ptr);
bool cross_right(float x1, float y1, float x2, float y2, float *y_ptr);
bool cross_top(float x1, float y1, float x2, float y2, float *x_ptr);
bool cross_bottom(float x1, float y1, float x2, float y2, float *x_ptr);
void ra_float2hourmin(float ra_hour, int *hour_ptr, int *min_ptr);
void print_ra_left(float x1, float y1, float x2, float y2, float ra_hour);
void print_ra_top(float x1, float y1, float x2, float y2, float ra_hour);
void print_ra_bottom(float x1, float y1, float x2, float y2, float ra_hour);
void print_dec_right(float x1, float y1, float x2, float y2, float dec_deg);
void print_dec_top(float x1, float y1, float x2, float y2, float dec_deg);
void print_dec_bottom(float x1, float y1, float x2, float y2, float dec_deg);
void precess(float *ra_hr_ptr, float *dec_deg_ptr);
enum CENTERPOS center_xy(float ra_hr, float dec_deg, float *out_x_ptr,
		float *out_y_ptr);
int hms_str2float(char *str, float *ans_ptr);
int dms_str2float(char *str, float *ans_ptr);
void error(char *format, ...);



/*
 * Function:	main
 *
 * Description:	This function processes the command line arguments and calls
 *		subroutines to load the data and generate the PostScript.
 *
 * Returns:	0 if success, 1 if error
 */

int
main(int argc, char *argv[])
{
	int c;		/* an option character */
	int idx;	/* loop index */
	char *end_ptr;	/* char after a number */
	char *end2_ptr;	/* another char after a number */

	/*
	 * Define and set default values for command line options that don't
	 * set global variables.
	 */
	bool do_const_boundaries = true;
	bool do_const_names = true;
	bool do_grid = true;
	bool do_const_outlines = true;
	bool do_edges = false;
	bool draw_x = false;
	char *centered_args = NULL;
	char *addobj = NULL;
	char *title = NULL;
	char *viewfields = NULL;
	char *zenith = NULL;

	bool got[256];		/* guard against multiple usages of options */
	bool got_i[256];	/* guard against mult usages of -i suboptions */

	float epsilon_rad;	/* obliquity of the ecliptic */


	Prog = argv[0];

	/* init that no options have been parsed yet */
	for (idx = 0; idx < 256; idx++) {
		got[idx] = false;
		got_i[idx] = false;
	}

	/*
	 * Loop through the options, storing that information.  Make sure none
	 * are used multiple times.
	 */
	while ((c = getopt(argc, argv, "a:d:g:i:l:m:p:s:t:v:z:behnquwx"))
				!= -1) {

		if (got[c]) {
			error("option -%c is used multiple times", c);
		}
		got[c] = true;

		switch (c) {

		/********** options with arguments **********/
		case 'a':
			addobj = optarg;
			break;
		case 'd':
			if (strcmp(optarg, "b") == 0) {
				Designation = DES_BAYER;
			} else if (strcmp(optarg, "f") == 0) {
				Designation = DES_FLAM;
			} else if (strcmp(optarg, "b-f") == 0) {
				Designation = DES_BAYER_OR_FLAM;
			} else if (strcmp(optarg, "b+f") == 0) {
				Designation = DES_BAYER_AND_FLAM;
			} else if (strcmp(optarg, "n") == 0) {
				Designation = DES_NEITHER;
			} else {
				error("-d option has invalid value; must be "
					"one of these:  b  f  b-f  b+f  n");
			}
			break;
		case 'g':
			if (strlen(optarg) != 1 || optarg[0] < '0' ||
					optarg[0] > '3') {
				error("argument of option 'g' must be a number 0 to 3");
			}
			if (optarg[0] == '0') {
				do_grid = false;
			} else {
				Grid_level = optarg[0] - '0';
			}
			break;
		case 'i':
			/* loop through the suboptions of -i */
			for (idx = 0; optarg[idx] != '\0'; idx++) {
				if (got_i[optarg[idx]]) {
					error("suboption '%c' in option -i is used multiple times",
							optarg[idx]);
				}
				got_i[optarg[idx]] = true;

				switch (optarg[idx]) {
				case 'b':
					do_const_boundaries = false;
					break;
				case 'n':
					do_const_names = false;
					break;
				case 'o':
					do_const_outlines = false;
					break;
				case 's':
					Print_star_namelet = false;
					break;
				default:
					error("invalid -i suboption '%c'",
							optarg[idx]);
					break;
				}
			}
			break;
		case 'l':
			errno = 0;
			Mag_limit = strtof(optarg, &end_ptr);
			if (errno != 0 || isspace(optarg[0]) ||
					end_ptr == optarg || *end_ptr != '\0') {
				error("invalid number in -l option");
			}
			if (Mag_limit < MIN_MAG_LIMIT ||
					Mag_limit > MAX_MAG_LIMIT) {
				error("-l option is outside of the allowed range %.1f to %.1f",
					MIN_MAG_LIMIT, MAX_MAG_LIMIT);
			}
			break;
		case 'm':
			if (optarg[0] >= '1' && optarg[0] <= '6' &&
					optarg[1] == '\0') {
				/* one of the standard six maps */
				Map = atoi(optarg);
				break;
			}
			/* assume a custom (centered) map */
			Map = 0;	/* mark centered map */
			centered_args = optarg;
			break;
		case 'p':
			errno = 0;
			Precess_year = strtof(optarg, &end_ptr);
			if (errno != 0 || isspace(optarg[0]) ||
					end_ptr == optarg || *end_ptr != '\0') {
				error("invalid number in -p option");
			}
			break;
		case 's':
			/* get first number for Id_limit */
			errno = 0;
			Bf_limit = strtof(optarg, &end_ptr);
			if (errno != 0 || isspace(optarg[0]) ||
					end_ptr == optarg ||
					*end_ptr != '\0' && *end_ptr != ',') {
				error("invalid format of -s option");
			}
			if (Bf_limit < MIN_ID_LIMIT ||
					Bf_limit > MAX_ID_LIMIT) {
				error("number in -s option is outside of the "
				"allowed range %.1f to %.1f",
				MIN_ID_LIMIT, MAX_ID_LIMIT);
			}
			if (*end_ptr == '\0') {
				/* only one number given, use it for both */
				Prop_limit = Bf_limit;
			} else {
				/* get second number for Prop_limit */
				Prop_limit = strtof(&end_ptr[1], &end2_ptr);
				if (errno != 0 || isspace(end_ptr[1]) ||
						*end2_ptr != '\0') {
					error("invalid format of -s option");
				}
				if (Prop_limit < MIN_ID_LIMIT ||
						Prop_limit > MAX_ID_LIMIT) {
					error("number in -s option is outside "
					"of the allowed range %.1f to %.1f",
					MIN_ID_LIMIT, MAX_ID_LIMIT);
				}
			}
			break;
		case 't':
			title = optarg;
			break;
		case 'v':
			viewfields = optarg;
			break;
		case 'z':
			zenith = optarg;
			break;

		/********** options without arguments **********/
		case 'b':
			Black_and_white_map = true;
			break;
		case 'e':
			do_edges = true;
			break;
		case 'h':
			usage();
		case 'n':
			Print_starnum = true;
			break;
		case 'q':
			Quantize = true;
			break;
		case 'u':
			Print_mag = true;
			break;
		case 'w':
			White_background = true;
			break;
		case 'x':
			draw_x = true;
			break;
		default:
			error("run with -h to see the list of options allowed");
		}
	}

	/* only options are allowed, no file name arguments */
	if (optind != argc) {
		error("invalid command line; run with -h to see the correct usage");
	}

	if (viewfields != NULL && ! CENTER) {
		error("option -v is allowed only for centered maps");
	}

	if (draw_x && ! CENTER) {
		error("option -x is allowed only for centered maps");
	}

	if (zenith != NULL && ! CENTER) {
		error("option -z is allowed only for centered maps");
	}

	if (Print_mag && Print_starnum) {
		error("options -n and -u are incompatible");
	}

	if (zenith != NULL) {
		get_zenith(zenith);
	}

	/* set cosine and sine of ecliptic obliquity, 23 degrees 26 minutes */
	epsilon_rad = DEG2RAD(23.0 + 26.0 / 60.0);
	Cos_epsilon = cos(epsilon_rad);
	Sin_epsilon = sin(epsilon_rad);

	/*
	 * For non-circumpolar maps we need to set up some globals.
	 */
	if ( ! CIRCUMPOLAR ) {
		Min_y = MID_Y - MID_GRID_HEIGHT / 2.0;
		Max_y = MID_Y + MID_GRID_HEIGHT / 2.0;
	}

	switch (Map) {
	case 0:
		/* centered map: allow for coords on left and right */
		Min_x = MIN_X + COORD_PTS;
		Max_x = MAX_X - COORD_PTS;
		break;
	case 1:
		break;
	case 2:
		break;
	case 3:
		/* allow for declination on right */
		Min_x = MIN_X;
		Max_x = MAX_X - COORD_PTS;
		Ra_offset = 0.0;
		break;
	case 4:
		/* allow for declination on left */
		Min_x = MIN_X + COORD_PTS;
		Max_x = MAX_X;
		Ra_offset = 7.5;
		break;
	case 5:
		/* allow for declination on right */
		Min_x = MIN_X;
		Max_x = MAX_X - COORD_PTS;
		Ra_offset = 12.0;
		break;
	case 6:
		/* allow for declination on left */
		Min_x = MIN_X + COORD_PTS;
		Max_x = MAX_X;
		Ra_offset = 19.5;
		break;
	default:
		error("bug, missing case for map %d", Map);
	}


	/* populate Stardata with info about stars and additional objects */
	load_stardata(addobj);

	/* for centered maps, we can now parse the -m option */
	if (centered_args != NULL) {
		get_centered_args(centered_args);
	}

	/* now we can complete the populating of Stardata */
	calc_stardata();

	/* had to be done after parsing the -m option */
	if (viewfields != NULL) {
		parse_viewfields(viewfields);
	}

	/* if -g was not used, set the default Grid_level */
	if (CENTER && Grid_level == 0) {
		if (SCALE < 10.0 / 24.0) {
			Grid_level = 3;
		} else if (SCALE < 1.0) {
			Grid_level = 2;
		} else {
			Grid_level = 1;
		}
	}

	/*
	 * Call the subroutines for drawing things.  The order is significant,
	 * because some subroutines print white and may wipe out something
	 * previously printed.  (Especially "whiteout".)
	 */
	output_prolog();

	print_misc(1);

	if (do_grid) {
		printf("%.2f setgray\n", GETGRAY(GRAY_GRID));
		print_grid(1);		/* all maps: print the coord grid */
		print_ecliptic();
		print_galactic();
		printf("%.2f setgray\n", GETGRAY(0.0));
	}

	if (do_const_boundaries) {
		print_boundaries();
	}

	if (do_const_outlines) {
		print_const_outlines();
	}

	if (do_const_names) {
		print_const_names();
	}

	print_stars();

	print_misc(2);

	if (viewfields != NULL) {
		print_view_fields();
	}

	if (zenith != NULL) {
		print_horizon();
	}

	whiteout();

	if (do_grid) {
		if ( CENTER ) {
			print_grid(2);	/* print grid coord numbers */
		} else {
			print_grid_numbers();
		}
	}

	/* if requested (-e), print edges of entire printable area */
	if (do_edges) {
		printf("1 setlinewidth\n");
		printf("newpath\n");
		printf("%.2f %.2f moveto\n", MIN_X, MIN_Y);
		printf("%.2f %.2f lineto\n", MAX_X, MIN_Y);
		printf("%.2f %.2f lineto\n", MAX_X, MAX_Y);
		printf("%.2f %.2f lineto\n", MIN_X, MAX_Y);
		printf("closepath stroke\n");
	}

	/* if requested, or always for center maps, print inner edges */
	if (CENTER || do_edges) {
		printf("1 setlinewidth\n");
		printf("newpath\n");
		printf("%.2f %.2f moveto\n", Min_x, Min_y);
		printf("%.2f %.2f lineto\n", Max_x, Min_y);
		printf("%.2f %.2f lineto\n", Max_x, Max_y);
		printf("%.2f %.2f lineto\n", Min_x, Max_y);
		printf("closepath stroke\n");
	}

	if (draw_x) {
		printf("1 setlinewidth\n");
		printf("newpath\n");
		printf("%.2f %.2f moveto\n", MID_X - 10, MID_Y - 10);
		printf("%.2f %.2f lineto\n", MID_X + 10, MID_Y + 10);
		printf("closepath stroke\n");
		printf("newpath\n");
		printf("%.2f %.2f moveto\n", MID_X - 10, MID_Y + 10);
		printf("%.2f %.2f lineto\n", MID_X + 10, MID_Y - 10);
		printf("closepath stroke\n");
	}

	if (title != NULL) {
		print_title(title);
	}

	output_postlog();

	return 0;
}



/*
 * Function:	get_centered_args
 *
 * Description:	This function parses the argument of -m when it is for a
 *		centered map.  The fields are right ascension and declination,
 *		or an ID; and (optionally) desired screen width; all comma
 *		separated.  It sets the globals Cen_ra_hour, Cen_dec_deg, and
 *		Cen_wid_deg.
 *
 * Returns:	void (does not return if error)
 */

void
get_centered_args(

char *args		/* the string that is the arg of -m */
)
{
	char buf[MAX_M_OPT + 1];/* copy of args, to avoid overwriting "args" */

	/* places for each field plus a bogus one to catch extra junk */
	char f1[MAX_M_OPT + 1];
	char f2[MAX_M_OPT + 1];
	char f3[MAX_M_OPT + 1];
	char f4[MAX_M_OPT + 1];

	int idx;		/* loop through args and buf */
	int idx2;		/* where to store translated character */
	int fields;		/* number of fields we found */


	if (strlen(args) >= sizeof (buf)) {
		error("argument to -m is too long; limit is %d characters",
			sizeof (buf) - 1);
	}

	/*
	 * Copy, removing blanks that are there already, and converting commas
	 * to blanks, so we can use sscanf.  Removing the original blanks also
	 * allows the user to use blanks in an object name.
	 */
	idx2 = 0;
	for (idx = 0; args[idx] != '\0'; idx++) {
		if (args[idx] == ',') {
			buf[idx2] = ' ';
			idx2++;
		} else if ( ! isspace(args[idx]) ) {
			buf[idx2] = args[idx];
			idx2++;
		}
	}
	buf[idx2] = '\0';

	/* get as many fields as we can */
	fields = sscanf(buf, "%s%s%s%s", f1, f2, f3, f4);

	switch (fields) {

	case 1:
		/* only one field, so it has to be an id */
		if (get_loc_from_id(f1, &Cen_ra_hour, &Cen_dec_deg) != 0) {
			error("argument to -m is incorrect; when only one "
			"field is given, it must be a number 1 to 6, or a "
			"proper name, or a Bayer or Flamsteed designation "
			"(including the constellation), or a constellation");
		}

		/* no width field, so default it */
		Cen_wid_deg = DEF_CEN_WID_DEG;
		break;

	case 2:
		/*
		 * We could have either ra,dec or id,width.  First we will
		 * guess ra,dec, since it doesn't involve searching Stardata.
		 * No existing id conflicts.  If the -a option has an object
		 * name that looks like a RA, sorry.
		 */
		if (hms_str2float(f1, &Cen_ra_hour) == 0) {

			/* f1 was ra, so f2 is declination */

			/* convert declination */
			if (dms_str2float(f2, &Cen_dec_deg) != 0 ||
			    Cen_dec_deg > 90.0 || Cen_dec_deg < -90.0) {
				error("argument to -m has invalid declination "
				"(second field); must be in one of these "
				"formats:  D  Do  DoM  DoM'  DoM'S  DoM'S\"  "
				"where D is degrees, M is arcminutes, and S is "
				"arcseconds, and D can have + or - in front of "
				"it");
			}

			/* no width field, so default it */
			Cen_wid_deg = DEF_CEN_WID_DEG;

		} else if (get_loc_from_id(f1, &Cen_ra_hour,
				&Cen_dec_deg) == 0) {

			/* f1 was an object/const, so f2 is the screen width */

			/* convert desired width of screen */
			if (dms_str2float(f2, &Cen_wid_deg) != 0) {
				error("argument to -m has invalid screen width "
				"(second field); must be in one of these "
				"formats:  D  Do  DoM  DoM'  DoM'S  DoM'S\"  "
				"where D is degrees, M is arcminutes, and S is "
				"arcseconds");
			}
			if (Cen_wid_deg < MIN_CEN_WID_DEG ||
			    Cen_wid_deg > MAX_CEN_WID_DEG) {
				error("argument to -m has out of range screen "
				"width (second field); valid range is %.0f to "
				"%0.f degrees",
				MIN_CEN_WID_DEG, MAX_CEN_WID_DEG);
			}

		} else {

			error("argument to -m has an incorrect first field; "
			"it must be either a right ascension or an ID of a "
			"known object or constellation.  A right ascension "
			"must be in one of these formats:  H  Hh  HhM  HhMm  "
			"HhMmS  HhMmSs  where H is hours, M is minutes, and S "
			"is seconds.  An object must be given as proper name, "
			"or a Bayer or Flamsteed designation including the "
			"constellation");
		}

		break;

	case 3:
		/* three fields must be ra, dec, width */

		/* convert right ascension */
		if (hms_str2float(f1, &Cen_ra_hour) != 0) {
			error("argument to -m has invalid right ascension "
			"(first field); must be in one of these formats:  "
			"H  Hh  HhM  HhMm  HhMmS  HhMmSs  where H is hours, "
			"M is minutes, and S is seconds");
		}

		/* convert declination */
		if (dms_str2float(f2, &Cen_dec_deg) != 0 ||
				Cen_dec_deg > 90.0 || Cen_dec_deg < -90.0) {
			error("argument to -m has invalid declination (second "
			"field); must be in one of these formats:  D  Do  DoM  "
			"DoM'  DoM'S  DoM'S\"  where D is degrees, M is "
			"arcminutes, and S is arcseconds, and D can have + or "
			"- in front of it");
		}

		/* convert desired width of screen */
		if (dms_str2float(f3, &Cen_wid_deg) != 0) {
			error("argument to -m has invalid screen width (third "
			"field); must be in one of these formats:  D  Do  DoM  "
			"DoM'  DoM'S  DoM'S\"  where D is degrees, M is "
			"arcminutes, and S is arcseconds");
		}
		if (Cen_wid_deg < MIN_CEN_WID_DEG ||
		    Cen_wid_deg > MAX_CEN_WID_DEG) {
			error("argument to -m has out of range screen width "
			"(third field); valid range is %.0f to %0.f degrees",
			MIN_CEN_WID_DEG, MAX_CEN_WID_DEG);
		}
		break;

	default:
		error("argument to -m must have comma separated fields in one "
		"of these formats: "
		"right_ascension,declination; "
		"right_ascension,declination,width; "
		"object; "
		"object,width");
	}
}



/*
 * Function:	get_loc_from_id
 *
 * Description:	This function is an ID for something.  It tries to match it to
 *		an object in Stardata (a proper name, or Bayer or Flamsteed
 *		designation) or a constellation.  If it succeeds, it sets
 *		*ra_hour_ptr and *dec_deg_ptr.
 *
 * Returns:	0 if it was found, -1 if not
 */

int
get_loc_from_id(

char *obj,		/* buffer hold object name or designation */
float *ra_hour_ptr,	/* set this to RA */
float *dec_deg_ptr	/* set this to dec */
)
{
	/* lower case versions of the given id, and a proper name in Stardata */
	char lowerobj[MAX_M_OPT + 1];
	char lowerprop[MAX_PROPNAME + 1];

	int sidx;		/* index into Stardata */
	int flamsteed;		/* a Flamsteed number */
	char flam_str[12];	/* Flamsteed number converted to a string */
	char *end_ptr;		/* point after last char of a number */
	char *const_abbrev;	/* 3 capital letters for constellation */
	FILE *bd_p;		/* for reading boundary data file */
	float ra_hour, dec_deg;	/* for a point on a constellation boundary */
	char buf[80];		/* holds one line from the file */
	char con[10];		/* constellation's abbreviation */

	/* max and min coords of the boundary of a constellation */
	float min_ra, max_ra;
	float min_ra_gt6, max_ra_lt18;	/* only the ones away from the 0h line*/
	float min_dec, max_dec;


	/* make everything lower case: we are to ignore case in the id */
	compress_lower(lowerobj, obj);

	/*
	 * See if the id is a proper name.  For the part of Stardata that comes
	 * from the stardata file, we only need to check up to the last entry
	 * that has a proper name.  If it's not found there, look through the
	 * items from the -a option, if any.  We need to compress_lower on all
	 * the names, so that blanks and case are ignored when comparing.
	 */
	for (sidx = 1; sidx <= Last_proper_idx; sidx++) {
		compress_lower(lowerprop, Stardata[sidx].propname);
		if (strcmp(lowerprop, lowerobj) == 0) {
			*ra_hour_ptr = Stardata[sidx].ra_hour;
			*dec_deg_ptr = Stardata[sidx].dec_deg;
			precess(ra_hour_ptr, dec_deg_ptr);
			return 0;
		}
	}
	for (sidx = Num_stars + 1; sidx <= Num_objects; sidx++) {
		compress_lower(lowerprop, Stardata[sidx].propname);
		if (strcmp(lowerprop, lowerobj) == 0) {
			*ra_hour_ptr = Stardata[sidx].ra_hour;
			*dec_deg_ptr = Stardata[sidx].dec_deg;
			precess(ra_hour_ptr, dec_deg_ptr);
			return 0;
		}
	}

	/*
	 * It isn't a proper name.  Check for a Flamsteed designation.  They,
	 * and only they, start with a digit 1 to 9.
	 */
	if (lowerobj[0] >= '1' && lowerobj[0] <= '9') {
		/* must find a number 1 to 999 */
		errno = 0;
		flamsteed = strtol(lowerobj, &end_ptr, 10);
		if (errno != 0 || flamsteed > 999) {
			return -1;
		}
		/* a constellation is required to follow; get its abbreviation*/
		const_abbrev = get_const_abbrev(end_ptr);
		if (const_abbrev == NULL) {
			return -1;
		}
		/* convert number to string so that we can strcmp */
		sprintf(flam_str, "%d", flamsteed);

		/* check Stardata up to last entry that has a flamsteed */
		for (sidx = 1; sidx <= Last_flamsteed_idx; sidx++) {
			if (strcmp(Stardata[sidx].flamsteed, flam_str) == 0 &&
			strcmp(Stardata[sidx].const_abbrev, const_abbrev) == 0){
				*ra_hour_ptr = Stardata[sidx].ra_hour;
				*dec_deg_ptr = Stardata[sidx].dec_deg;
				precess(ra_hour_ptr, dec_deg_ptr);
				return 0;
			}
		}
		return -1;
	}

	/* try Bayer, where the Greek letter is spelled out */
	if (get_bayer(lowerobj, false, ra_hour_ptr, dec_deg_ptr) == 0) {
		return 0;
	}

	/* try Bayer, where it is represented by a single Roman letter */
	if (get_bayer(lowerobj, true, ra_hour_ptr, dec_deg_ptr) == 0) {
		return 0;
	}

	/* at this point, the id needs to be a constellation */
	const_abbrev = get_const_abbrev(lowerobj);
	if (const_abbrev == NULL) {
		return -1;
	}

	/*
	 * We need to decide what RA and dec to use for the constellation.  The
	 * algorithm later handles most of them, but a few need to be hard
	 * coded:  the ones at the poles, and Serpens.
	 */
	if (strcmp(const_abbrev, "UMI") == 0) {
		*ra_hour_ptr = 15.0;
		*dec_deg_ptr = 78.0;
		precess(ra_hour_ptr, dec_deg_ptr);
		return 0;
	} else if (strcmp(const_abbrev, "OCT") == 0) {
		*ra_hour_ptr = 21.0;
		*dec_deg_ptr = -86.0;
		precess(ra_hour_ptr, dec_deg_ptr);
		return 0;
	} else if (strcmp(const_abbrev, "SER") == 0) {
		if (lowerobj[strlen(lowerobj)-1] == 't') {
			/* Serpens Caput */
			*ra_hour_ptr = 15.7;
			*dec_deg_ptr = 11.0;
		} else if (lowerobj[strlen(lowerobj)-1] == 'a') {
			/* Serpens Cauda */
			*ra_hour_ptr = 18.2;
			*dec_deg_ptr = -5.0;
		} else {
			/* Serpens in general */
			*ra_hour_ptr = 17.0;
			*dec_deg_ptr = 0.0;
		}
		precess(ra_hour_ptr, dec_deg_ptr);
		return 0;
	}

	/*
	 * We're going to find the max and min RA and dec of any of the
	 * boundary points, by reading BOUNDARY_FILE, and average them to get
	 * our answers.  But there are two issues with RA.  We need to ignore
	 * points that are too near the poles, since they sometimes throw off
	 * the results.  And for constellations that overlap the 0h line, we
	 * have to use special logic for what we mean by max and min.
	 */
	/* init to extreme values that will be changed later */
	min_ra = min_ra_gt6 = 24.0;
	max_ra = max_ra_lt18 = 0.0;
	min_dec = 90.0;
	max_dec = -90.0;

	bd_p = fopen(BOUNDARY_FILE, "r");
	if (bd_p == NULL) {
		error("can't read %s", BOUNDARY_FILE);
	}
	while (fgets(buf, sizeof (buf), bd_p) != NULL) {
		sscanf(buf, "%f %f %s", &ra_hour, &dec_deg, con);
		if (strcmp(con, const_abbrev) != 0) {
			continue;	/* not our constellation; ignore */
		}

		/*
		 * These cutoff values make sure that every constellation will
		 * have enough boundary within to give a good result, yet avoid
		 * getting too near the polls and getting bad results.
		 */
		if (dec_deg > -77.0 && dec_deg < 70.0) {
			min_ra = ra_hour < min_ra ? ra_hour : min_ra;
			max_ra = ra_hour > max_ra ? ra_hour : max_ra;

			/* also keep track of "min RA > 6 hours" and "max RA <
			 * 18 hours", in case we overlap the 0h line */
			min_ra_gt6 = ra_hour > 6.0 && ra_hour < min_ra_gt6 ?
					ra_hour : min_ra_gt6;
			max_ra_lt18 = ra_hour < 18.0 && ra_hour > max_ra_lt18 ?
					ra_hour : max_ra_lt18;
		}

		min_dec = dec_deg < min_dec ? dec_deg : min_dec;
		max_dec = dec_deg > max_dec ? dec_deg : max_dec;
	}
	fclose(bd_p);

	if (min_ra < 6.0 && max_ra > 18.0) {
		/* might overlap 0h; use special logic */
		*ra_hour_ptr = (min_ra_gt6 + max_ra_lt18 - 24.0) / 2.0;
		if (*ra_hour_ptr < 0.0) {
			*ra_hour_ptr += 24.0;
		}
	} else {
		*ra_hour_ptr = (min_ra + max_ra) / 2.0;
	}
	*dec_deg_ptr = (min_dec + max_dec) / 2.0;

	precess(ra_hour_ptr, dec_deg_ptr);
	return 0;
}



/*
 * Function:	compress_lower
 *
 * Description:	This function copies out_str to in_str, discarding spaces and
 *		converting upper case letters to lower case.
 *
 * Returns:	void
 */

void
compress_lower(

char *out_str,		/* the result goes here */
char *in_str		/* the input comes from here */
)
{
	int in_idx;		/* loop through in_str */
	int out_idx;		/* where to store translated character */


	out_idx = 0;
	for (in_idx = 0; in_str[in_idx] != '\0'; in_idx++) {
		if ( ! isspace(in_str[in_idx]) ) {
			out_str[out_idx] = tolower(in_str[in_idx]);
			out_idx++;
		}
	}
	out_str[out_idx] = '\0';
}



/*
 * Function:	get_const_abbrev
 *
 * Description:	Given a string that may be the name or abbreviation of a
 *		constellation, return the abbreviation in capital letters
 *		(capital so that it matches the abbreviation in stardata).
 *		The input must be all small letters with no blanks.  Both
 *		nominative and genitive versions of names are recognized.
 *
 * Returns:	pointer to the abbreviation in caps, or NULL if not found.
 */

char *
get_const_abbrev(

char *const_string	/* constellation string */
)
{
	static struct {
		char *abbrev;
		char *full;
	} constlist[] = {
				/* nominative forms */
		{ "and",	"andromeda"		},
		{ "ant",	"antlia"		},
		{ "aps",	"apus"			},
		{ "aqr",	"aquarius"		},
		{ "aql",	"aquila"		},
		{ "ara",	"ara"			},
		{ "ari",	"aries"			},
		{ "aur",	"auriga"		},
		{ "boo",	"boötes"		},
		{ "cae",	"caelum"		},
		{ "cam",	"camelopardalis"	},
		{ "cnc",	"cancer"		},
		{ "cvn",	"canesvenatici"		},
		{ "cma",	"canismajor"		},
		{ "cmi",	"canisminor"		},
		{ "cap",	"capricornus"		},
		{ "car",	"carina"		},
		{ "cas",	"cassiopeia"		},
		{ "cen",	"centaurus"		},
		{ "cep",	"cepheus"		},
		{ "cet",	"cetus"			},
		{ "cha",	"chamaeleon"		},
		{ "cir",	"circinus"		},
		{ "col",	"columba"		},
		{ "com",	"comaberenices"		},
		{ "cra",	"coronaaustralis"	},
		{ "crb",	"coronaborealis"	},
		{ "crv",	"corvus"		},
		{ "crt",	"crater"		},
		{ "cru",	"crux"			},
		{ "cyg",	"cygnus"		},
		{ "del",	"delphinus"		},
		{ "dor",	"dorado"		},
		{ "dra",	"draco"			},
		{ "equ",	"equuleus"		},
		{ "eri",	"eridanus"		},
		{ "for",	"fornax"		},
		{ "gem",	"gemini"		},
		{ "gru",	"grus"			},
		{ "her",	"hercules"		},
		{ "hor",	"horologium"		},
		{ "hya",	"hydra"			},
		{ "hyi",	"hydrus"		},
		{ "ind",	"indus"			},
		{ "lac",	"lacerta"		},
		{ "leo",	"leo"			},
		{ "lmi",	"leominor"		},
		{ "lep",	"lepus"			},
		{ "lib",	"libra"			},
		{ "lup",	"lupus"			},
		{ "lyn",	"lynx"			},
		{ "lyr",	"lyra"			},
		{ "men",	"mensa"			},
		{ "mic",	"microscopium"		},
		{ "mon",	"monoceros"		},
		{ "mus",	"musca"			},
		{ "nor",	"norma"			},
		{ "oct",	"octans"		},
		{ "oph",	"ophiuchus"		},
		{ "ori",	"orion"			},
		{ "pav",	"pavo"			},
		{ "peg",	"pegasus"		},
		{ "per",	"perseus"		},
		{ "phe",	"phoenix"		},
		{ "pic",	"pictor"		},
		{ "psc",	"pisces"		},
		{ "psa",	"piscisaustrinus"	},
		{ "pup",	"puppis"		},
		{ "pyx",	"pyxis"			},
		{ "ret",	"reticulum"		},
		{ "sge",	"sagitta"		},
		{ "sgr",	"sagittarius"		},
		{ "sco",	"scorpius"		},
		{ "scl",	"sculptor"		},
		{ "sct",	"scutum"		},
		{ "ser",	"serpens"		},
		{ "sex",	"sextans"		},
		{ "tau",	"taurus"		},
		{ "tel",	"telescopium"		},
		{ "tri",	"triangulum"		},
		{ "tra",	"triangulumaustrale"	},
		{ "tuc",	"tucana"		},
		{ "uma",	"ursamajor"		},
		{ "umi",	"ursaminor"		},
		{ "vel",	"vela"			},
		{ "vir",	"virgo"			},
		{ "vol",	"volans"		},
		{ "vul",	"vulpecula"		},

				/* genitive forms */
		{ "and",	"andromedae"		},
		{ "ant",	"antilae"		},
		{ "aps",	"apodis"		},
		{ "aqr",	"aquarii"		},
		{ "aql",	"aquilae"		},
		{ "ara",	"arae"			},
		{ "ari",	"arietis"		},
		{ "aur",	"aurigae"		},
		{ "boo",	"boötis"		},
		{ "cae",	"caeli"			},
		{ "cam",	"camelopardalis"	},
		{ "cnc",	"cancri"		},
		{ "cvn",	"canesvenaticorum"	},
		{ "cma",	"canismajoris"		},
		{ "cmi",	"canisminoris"		},
		{ "cap",	"capricorni"		},
		{ "car",	"carinae"		},
		{ "cas",	"cassiopeiae"		},
		{ "cen",	"centauri"		},
		{ "cep",	"cephei"		},
		{ "cet",	"ceti"			},
		{ "cha",	"chamaeleontis"		},
		{ "cir",	"circini"		},
		{ "col",	"columbae"		},
		{ "com",	"comaeberenices"	},
		{ "cra",	"coronaeaustralis"	},
		{ "crb",	"coronaeborealis"	},
		{ "crv",	"corvi"			},
		{ "crt",	"crateris"		},
		{ "cru",	"crucis"		},
		{ "cyg",	"cygni"			},
		{ "del",	"delphini"		},
		{ "dor",	"doradus"		},
		{ "dra",	"draconis"		},
		{ "equ",	"equulei"		},
		{ "eri",	"eridani"		},
		{ "for",	"fornacis"		},
		{ "gem",	"geminorum"		},
		{ "gru",	"gruis"			},
		{ "her",	"herculis"		},
		{ "hor",	"horologii"		},
		{ "hya",	"hydrae"		},
		{ "hyi",	"hydrii"		},
		{ "ind",	"indi"			},
		{ "lac",	"lacertae"		},
		{ "leo",	"leonis"		},
		{ "lmi",	"leonisminoris"		},
		{ "lep",	"leporis"		},
		{ "lib",	"librae"		},
		{ "lup",	"lupi"			},
		{ "lyn",	"lyncis"		},
		{ "lyr",	"lyrae"			},
		{ "men",	"mensae"		},
		{ "mic",	"microscopii"		},
		{ "mon",	"monocerotis"		},
		{ "mus",	"muscae"		},
		{ "nor",	"normae"		},
		{ "oct",	"octanis"		},
		{ "oph",	"ophiuchi"		},
		{ "ori",	"orionis"		},
		{ "pav",	"pavonis"		},
		{ "peg",	"pegasi"		},
		{ "per",	"persei"		},
		{ "phe",	"phoenicis"		},
		{ "pic",	"pictoris"		},
		{ "psc",	"piscium"		},
		{ "psa",	"piscisaustrini"	},
		{ "pup",	"puppis"		},
		{ "pyx",	"pyxidis"		},
		{ "ret",	"recticuli"		},
		{ "sge",	"sagittae"		},
		{ "sgr",	"sagittarii"		},
		{ "sco",	"scorpii"		},
		{ "scl",	"sculptoris"		},
		{ "sct",	"scuti"			},
		{ "ser",	"serpentis"		},
		{ "sex",	"sextantis"		},
		{ "tau",	"tauri"			},
		{ "tel",	"telescopii"		},
		{ "tri",	"trianguli"		},
		{ "tra",	"trianguliaustralis"	},
		{ "tuc",	"tucanae"		},
		{ "uma",	"ursaemajoris"		},
		{ "umi",	"ursaeminoris"		},
		{ "vel",	"velorum"		},
		{ "vir",	"virginis"		},
		{ "vol",	"volntis"		},
		{ "vul",	"vulpeculae"		},

				/* alternate versions */
		{ "cam",	"camelopardus"		},
		{ "cam",	"camelopardi"		},
		{ "lmi",	"leominoris"		},
		{ "sco",	"scorpio"		},
		{ "sco",	"scorpionis"		},
		{ "ser",	"serpenscaput"		},
		{ "ser",	"serpenscauda"		},
		{ "uma",	"ursamajoris"		},
		{ "umi",	"ursaminoris"		},
	};

	static char cap_abbrev[4];

	int idx;


	/* linear search to see if we match abbrev or full name of anything */
	for (idx = 0; idx < sizeof (constlist) / sizeof (constlist[0]); idx++) {

		if (strcmp(const_string, constlist[idx].abbrev) == 0 ||
		    strcmp(const_string, constlist[idx].full) == 0) {

			/* found it; convert to upper case and return that */
			cap_abbrev[0] = constlist[idx].abbrev[0] + 'A' - 'a';
			cap_abbrev[1] = constlist[idx].abbrev[1] + 'A' - 'a';
			cap_abbrev[2] = constlist[idx].abbrev[2] + 'A' - 'a';
			cap_abbrev[3] = '\0';
			return cap_abbrev;
		}
	}

	return NULL;
}



/*
 * Function:	get_roman_letter
 *
 * Description:	Given a string that may start with the name of a Greek letter,
 *		find the Roman letter equivalent.  It sets *len_ptr to the
 *		length of the name if found, or garbage if not found.
 *
 * Returns:	the Roman letter, or '\0' if not found
 */

char
get_roman_letter(

char *greek_name,	/* the input string */
int *len_ptr		/* return the length of the letter name here */
)
{
	static struct {
		char roman;
		char *name;
	} greek[] = {
		{ 'a',	"alpha"		},
		{ 'b',	"beta"		},
		{ 'g',	"gamma"		},
		{ 'd',	"delta"		},
		{ 'e',	"epsilon"	},
		{ 'z',	"zeta"		},
		{ 'h',	"eta"		},
		{ 'q',	"theta"		},
		{ 'i',	"iota"		},
		{ 'k',	"kappa"		},
		{ 'l',	"lambda"	},
		{ 'm',	"mu"		},
		{ 'n',	"nu"		},
		{ 'x',	"xi"		},
		{ 'o',	"omicron"	},
		{ 'p',	"pi"		},
		{ 'r',	"rho"		},
		{ 's',	"sigma"		},
		{ 't',	"tau"		},
		{ 'u',	"upsilon"	},
		{ 'f',	"phi"		},
		{ 'c',	"chi"		},
		{ 'y',	"psi"		},
		{ 'w',	"omega"		},
	};

	int idx;

	/* linear search to find the letter name */
	for (idx = 0; idx < sizeof (greek) / sizeof (greek[0]); idx++) {

		/* set its length in case this letter will match */
		*len_ptr = strlen(greek[idx].name);
		if (strncmp(greek_name, greek[idx].name, *len_ptr) == 0) {
			return greek[idx].roman;
		}
	}

	return '\0';
}



/*
 * Function:	get_bayer
 *
 * Description:	This function is given an ID.  It tries to interpret it as a
 *		Bayer designation.  If it succeeds, it sets *ra_hour_ptr and
 *		float *dec_deg_ptr.
 *
 * Returns:	0 if object was found, -1 if not
 */

int
get_bayer(

char *lowerobj,		/* the ID to be interpreted */
bool single_letter,	/* try single Roman letter (versus spelled out)? */
float *ra_hour_ptr,
float *dec_deg_ptr
)
{
	char bayer_str[3];	/* roman letter and optional digit */
	int greek_len;		/* length of a Greek letter name */
	char *c_ptr;		/* point into lowerobj */
	char *const_abbrev;	/* 3 capital letters for constellation */
	int sidx;		/* index into Stardata */


	if (single_letter) {
		/*
		 * Assume the first char is a Roman letter that represents a
		 * Greek letter.  All but j and v can be used that way.
		 */
		bayer_str[0] = lowerobj[0];
		if (bayer_str[0] == 'j' || bayer_str[0] == 'v') {
			return -1;
		}
		greek_len = 1;
	} else {
		/* assume lowerobj starts with a spelled out Greek letter */
		bayer_str[0] = get_roman_letter(lowerobj, &greek_len);
		if (bayer_str[0] == '\0') {
			return -1;	/* failed to match any letter name */
		}
	}

	/*
	 * Point at the char after the Greek letter.  It could be a single
	 * digit, or the constellation could start there.
	 */
	c_ptr = &lowerobj[greek_len];
	if (isdigit(*c_ptr)) {
		/* bayer_str is form letter, digit, \0 */
		bayer_str[1] = *c_ptr;
		bayer_str[2] = '\0';
		c_ptr++;	/* point after the digit */
	} else {
		/* bayer_str is form letter, \0 */
		bayer_str[1] = '\0';
	}

	/* we require the constellation here */
	const_abbrev = get_const_abbrev(c_ptr);
	if (const_abbrev == NULL) {
		return -1;
	}

	/* check Stardata up to last entry that has a bayer */
	for (sidx = 1; sidx <= Last_bayer_idx; sidx++) {
		if (strcmp(Stardata[sidx].bayer, bayer_str) == 0 &&
		    strcmp(Stardata[sidx].const_abbrev, const_abbrev) == 0) {
			*ra_hour_ptr = Stardata[sidx].ra_hour;
			*dec_deg_ptr = Stardata[sidx].dec_deg;
			precess(ra_hour_ptr, dec_deg_ptr);
			return 0;
		}
	}

	return -1;
}



/*
 * Function:	load_stardata
 *
 * Description:	This function reads in the files STARDATA_FILE, BAYER_POS_FILE,
 *		and PROPER_POS_FILE, storing the information into Stardata.  It
 *		also loads information there from the -a command line option
 *		for additional objects.
 *
 * Returns:	void (does not return if error)
 */

void
load_stardata(char *addobj)
{
	FILE *f_ptr;		/* for reading a file */
	char buf[10000];	/* read a line from a file, or arg of "-a" */
	int sidx;		/* index into Stardata */
	int fields;		/* fields scanned by scanf */
	char *prop_ptr;		/* point at a proper name */
	char *wid_ptr;		/* point at screen width after a proper name */
	float widnum;		/* the width number (in degrees) */
	char *obj_ptr;		/* point at a field in addobj */
	int bidx;		/* index into buf */
	int idx;		/* index into a line for "-a" file */
	int seps;		/* count separators in a line for "-a" file*/
	bool got_nonspace;	/* nonspace found in a line of "-a" file */


	/************* load the stars from stardata ************/

	f_ptr = fopen(STARDATA_FILE, "r");
	if (f_ptr == NULL) {
		error("can't read %s", STARDATA_FILE);
	}

	sidx = 0;
	while (fgets(buf, sizeof (buf), f_ptr) != NULL) {

		/* increment first, so that line 1 goes into Stardata[1], etc.*/
		sidx++;

		buf[strlen(buf) - 1] = '\0';	/* zap the \n */

		/* load these items */
		fields = sscanf(buf, "%f%f%f%s%s%s",
			&Stardata[sidx].ra_hour,
			&Stardata[sidx].dec_deg,
			&Stardata[sidx].mag,
			 Stardata[sidx].const_abbrev,
			 Stardata[sidx].bayer,
			 Stardata[sidx].flamsteed);

		switch (fields) {
		case 3:
			/* these fields are not available, so default them */
			Stardata[sidx].const_abbrev[0] = '\0';
			Stardata[sidx].bayer[0] = '\0';
			Stardata[sidx].flamsteed[0] = '\0';
			Stardata[sidx].propname[0] = '\0';

			/*
			 * The stars are in order of decreasing magnitude,
			 * except for a few exceptions in the first 4000 lines
			 * of the file.  But after that point, we still need to
			 * read more, in case we need ID information for
			 * get_loc_from_id().  At this point, though, there
			 * is never any ID information, so if we are fainter
			 * than the magnitude limit, stop reading.
			 */
			if (Stardata[sidx].mag > Mag_limit) {
				goto quit_stardata;
			}
			break;
		case 6:
			/*
			 * If there is no constellation abbreviation, wipe out
			 * the dash.  It was just there for parsing.
			 */
			if (Stardata[sidx].const_abbrev[0] == '-') {
				Stardata[sidx].const_abbrev[0] = '\0';
			}

			/*
			 * If there is no Bayer, wipe out the dash.  It was
			 * just there for parsing.  Otherwise, remember that
			 * this was the last Bayer seen so far.
			 */
			if (Stardata[sidx].bayer[0] == '-') {
				Stardata[sidx].bayer[0] = '\0';
			} else {
				Last_bayer_idx = sidx;
			}

			/*
			 * If there is no Flamsteed, wipe out the zero.  It was
			 * just there for parsing.  Otherwise, remember that
			 * this was the last Flamsteed seen so far.
			 */
			if (Stardata[sidx].flamsteed[0] == '0') {
				Stardata[sidx].flamsteed[0] = '\0';
			} else {
				Last_flamsteed_idx = sidx;
			}

			/*
			 * There must be an equal sign; and the proper name,
			 * if any, follows it.
			 */
			prop_ptr = strchr(buf, '=');
			if (prop_ptr == NULL) {
				error("stardata file is corrupt: star %d has "
				"%d fields but no \"=\" in that line",
					sidx, fields);
			}
			prop_ptr++;	/* point at proper name if any */

			/* check for a limiting screen width */
			wid_ptr = strchr(prop_ptr, ':');
			if (wid_ptr != NULL) {
				*wid_ptr = '\0';/* terminate the proper name */
				wid_ptr++;	/* point at the width number */
				if (sscanf(wid_ptr, "%f",
					    &Stardata[sidx].width_limit) != 1) {
					error("stardata file is corrupt: "
					"screen width for star %d is invalid",
					sidx);
				}
				/*
				 * Discard proper name if not a centered map.
				 * Later we will discard it if the map covers
				 * too wide an area to print the name without
				 * colliding with things.  But we can't do that
				 * until we know the map width.
				 */
				if ( ! CENTER ) {
					prop_ptr[0] = '\0';
				}
			}

			/*
			 * If there is a proper name, enforce length limit,
			 * store it, and remember that this was the last one
			 * seen so far.
			 */
			if (prop_ptr[0] != '\0') {
				if (strlen(prop_ptr) > MAX_PROPNAME) {
					error("stardata file is corrupt: "
					"proper name of star %d is too long "
					"(limit is %d characters)",
					sidx, MAX_PROPNAME);
				}
				strcpy(Stardata[sidx].propname, prop_ptr);
				Last_proper_idx = sidx;
			}
			break;
		default:
			error("stardata file is corrupt: sscanf of star %d got "
			"%d fields", sidx, fields);
		}
	}

quit_stardata:
	fclose(f_ptr);

	Num_stars = sidx;	/* remember how many stars there are */


	/************* load bayer position information for stars ************/

	f_ptr = fopen(BAYER_POS_FILE, "r");
	if (f_ptr == NULL) {
		error("can't read %s", BAYER_POS_FILE);
	}

	sidx = 0;
	while (fgets(buf, sizeof (buf), f_ptr) != NULL) {
		sidx++;
		sscanf(buf, "%f%f%f%f%f%f",
			&Stardata[sidx].bayerpos[0].xoff,
			&Stardata[sidx].bayerpos[0].yoff,
			&Stardata[sidx].bayerpos[1].xoff,
			&Stardata[sidx].bayerpos[1].yoff,
			&Stardata[sidx].bayerpos[2].xoff,
			&Stardata[sidx].bayerpos[2].yoff);
	}
	fclose(f_ptr);


	/********** load proper name position information for stars *********/

	f_ptr = fopen(PROPER_POS_FILE, "r");
	if (f_ptr == NULL) {
		error("can't read %s", PROPER_POS_FILE);
	}

	sidx = 0;
	while (fgets(buf, sizeof (buf), f_ptr) != NULL) {
		sidx++;
		sscanf(buf, "%f%f%f%f%f%f",
			&Stardata[sidx].properpos[0].xoff,
			&Stardata[sidx].properpos[0].yoff,
			&Stardata[sidx].properpos[1].xoff,
			&Stardata[sidx].properpos[1].yoff,
			&Stardata[sidx].properpos[2].xoff,
			&Stardata[sidx].properpos[2].yoff);
	}
	fclose(f_ptr);


	/************ load any additional objects on command line ***********/

	if (addobj == NULL) {
		Num_objects = Num_stars;
		return;		/* there are none */
	}

	if (strncmp(addobj, "file=", 5) == 0) {
		/*
		 * We are to read the objects from a file.  The name starts
		 * right after the "=".
		 */
		f_ptr = fopen(&addobj[5], "r");
		if (f_ptr == NULL) {
			error("can't read -a option's file \"%s\"", &addobj[5]);
		}

		/*
		 * Convert the contents of the file into the format that is used
		 * when the data is listed on the command line:  convert white
		 * space separators into commas, and newlines into colons.
		 * That way the same code can process either.
		 */
		bidx = 0;
		/* read line into where the previous one ended */
		while (fgets(&buf[bidx], sizeof (buf) - bidx, f_ptr) != NULL) {
			idx = bidx;
			seps = 0;
			got_nonspace = false;

			/* ignore leading white space */
			while (isspace(buf[idx])) {
				idx++;
			}

			/* loop through line, compressing and altering seps */
			while (buf[idx] != '\n' && buf[idx] != '\0') {
				/*
				 * If not a space, just copy the char.  But, if
				 * it is a comma, count it as a separator.  And
				 * if it is the third one, just copy the rest of
				 * the line and get out.  This is so that a name
				 * can contain blanks.
				 */
				if ( ! isspace(buf[idx]) ) {
					got_nonspace = true;
					buf[bidx++] = buf[idx++];
					if (buf[bidx - 1] == ',') {
						seps++;
						if (seps == 3) {
							while (buf[idx] != '\n'
							&& buf[idx] != '\0') {
								buf[bidx++] =
								buf[idx++];
							}
							break;
						}
					}
					continue;
				}

				/* it was a space; skip over multiple spaces */
				while ( isspace(buf[idx]) ) {
					idx++;
				}
				/* use a comma separator in its place */
				buf[bidx++] = ',';
				seps++;

				/* if third one, copy rest of line & get out */
				if (seps == 3) {
					while (buf[idx] != '\n' &&
							buf[idx] != '\0') {
						buf[bidx++] = buf[idx++];
					}
					break;
				}
			}

			/* if there was anything on this line, add colon */
			if (got_nonspace) {
				buf[bidx++] = ':';
			}
		}

		if (bidx == 0) {
			error("file \"%s\" in -a option is empty", &addobj[5]);
		}
		buf[--bidx] = '\0';
		fclose(f_ptr);
	} else {
		/* copy the arg into local buf to avoid trashing command line */
		strncpy(buf, addobj, sizeof (buf));
		buf[sizeof (buf) - 1] = '\0';	/* cut off if need be */
	}

	/*
	 * The buf now has a field for each object, separated by colons.  Loop
	 * once per each object.
	 */
	sidx = Num_stars;	/* start after last star */
	for (obj_ptr = strtok(buf, ":"); obj_ptr != NULL;
				obj_ptr = strtok(NULL, ":")) {
		sidx++;
		if (sidx >= MAX_OBJECTS) {
			error("too many additional objects in -a options "
				"(limit is %d); skipping the rest",
				sidx - Num_stars - 1);
		}

		/* parse and load the info for this object */
		get_additional_obj(obj_ptr, 
				&Stardata[sidx].ra_hour,
				&Stardata[sidx].dec_deg,
				&Stardata[sidx].mag,
				 Stardata[sidx].propname);

		/*
		 * We never use constellation, bayer, or flamsteed, and we
		 * default the proper name position.
		 */
		Stardata[sidx].const_abbrev[0] = '\0';
		Stardata[sidx].bayer[0] = '\0';
		Stardata[sidx].flamsteed[0] = '\0';
		Stardata[sidx].properpos[0].xoff = 0;
		Stardata[sidx].properpos[0].yoff = 0;
		Stardata[sidx].properpos[1].xoff = 0;
		Stardata[sidx].properpos[1].yoff = 0;
		Stardata[sidx].properpos[2].xoff = 0;
		Stardata[sidx].properpos[2].yoff = 0;
	}

	Num_objects = sidx;	/* set the number of objects */
}



/*
 * Function:	get_additional_obj
 *
 * Description:	This function parses one piece of the data from -a, the info
 *		for on additional object to be printed.  The fields are right
 *		ascension, declination, magnitude, and name, comma separated.
 *		The name is optional, and if none is given, the last comma may
 *		be omitted.
 *
 * Returns:	void (does not return if error)
 */

void
get_additional_obj(

char *str,		/* the string that is the piece of the arg of -a */
float *ra_hour_ptr,	/* to be filled in */
float *dec_deg_ptr,	/* to be filled in */
float *mag_ptr,		/* to be filled in */
char *name_ptr		/* to be filled in */
)
{
	char buf[100];		/* copy of str, to avoid overwriting str */
	/* places for each field */
	char ra[100], dec[100], mag[100], name[100], junk[100];
	int idx;		/* loop through str and buf */
	int fields;		/* number of fields we found */


	if (strlen(str) >= sizeof (buf)) {
		error("the entry for an object in -a is too long; limit is "
			"%d characters", sizeof (buf) - 1);
	}

	/*
	 * Copy, converting commas to blanks and blanks to some character that
	 * will never be used in a name, so we can use sscanf to extract the
	 * fields while allowing blanks in names.
	 */
	for (idx = 0; str[idx] != '\0'; idx++) {
		if (str[idx] == ',') {
			buf[idx] = ' ';
		} else if (str[idx] == ' ') {
			buf[idx] = '\a';
		} else {
			buf[idx] = str[idx];
		}
	}
	buf[idx] = '\0';

	/* get the fields */
	fields = sscanf(buf, "%s%s%s%s%s", ra, dec, mag, name, junk);

	if (fields != 3 && fields != 4) {
		error("an object in -a must have either 3 or 4 fields:  right "
		"ascension, declination, magnitude, and optional name.  If "
		"given on the command line, they must be separated by commas.  "
		"If given in a file, they can be separated by commas, spaces, "
		"or tabs");
	}

	/* convert right ascension */
	if (hms_str2float(ra, ra_hour_ptr) != 0) {
		error("an object in the -a option (%s) has invalid right "
		"ascension (first field); must be in one of these formats:  "
		"H  Hh  HhM  HhMm  HhMmS  HhMmSs  where H is hours, M is "
		"minutes, and S is seconds", str);
	}

	/* convert declination */
	if (dms_str2float(dec, dec_deg_ptr) != 0 ||
			*dec_deg_ptr > 90.0 || *dec_deg_ptr < -90.0) {
		error("an object in the -a option (%s) has invalid declination "
		"(second field); must be in one of these formats:  "
		"D  Do  DoM  DoM'  DoM'S  DoM'S\"  where D is degrees, M is "
		"arcminutes, and S is arcseconds, and D can have + or - in "
		"front of it", str);
	}

	/* get magnitude, but don't impose any limits on it */
	if (sscanf(mag, "%f", mag_ptr) != 1) {
		error("an object in the -a option has invalid magnitude");
	}

	if (fields == 4) {
		/* restore the funny character \a back to the blank it was */
		for (idx = 0; name[idx] != '\0'; idx++) {
			if (name[idx] == '\a') {
				name[idx] = ' ';
			}
		}
		/* copy the name, null terminate if too long */
		strncpy(name_ptr, name, MAX_PROPNAME);
		name_ptr[MAX_PROPNAME] = '\0';
	} else {
		/* set to no name */
		name_ptr[0] = '\0';
	}
}



/*
 * Function:	calc_stardata
 *
 * Description:	This function completes the populating of Stardata.  This
 *		(well, at least some parts of this) had to be done after
 *		get_centered_args was done.
 *
 * Returns:	void
 */

void
calc_stardata()
{
	int sidx;		/* index into Stardata */


	for (sidx = 1; sidx <= Num_objects; sidx++) {

		/* apply precession, if any, to the coordinates */
		precess(&Stardata[sidx].ra_hour, &Stardata[sidx].dec_deg);

		if ( ! CIRCUMPOLAR) {
			/* set x and y for these maps */
			set_star_coords(sidx);
		}

		Stardata[sidx].radius = star_radius(Stardata[sidx].mag);
	}
}



/*
 * Function:	parse_viewfields
 *
 * Description:	This function parses the argument of -v, the viewfield circles.
 *		They are colon separated.  Each one has a diameter, and
 *		optionally RA/declination or ID, comma separated.  It is called
 *		early in the program, before we output any PostScript.  Later
 *		we will draw the circles.
 *
 * Returns:	void (does not return if error)
 */

void
parse_viewfields(

char *str		/* the string that is the arg of -v */
)
{
	char buf[MAX_V_OPT + 1];/* copy of str, to avoid overwriting argv */
	char saveview[MAX_V_OPT + 1];

	/* places for each subfield plus a bogus one to catch extra junk */
	char diam[MAX_V_OPT + 1];
	char ra[MAX_V_OPT + 1];		/* RA, or an ID */
	char dec[MAX_V_OPT + 1];
	char junk[MAX_V_OPT + 1];

	int idx;		/* loop through args and buf */
	char *view_ptr;		/* point at a viewfield */
	float view_diam_deg;	/* diameter of a viewfield circle */
	float ra_hour;		/* RA of center of viewfield circle */
	float dec_deg;		/* declination of center of viewfield circle */
	int subfields;		/* number of subfields in a view field */


	if (strlen(str) >= sizeof (buf)) {
		error("argument to -v is too long; limit is %d characters",
			sizeof (buf) - 1);
	}

	/* copy to avoid destroying the input str */
	strcpy(buf, str);

	/* loop through each viewfield to validate and store the view fields */
	for (view_ptr = strtok(buf, ":"); view_ptr != NULL;
				view_ptr = strtok(NULL, ":")) {

		strcpy(saveview, view_ptr);	/* save for error messages */

		/*
		 * Convert commas to blanks so we can use sscanf to extract
		 * the subfields.
		 */
		for (idx = 0; view_ptr[idx] != '\0'; idx++) {
			if (view_ptr[idx] == ',') {
				view_ptr[idx] = ' ';
			}
		}

		/* get the subfields */
		subfields = sscanf(view_ptr, "%s%s%s%s", diam, ra, dec, junk);

		if (subfields < 1 || subfields > 3) {
			error("a view field in the -v option (%s) is invalid; "
			"it must have 1, 2, or 3 subfields:  diameter, and "
			"optionally either a right ascension and declination "
			"or an object.  They must be separated by commas",
			saveview);
		}

		if (dms_str2float(diam, &Vf_diam_deg[Num_view_fields]) != 0) {
			error("a viewfield in -v (%s) has invalid diameter; "
			"it must be in one of these formats:  D  Do  DoM  "
			"DoM'  DoM'S  DoM'S\"  where D is degrees, M is "
			"arcminutes, and S is arcseconds", saveview);
		}
		if (Vf_diam_deg[Num_view_fields] > 90.0 ||
		    Vf_diam_deg[Num_view_fields] <= 0.0) {
			error("viewfield in -v must be greater than 0 and not "
				"exceed 90 degrees");
		}

		switch (subfields) {
		case 1:
			/* no RA/dec or object given, so use center point */
			Vf_ra_hour[Num_view_fields] = Cen_ra_hour;
			Vf_dec_deg[Num_view_fields] = Cen_dec_deg;
			Num_view_fields++;
			break;

		case 2:
			if (get_loc_from_id(ra, &Vf_ra_hour[Num_view_fields],
					&Vf_dec_deg[Num_view_fields]) != 0) {
				error("a view field in the -v option (%s) is "
				"invalid; when two subfields are given, the "
				"second must be a proper name, or a Bayer or "
				"Flamsteed designation (including the "
				"constellation), or a constellation", saveview);
			}

			/* un-precess the coords by temporarily altering year */
			Precess_year = EPOCH - (Precess_year - EPOCH);
			precess(&Vf_ra_hour[Num_view_fields],
				&Vf_dec_deg[Num_view_fields]);
			Precess_year = EPOCH - (Precess_year - EPOCH);
			break;

		case 3:
			/* convert right ascension */
			if (hms_str2float(ra, &Vf_ra_hour[Num_view_fields])
					!= 0) {
				error("a view field in the -v option (%s) is "
				"invalid; when three subfields are given, the "
				"second must be right ascension in one of "
				"these formats:  H  Hh  HhM  HhMm  HhMmS  "
				"HhMmSs  where H is hours, M is minutes, and "
				"S is seconds", saveview);
			}
	
			/* convert declination */
			if (dms_str2float(dec, &Vf_dec_deg[Num_view_fields])
					!= 0 ||
					Vf_dec_deg[Num_view_fields] > 90.0 ||
					Vf_dec_deg[Num_view_fields] < -90.0) {
				error("a view field in the -v option (%s) is "
				"invalid: when three subfields are given, the "
				"third must be declination in one of these "
				"formats:  D  Do  DoM  DoM'  DoM'S  DoM'S\"  "
				"where D is degrees, M is arcminutes, and S "
				"is arcseconds, and D can have + or - in "
				"front of it", saveview);
			}
			break;
		}

		Num_view_fields++;
	}
}



/*
 * Function:	get_zenith
 *
 * Description:	This function parses the argument of -z, the coordinates of the
 *		zenith.  The fields are right ascension and declination, comma
 *		separated.
 *
 * Returns:	void (does not return if error)
 */

void
get_zenith(

char *args		/* the string that is the arg of -z */
)
{
	char buf[100];		/* copy of args, to avoid overwriting "args" */
	/* places for each field plus a bogus one to catch extra junk */
	char ra[100], dec[100], junk[100];
	int idx;		/* loop through args and buf */
	int fields;		/* number of fields we found */


	if (strlen(args) >= sizeof (buf)) {
		error("argument to -z is too long; limit is %d characters",
			sizeof (buf) - 1);
	}

	/* copy, converting commas to blanks, so we can use sscanf */
	for (idx = 0; args[idx] != '\0'; idx++) {
		if (args[idx] == ',') {
			buf[idx] = ' ';
		} else {
			buf[idx] = args[idx];
		}
	}
	buf[idx] = '\0';

	/* get as many fields as we can */
	fields = sscanf(buf, "%s%s%s", ra, dec, junk);

	if (fields != 2) {
		error("argument to -z must have 2 comma separated fields, "
			"right ascension and declination");
	}

	/* convert right ascension */
	if (hms_str2float(ra, &Zenith_ra_hour) != 0) {
		error("argument to -z has invalid right ascension (first "
		"field); must be in one of these formats:  H  Hh  HhM  HhMm  "
		"HhMmS  HhMmSs  where H is hours, M is minutes, and S is "
		"seconds");
	}

	/* convert declination */
	if (dms_str2float(dec, &Zenith_dec_deg) != 0 ||
			Zenith_dec_deg > 90.0 || Zenith_dec_deg < -90.0) {
		error("argument to -z has invalid declination (second field); "
		"must be in one of these formats:  D  Do  DoM  DoM'  DoM'S  "
		"DoM'S\"  where D is degrees, M is arcminutes, and S is "
		"arcseconds, and D can have + or - in front of it");
	}
}



/*
 * Function:	output_prolog
 *
 * Description:	This function outputs some initial postscript statements.
 *
 * Returns:	void
 */

void
output_prolog()
{
	/* some general setup */
	printf("%.2f setlinewidth\n", THIN);
	printf("1 setlinejoin\n");

	/*
	 * If white against black, draw the black background, a big black
	 * rectangle covering more than the whole screen.
	 */
	if (White_background == false) {
		printf("0 setgray\n");
		printf("newpath\n");
		printf("%.2f %.2f moveto\n", MIN_X - 100, MIN_Y - 100);
		printf("%.2f %.2f lineto\n", MIN_X - 100, MAX_Y + 100);
		printf("%.2f %.2f lineto\n", MAX_X + 100, MAX_Y + 100);
		printf("%.2f %.2f lineto\n", MAX_X + 100, MIN_Y - 100);
		printf("closepath fill stroke\n");
	}
}



/*
 * Function:	print_title
 *
 * Description:	This function prints the title, based on the -t option.
 *
 * Returns:	void
 */

void
print_title(

char *title
)
{
	char buf[1000];		/* copy of title, to avoid overwriting title */
	float startx, starty;	/* place to start printing the first line */
	float fontsize;		/* either default or get from the title */
	int extra_lines;	/* how many lines beyond the first line? */
	char *c_ptr;		/* point along the title */
	char *line_ptr;		/* point at the start of each line */


	if (strlen(title) >= sizeof (buf)) {
		error("argument to -t is too long; limit is %d characters",
			sizeof (buf) - 1);
	}

	/*
	 * We will print the title above where the normal map is printed.  The
	 * place is different for circumpolar maps.
	 */
	if (CIRCUMPOLAR) {
		startx = MIN_X + 10.0;
		starty = MID_Y + (MAX_X - MIN_X) / 2.0 + 3.0;
	} else {	/* midsection, or a centered map */
		startx = Min_x;
		starty = Max_y + 27.0;
	}

	/*
	 * Try to get the fontsize from the start of the title string.  If what
	 * is there doesn't match this syntax, it means the size was not given,
	 * so default it.
	 */
	if (sscanf(title, "s(%f)", &fontsize) == 0) {
		fontsize = 14.0;	/* default */
	} else {
		/* skip by the font size to get to the real string */
		while (*title != ')') {
			title++;
		}
		title++;
	}

	/* copy so we don't destroy title */
	strcpy(buf, title);

	/*
	 * Count how many additional lines by searching for \n.  The user isn't
	 * supposed to end the last line with \n, but if they did, ignore it.
	 */
	extra_lines = 0;
	for (c_ptr = buf; *c_ptr != '\0'; c_ptr++) {
		if (c_ptr[0] == '\\' && c_ptr[1] == 'n' && c_ptr[2] != '\0') {
			extra_lines++;
		}
	}

	/* start high enough so there is room for all lines */
	starty += extra_lines * fontsize;

	printf("/Times-Roman findfont\n");
	printf("%f scalefont setfont\n", fontsize);

	/* loop once per line, printing them */
	line_ptr = buf;
	for (c_ptr = buf; *c_ptr != '\0'; c_ptr++) {
		if (c_ptr[0] == '\\' && c_ptr[1] == 'n') {
			/* terminate the preceding line and print it */
			c_ptr[0] = '\0';
			printf("%.2f %.2f moveto\n", startx, starty);
			printf("(%s) show\n", line_ptr);
			line_ptr = &c_ptr[2];	/* point at next line */
			if (*line_ptr == '\0') {
				return;	/* there is nothing after the \n */
			}
			starty -= fontsize;
			c_ptr++;
		}
	}

	/* print the last line */
	printf("%.2f %.2f moveto\n", startx, starty);
	printf("(%s) show\n", line_ptr);
}



/*
 * Function:	output_postlog
 *
 * Description:	This function outputs some final postscript statements.
 *
 * Returns:	void
 */

void
output_postlog()
{
	printf("showpage\n");
	printf("quit\n");
}



/*
 * Function:	print_grid
 *
 * Description:	This function prints the coordinate grid, the lines of right
 *		ascension and declination.  For the standard (non-centered)
 *		maps, it does that and never prints the coordinate numbers,
 *		which is done by print_grid_numbers.  For centered maps, it
 *		prints the grid when call == 1; but when call == 2 it prints
 *		the numbers; print_grid_numbers is never used. 	This is to
 *		avoid duplicating a lot of logic, and to still be able to call
 *		whiteout in between.  Standard could have been done like
 *		centered, but wasn't, for historical reasons.
 *
 * Returns:	void
 */

void
print_grid(

int call	/* first call (1) or seconds call (2) */
)
{
	printf("%% print_grid %d\n", call);

	printf("gsave\n");

	if (CENTER) {
		print_grid_center(call);
	} else {
		if (Black_and_white_map) {
			/* dashed lines to differentiate from other lines */
			printf("[5] 0 setdash\n");
		} else {
			if (White_background) {
				printf("0.1 1 0.1 setrgbcolor\n");
			} else {
				printf("0 0.6 0 setrgbcolor\n");
			}
		}

		if (CIRCUMPOLAR) {
			print_grid_polar();
		} else if (MIDSECTION) {
			print_grid_mid();
		}
	}

	printf("grestore\n");
}



/*
 * Function:	print_grid_polar
 *
 * Description:	This function prints the coordinate grid for the circumpolar
 *		maps.
 *
 * Returns:	void
 */

void
print_grid_polar()
{
	int ra_hour;		/* right ascension */
	int dec_deg;		/* declination */


	printf("gsave\n");

	/* move to center of the page */
	printf("%.2f %.2f translate\n", MID_X, MID_Y);

	/* draw RA (radial) lines every 2 hours, but only to 80 dec (not 90) */
	for (ra_hour = 0; ra_hour < 24; ra_hour += 2) {
		printf("0 %.2f moveto 0 %.2f lineto stroke\n",
			DEC_DEG2R(80), DEC_DEG2R(35));
		printf("30 rotate\n");	/* rotate 2 hours */
	}

	/* draw declination circles every 10 degrees */
	for (dec_deg = 80; dec_deg >= 40; dec_deg -= 10) {
		printf("newpath 0 0 %.2f 0 360 arc stroke closepath\n",
			DEC_DEG2R(dec_deg));
	}

	printf("grestore\n");
}



/*
 * Function:	print_grid_mid
 *
 * Description:	This function prints the coordinate grid for the midsection
 *		maps.
 *
 * Returns:	void
 */

void
print_grid_mid()
{
	int ra_hour;		/* right ascension */
	int dec_deg;		/* declination */


	/* RA (vertical) lines; the +6 is to allow for wrap around */
	for (ra_hour = 0; ra_hour <= 24 + 6; ra_hour++) {
		printf("%.2f %.2f moveto %.2f %.2f lineto stroke\n",
			RA_HOUR2X(ra_hour), Min_y, RA_HOUR2X(ra_hour), Max_y);
	}

	/* declination (horizontal) lines */
	for (dec_deg = -60; dec_deg <= 60; dec_deg += 10) {
		/* the "100" is to let it overflow into the margin */
		if (Map == 3 || Map == 5) {
			printf("%.2f %.2f moveto %.2f %.2f lineto stroke\n",
					Min_x - 100, DEC_DEG2Y(dec_deg),
					Max_x, DEC_DEG2Y(dec_deg));
		} else {
			printf("%.2f %.2f moveto %.2f %.2f lineto stroke\n",
					Min_x, DEC_DEG2Y(dec_deg),
					Max_x + 100, DEC_DEG2Y(dec_deg));
		}
	}
}



/*
 * Function:	print_grid_center
 *
 * Description:	For call == 1, this function prints the coordinate grid for a
 *		centered map.  For call == 2, it prints the coordinate numbers
 *		for a centered map.
 *
 * Returns:	void
 */

void
print_grid_center(

int call		/* which call to this function */
)
{
	float ra_hour;		/* right ascension */
	float dec_deg;		/* declination */
	int iter;		/* iteration of loop */
	float ra_limit_even;	/* up to what dec do we draw even hour line? */
	float ra_limit_odd;	/* up to what dec do we draw old hour line? */
	float ra_limit_half;	/* up to what dec do we draw half line? */
	float ra_limit_5min;	/* up to what dec do we draw 5 min line? */


	/* decide how close to the pole to draw RA lines */
	switch (Grid_level) {
	case 1:
		ra_limit_even = 80.0;	/* actually 90 when RA % 6 == 0 */
		ra_limit_odd  = 60.0;
		break;
	case 2:
		ra_limit_even = 85.0;
		ra_limit_odd  = 75.0;
		ra_limit_half = 65.0;
		break;
	case 3:
		ra_limit_even = 85.0;
		ra_limit_odd  = 85.0;
		ra_limit_half = 80.0;
		ra_limit_5min = 70.0;
		break;
	}

	if (call == 2) {
		/* prepare for printing numbers */
		printf("/Times-Roman findfont\n");
		printf("12 scalefont setfont\n");
	}

	/*
	 * For each loop on a floating point number, stop short of, or go
	 * beyond, an integer, so that floating point roundoff won't cause an
	 * extra or missing loop iteration.  For loops that need to check the
	 * loop variable with a "%" operation, we instead loop on an integer.
	 */

	if (call == 1) {
		set_grid_appearance(1);	/* set appearance for level 1 lines */
	}

	/* every even hour draw a level 1 line */
	for (iter = 0; iter < 12; iter++) {
		ra_hour = iter * 2.0;
		print_grid_center_raline(call, ra_hour,
			iter % 3 == 0 ? 90.0 : ra_limit_even, 1);
	}

	/* every odd hour, draw a level 1 line */
	for (ra_hour = 1.0; ra_hour < 23.999; ra_hour += 2.0) {
		print_grid_center_raline(call, ra_hour, ra_limit_odd, 1);
	}

	/* draw a level 1 line every 10 degrees */
	for (dec_deg = -80.0; dec_deg < 80.001; dec_deg += 10.0) {
		print_grid_center_decline(call, dec_deg, 1);
	}

	/* if we should only due level 1 lines, get out */
	if (Grid_level == 1) {
		return;
	}

	if (call == 1) {
		set_grid_appearance(2);	/* set appearance for level 2 lines */
	}

	/* draw half hour lines */
	for (ra_hour = 0.5; ra_hour < 23.999; ra_hour += 1.0) {
		print_grid_center_raline(call, ra_hour, ra_limit_half, 2);
	}

	/* draw a line every 5 degrees */
	for (dec_deg = -85.0; dec_deg < 85.001; dec_deg += 10.0) {
		print_grid_center_decline(call, dec_deg, 2);
	}

	/*
	 * If we should only due level 1 and 2 lines, get out.  Also get out if
	 * we are printing numbers.  We don't want to print level 3 numbers
	 * unless the screen width is 10 degrees or less.
	 */
	if (Grid_level == 2 || call == 2 && Cen_wid_deg > 10.0) {
		return;
	}

	if (call == 1) {
		set_grid_appearance(3);	/* set appearance for level 3 lines */
	}

	/* draw 5 minute lines */
	for (iter = 1; iter < 12 * 24; iter++) {
		if (iter % 6 != 0) {
			ra_hour = iter / 12.0;
			print_grid_center_raline(call, ra_hour,
				ra_limit_5min, 3);
		}
	}

	/* draw a line every degree */
	for (iter = -89; iter <= 89; iter++) {
		if (iter % 5 != 0) {
			dec_deg = iter;
			print_grid_center_decline(call, dec_deg, 3);
		}
	}
}



/*
 * Function:	print_grid_center_raline
 *
 * Description:	For call == 1, this function draws one right ascension line
 *		for a centered map.  For call == 2, it prints numbers where it
 *		crosses a map edge.  Because these lines may be curved in
 *		various ways, we can't use PostScript's "dashed" option to
 *		draw the line in a single command.  We draw individual dashes
 *		as short, straight lines, indistinguishable from curved lines.
 *
 * Returns:	void
 */

void
print_grid_center_raline(

int call,		/* draw line if 1, print numbers if 2 */
float ra_hour,		/* right ascension of this line */
float declim,		/* what declination we stop at */
int level		/* which level of line this is */
)
{
	/*
	 * The "1" variables are for the start of a dash.
	 * The "2" variables are for the end of a dash.
	 * The "prev" variables are for the start of the previous dash.
	 */
	float dec1_deg, dec2_deg;		/* declination */
	float x1, x2, prev_x1;			/* x coord */
	float y1, y2, prev_y1;			/* y coord */
	enum CENTERPOS pos1, pos2, prev_pos1;	/* position on or off map */

	float dist;		/* dist between starts of neighboring dashes */
	float len;		/* length of a dash */


	/*
	 * Make the distance between the starts of neighboring dashes and the
	 * length of each dash proportional to the scale of degrees, so that
	 * the actual printed distances end up the same, regardless of scale.
	 */
	dist = 1.5 * SCALE;

	if (Black_and_white_map) {
		len = 0.75 * SCALE;
	} else {
		len = dist;		/* join dashes into a solid line */
	}

	/* loop once for each dash we might draw */
	for (dec1_deg = -declim; dec1_deg < declim; dec1_deg += dist) {

		dec2_deg = dec1_deg + len;	/* get other end of dash */

		/* get x,y of dash's endpoints */
		pos1 = center_xy(ra_hour, dec1_deg, &x1, &y1);
		pos2 = center_xy(ra_hour, dec2_deg, &x2, &y2);

		/* if first loop iter, set prev to be the current */
		if (dec1_deg == -declim) {
			prev_x1 = x1;
			prev_y1 = y1;
			prev_pos1 = pos1;
		}

		/*
		 * In call 1, if either end is on the map and the other end is
		 * at least on the correct hemisphere, draw the dash.
		 */
		if (call == 1 && (pos1 == CP_ONMAP || pos2 == CP_ONMAP) &&
				pos1 != CP_NOTSET && pos2 != CP_NOTSET) {
			printf("%.2f %.2f moveto %.2f %.2f lineto stroke\n",
				x1, y1, x2, y2);
		}

		/*
		 * In call 2, if not the first dash, and if we crossed a
		 * boundary since thestart of the previous dash, print a number
		 * where appropriate.
		 */
		if (call == 2 && pos1 != prev_pos1 &&
				(pos1 == CP_ONMAP || prev_pos1 == CP_ONMAP) &&
				pos1 != CP_NOTSET && prev_pos1 != CP_NOTSET) {

			/* always check the left side */
			print_ra_left(x1, y1, prev_x1, prev_y1, ra_hour);

			/*
			 * Check either top or bottom, whichever side the lines
			 * are farthest apart on.
			 */
			if (Cen_dec_deg > 0.0) {
				print_ra_bottom(x1, y1, prev_x1, prev_y1,
					ra_hour);
			} else {
				print_ra_top(x1, y1, prev_x1, prev_y1, ra_hour);
			}
		}

		prev_x1 = x1;
		prev_y1 = y1;
		prev_pos1 = pos1;
	}
}



/*
 * Function:	print_grid_center_decline
 *
 * Description:	For call == 1, this function draws one declination line
 *		for a centered map.  For call == 2, it prints numbers where it
 *		crosses a map edge.  Because these lines may be curved in
 *		various ways, we can't use PostScript's "dashed" option to
 *		draw the line in a single command.  We draw individual dashes
 *		as short, straight lines, indistinguishable from curved lines.
 *		But for colored maps, we join the dashes to make a solid line.
 *
 * Returns:	void
 */

void
print_grid_center_decline(

int call,		/* draw line if 1, print numbers if 2 */
float dec_deg,		/* declination of this line */
int level		/* which level of line this is */
)
{
	/*
	 * The "1" variables are for the start of a dash.
	 * The "2" variables are for the end of a dash.
	 * The "prev" variables are for the start of the previous dash.
	 */
	float ra1_hour, ra2_hour;		/* right ascension */
	float x1, x2, prev_x1;			/* x coord */
	float y1, y2, prev_y1;			/* y coord */
	enum CENTERPOS pos1, pos2, prev_pos1;	/* position on or off map */

	float dist;		/* dist between starts neighboring of dashes */
	float len;		/* length of a dash */


	/*
	 * Make the distance between the starts of neighboring dashes and the
	 * length of each dash proportional to the scale of degrees, so that
	 * the actual printed distances end up the same, regardless of scale.
	 * We have to divide by cosine of declination because the RA lines
	 * get closer together toward the poles.
	 */
	dist = DEG2HOUR(1.5) * SCALE / cos(DEG2RAD(dec_deg));

	if (Black_and_white_map) {
		len = 0.75 * SCALE;
		len = DEG2HOUR(0.75) * SCALE / cos(DEG2RAD(dec_deg));
	} else {
		/* close to the pole we need small steps to keep circle round */
		if (dec_deg < -80.0 || dec_deg > 80.0) {
			dist /= 5.0;
		}
		len = dist;		/* join dashes into a solid line */
	}

	/*
	 * Loop once for each dash we might draw.  Wrap around a little, to
	 * ensure that if call == 2 we will be sure to print the number if the
	 * 0h position is near the margin.  But we will avoid actually drawing
	 * a dash beyond the wrap point, since it could merge with the first
	 * dash, and look bad.
	 */
	for (ra1_hour = 0.0; ra1_hour <= 24.0 + dist; ra1_hour += dist) {

		ra2_hour = ra1_hour + len;	/* get other end of dash */

		/* get x,y of dash's endpoints */
		pos1 = center_xy(ra1_hour, dec_deg, &x1, &y1);
		pos2 = center_xy(ra2_hour, dec_deg, &x2, &y2);

		/* if first loop iter, set prev to be the current */
		if (ra1_hour == 0.0) {
			prev_x1 = x1;
			prev_y1 = y1;
			prev_pos1 = pos1;
		}

		/*
		 * In call 1, if either end is on the map and the other end is
		 * at least on the correct hemisphere, draw the dash.  But not
		 * if we are beyond the wrap point.
		 */
		if (call == 1 && (pos1 == CP_ONMAP || pos2 == CP_ONMAP) &&
				pos1 != CP_NOTSET && pos2 != CP_NOTSET &&
				ra1_hour < 24.0) {
			printf("%.2f %.2f moveto %.2f %.2f lineto stroke\n",
				x1, y1, x2, y2);
		}

		/*
		 * In call 2, if not the first dash, and if we crossed a
		 * boundary since thestart of the previous dash, print a number
		 * where appropriate.
		 */
		if (call == 2 && pos1 != prev_pos1 &&
				(pos1 == CP_ONMAP || prev_pos1 == CP_ONMAP) &&
				pos1 != CP_NOTSET && prev_pos1 != CP_NOTSET) {

			/* always check the right side */
			print_dec_right(x1, y1, prev_x1, prev_y1, dec_deg);

			/*
			 * Check either top or bottom, whichever side the lines
			 * are closest together on.
			 */
			if (Cen_dec_deg > 0.0) {
				print_dec_top(x1, y1, prev_x1, prev_y1,
					dec_deg);
			} else {
				print_dec_bottom(x1, y1, prev_x1,prev_y1,
					dec_deg);
			}
		}

		prev_x1 = x1;
		prev_y1 = y1;
		prev_pos1 = pos1;
	}
}



/*
 * Function:	set_grid_appearance
 *
 * Description:	Set how the grid lines should look for centered maps.
 *
 * Returns:	void
 */

void
set_grid_appearance(

int level		/* which level of line this is */
)
{
	if (Black_and_white_map) {
		/* not using color, set the darkness of the dashed lines */
		switch (level) {
		case 1:
			printf("%.2f setgray\n", GETGRAY(0.45));
			break;
		case 2:
			printf("%.2f setgray\n", GETGRAY(0.70));
			break;
		case 3:
			printf("%.2f setgray\n", GETGRAY(0.80));
			break;
		}
	} else {
		/* using color, set the amount of color for the solid lines */
		switch (level) {
		case 1:
			if (White_background) {
				printf("0.1 1 0.1 setrgbcolor\n");
			} else {
				printf("0 0.6 0 setrgbcolor\n");
			}
			break;
		case 2:
			if (White_background) {
				printf("0.7 1 0.7 setrgbcolor\n");
			} else {
				printf("0 0.4 0 setrgbcolor\n");
			}
			break;
		case 3:
			if (White_background) {
				printf("0.85 1 0.85 setrgbcolor\n");
			} else {
				printf("0 0.2 0 setrgbcolor\n");
			}
			break;
		}
	}
}



/*
 * Function:	print_grid_numbers
 *
 * Description:	For standard (non-centered) maps, print right ascension and
 *		declination numbers.  (For centered maps, print_grid does this.)
 *
 * Returns:	void
 */

void
print_grid_numbers()
{
	printf("gsave\n");

	printf("/Times-Roman findfont\n");
	printf("12 scalefont setfont\n");

	if (CIRCUMPOLAR) {
		print_grid_numbers_polar();
	} else {
		print_grid_numbers_mid();
	}

	printf("grestore\n");
}



/*
 * Function:	print_grid_numbers_polar
 *
 * Description:	For the circumpolar maps, print right ascension and declination
 *		numbers.
 *
 * Returns:	void
 */

void
print_grid_numbers_polar()
{
	int ra_hour;		/* right ascension */
	int dec_deg;		/* declination */


	/* move to center of page */
	printf("%.2f %.2f translate\n", MID_X, MID_Y);

	/* print RA numbers every 2 hours around outside of the circle */
	for (ra_hour = 0; ra_hour < 24; ra_hour += 2) {
		printf("-9 %.2f moveto (%dh) show\n",
				DEC_DEG2R(35) + 2.0, ra_hour);
		printf("%d rotate\n", NORTH ? -30 : 30);
	}

	/* print declination numbers every 10 degrees along the RA=0 line */
	for (dec_deg = 80; dec_deg >= 40; dec_deg -= 10) {
		if (NORTH) {
			printf("-20 %.2f moveto (+%d) show\n",
					DEC_DEG2R(dec_deg) - 9.0, dec_deg);
		} else {
			printf("-17 %.2f moveto (-%d) show\n",
					DEC_DEG2R(dec_deg) - 9.0, dec_deg);
		}
	}
}



/*
 * Function:	print_grid_numbers_mid
 *
 * Description:	For the midsection maps, print right ascension and declination
 *		numbers.
 *
 * Returns:	void
 */

void
print_grid_numbers_mid()
{
	int ra_hour;		/* right ascension */
	int dec_deg;		/* declination */
	char buf[10];		/* holding place for string to print */
	float shift;
	float x, y;


	/* print RA numbers every hour at top and bottom */
	for (ra_hour = 0; ra_hour <= 24 + 6; ra_hour++) { /* 6 for wrap around*/
		sprintf(buf, "%2d", ra_hour % 24);
		printf("%.2f %2f moveto (%sh) show\n",
				RA_HOUR2X(ra_hour) - 9, Max_y + 2, buf);
		printf("%.2f %2f moveto (%sh) show\n",
				RA_HOUR2X(ra_hour) - 9, Min_y - 10, buf);
	}


	printf("0.4 setlinewidth\n");	/* for drawing degree symbol */

	/*
	 * Print declination numbers every 10 degrees along right or left,
	 * depending on which map this is.
	 */
	for (dec_deg = -60; dec_deg <= 60; dec_deg += 10) {
		if (dec_deg > 0) {
			sprintf(buf, "%c", '+');
			shift = 0.0;
		} else if (dec_deg < 0) {
			sprintf(buf, "%c", '-');
			shift = 2.5;
		} else if (dec_deg == 0) {
			buf[0] = '\0';
			shift = 12.0;
		}
		sprintf(buf + strlen(buf), "%d", abs(dec_deg));

		y = DEC_DEG2Y(dec_deg) - 3;

		if (Map == 3 || Map == 5) {
			/* print along right side */
			x = Max_x + shift;

			/* print number and degree symbol (circle) */
			printf("%.2f %2f moveto (%s) show\n", x, y, buf);
			printf("newpath %.2f %.2f %.2f 0 360 arc "
					"stroke closepath\n",
					x + 21.5 - shift, y + 7.0, 2.0);
		} else {
			/* print along left side */
			x = MIN_X + shift;

			/* print number and degree symbol (circle) */
			printf("%.2f %2f moveto (%s) show\n", x, y, buf);
			printf("newpath %.2f %.2f %.2f 0 360 arc "
					"stroke closepath\n",
					x + 21.5 - shift, y + 7.0, 2.0);
		}
	}
}



/*
 * Function:	print_ecliptic
 *
 * Description:	Draw the ecliptic.  It's done as a series of 1/2 degree long
 *		straight line segments.
 *
 * Returns:	void
 */

void
print_ecliptic()
{
	if (CIRCUMPOLAR) {
		/* the ecliptic never gets this far north or south */
		return;
	}

	printf("%% ecliptic\n");
	printf("gsave\n");

	/*
	 * In both subroutines, we loop through points along the ecliptic.
	 * We figure out the RA and dec as follows.  Assume the sky is a
	 * sphere of radius 1, we are at the origin O, P is our point, and
	 * V is the vernal equinox point.  Let B be the point you get by
	 * by dropping a perpendicular from P to the equatorial place, and A
	 * is the point you get by dropping a perpendicular from B to OV.
	 * The length of OP is 1.  Angle AOP is lambda.  So OA is cos lambda
	 * and AP is sin lambda.  Let epsilon be angle PAB, the obliquity of
	 * the ecliptic.  ABP is a right angle, so BP is sin lambda sin epsilon
	 * and AB is sin lambda cos epsilon.  AOB, the right ascension, has
	 * a tangent of tan lambda cos epsilon.  POB, the declination, has a
	 * sine of sin lambda sin epsilon.
	 */

	if (CENTER) {
		print_ecliptic_center();
	} else {
		print_ecliptic_mid();
	}

	printf("grestore\n");
}



/*
 * Function:	print_ecliptic_mid
 *
 * Description:	Draw the ecliptic for a midsection map.
 *
 * Returns:	void
 */

void
print_ecliptic_mid()
{
	float ra_rad, dec_rad;		/* for a point on the ecliptic */
	float start_deg;		/* starting point (on right edge) */
	float inc_deg;			/* distance from starting point */
	float lambda_deg, lambda_rad;	/* ecliptic longitude */
	float x, y;			/* screen coords of an ecliptic point */


	/* set up right endpoint; treat maps 3-4 as one map, also 5-6 */
	if (Map == 3 || Map == 4) {
		x = RA_HOUR2X(0.0);
		y = DEC_DEG2Y(0.0);
		start_deg = 0;
	} else {
		x = RA_HOUR2X(12.0);
		y = DEC_DEG2Y(0.0);
		start_deg = 180;
	}

	printf("newpath %.2f %.2f moveto\n", x, y);

	/*
	 * For every 1/2 degree around the ecliptic, working left from the
	 * start point to the left edge of these two combined maps, find that
	 * point's RA and declination.  Draw a straight line from the previous
	 * point to this point.  The left edge is about 227.5 degrees from the
	 * start, as measured along the ecliptic.
	 */
	for (inc_deg = 1; inc_deg <= 227.51; inc_deg += 0.5) {
		lambda_deg = start_deg + inc_deg;
		lambda_rad = DEG2RAD(lambda_deg);
		ra_rad = atan2(sin(lambda_rad) * Cos_epsilon, cos(lambda_rad));
		if (ra_rad < 0.0) {
			ra_rad += 2 * PI;
		}
		if ((Map == 5 || Map == 6) && ra_rad < PI / 2.0) {
			ra_rad += 2 * PI;
		}
		dec_rad = asin(sin(lambda_rad) * Sin_epsilon);
		x = RA_HOUR2X(RAD2HOUR(ra_rad));
		y = DEC_DEG2Y(RAD2DEG(dec_rad));
		printf("%.2f %.2f lineto\n", x, y);
	}

	printf("stroke\n");
}



/*
 * Function:	print_ecliptic_center
 *
 * Description:	Draw the ecliptic for a centered map.
 *
 * Returns:	void
 */

void
print_ecliptic_center()
{
	float ra_hour, dec_deg;		/* for a point on the ecliptic */
	float ra_rad, dec_rad;		/* for a point on the ecliptic */
	float lambda_deg, lambda_rad;	/* ecliptic longitude */
	float x1, y1, x2, y2;		/* screen coords of 2 ecliptic points */
	enum CENTERPOS pos1, pos2;	/* screen position of the 2 points */


	/* set x,y of starting point: vernal equinox */
	ra_hour = 0.0;
	dec_deg = 0.0;
	pos1 = center_xy(ra_hour, dec_deg, &x1, &y1);

	/*
	 * For every 1/2 degree around the ecliptic, working left from the
	 * start point all the way around back to the start, find that point's
	 * RA and declination.  Draw a straight line from the previous point to
	 * this point.
	 */
	for (lambda_deg = 0.5; lambda_deg <= 360.01; lambda_deg += 0.5) {
		/* get x,y of end of this segment */
		lambda_rad = DEG2RAD(lambda_deg);
		ra_rad = atan2(sin(lambda_rad) * Cos_epsilon, cos(lambda_rad));
		if (ra_rad < 0.0) {
			ra_rad += 2 * PI;
		}
		dec_rad = asin(sin(lambda_rad) * Sin_epsilon);
		ra_hour = RAD2HOUR(ra_rad);
		dec_deg = RAD2DEG(dec_rad);
		pos2 = center_xy(ra_hour, dec_deg, &x2, &y2);

		/* if prev point and this point in this hemisphere, draw line */
		if (pos1 != CP_NOTSET && pos2 != CP_NOTSET) {
			printf("newpath %.2f %.2f moveto\n", x1, y1);
			printf("%.2f %.2f lineto stroke\n", x2, y2);
		}

		/* copy this point to prev point */
		x1 = x2;
		y1 = y2;
		pos1 = pos2;
	}
}



/*
 * Function:	print_galactic
 *
 * Description:	Draw the ecliptic equator.  It's done as a series of 1/2 degree
 *		long straight line segments.
 *
 * Returns:	void
 */
#define TILT_DEG	(27.4)		/* 90 - angle with equator */

void
print_galactic()
{
	float l_123_rad, l_123_deg;	/* galactic latitude minus 123 degs */
	float ra_rad, ra_hour;		/* right ascension of a point on equ.*/
	float dec_rad, dec_deg;		/* declination of a point on equator */
	float sin_27p4, cos_27p4;	/* sin and cos of 27.4 degrees */
	float prev_ra_hour;		/* RA of previous point */
	bool first_iter;		/* first iteration of loop? */
	float x1, y1, x2, y2;		/* coords on the page for 2 points */
	enum CENTERPOS pos1, pos2;	/* position of point on the map */
	float r;			/* distance from pole */
	float theta_deg;		/* angle at the pole */


	/*
	 * This algorithm was taken from these formulas at
	 * http://en.wikipedia.org/wiki/Celestial_coordinate_system as follows:
	 *
	 * alpha = arctan( sin(L - 123 deg) /
	 *  ( cos(L - 123 deg) sin 27.4 deg - tan b cos 27.4 deg ) ) + 12.25 deg
	 * sin delta = sin b sin 27.4 deg + cos b cos 27.4 deg cos(L - 123 deg)
	 *
	 * alpha is RA, delta is declination, L is galactic longitude, b is
	 * galactic latitude.  At the equator, b is zero, so we set b to zero
	 * and calculate a point at every 0.5 degree of L.  But we start at
	 * (L - 123 deg) = 0.25 so that it will never hit 90 or 270, and we
	 * don't divide by the cosine of 90 or 270.
	 *
	 * However, these equations are for 1950.  So we temporarily add 50 to
	 * Precess_year so that we precess 50 more years ahead.
	 */

	Precess_year += 50.0;

	printf("%% galactic equator\n");

	printf("gsave\n");

	if (CIRCUMPOLAR) {
		printf("%.2f %.2f translate\n", MID_X, MID_Y);
	}

	/* we keep using these, so just calculate them once */
	sin_27p4 = sin(DEG2RAD(27.4));
	cos_27p4 = cos(DEG2RAD(27.4));

	pos1 = pos2 = CP_ONMAP;		/* default to this */
	first_iter = true;		/* first loop iteration is coming */

	for (l_123_deg = 0.25; l_123_deg < 360.5; l_123_deg += 0.5) {
		l_123_rad = DEG2RAD(l_123_deg);
		ra_rad = atan(sin(l_123_rad) / (cos(l_123_rad) * sin_27p4))
				+ DEG2RAD(12.25);
		dec_rad = asin(cos_27p4 * cos(l_123_rad));
		ra_hour = RAD2HOUR(ra_rad);
		dec_deg = RAD2DEG(dec_rad);

		/*
		 * Due to quadrant issues, RA jumps back 12 hours at certain
		 * places, whereas for each new point it should increase
		 * slightly.  So add 12 as necessary to keep it increasing.
		 */
		if ( ! first_iter ) {
			while (ra_hour < prev_ra_hour) {
				ra_hour += 12.0;
			}
		}

		/*
		 * Remember this as the previously calculated RA.  But then
		 * normalize it as less than 24, and precess the RA and dec.
		 */
		prev_ra_hour = ra_hour;
		if (ra_hour > 24.0) {
			ra_hour -= 24.0;
		}
		precess(&ra_hour, &dec_deg);

		/*
		 * Find the x, y, and pos for this point, based on which map.
		 */
		if (CENTER) {
			pos2 = center_xy(ra_hour, dec_deg, &x2, &y2);
		} else if (CIRCUMPOLAR) {
			if (NORTH && dec_deg < 0.0 || SOUTH && dec_deg > 0.0) {
				pos2 = CP_NOTSET;  /* wrong half of the sky */
			} else {
				pos2 = CP_ONMAP;
				/* convert RA to our coord system's theta */
				if (NORTH) {
					theta_deg = 90.0 - HOUR2DEG(ra_hour);
				} else {
					theta_deg = 90.0 + HOUR2DEG(ra_hour);
				}
				r = DEC_DEG2R(dec_deg);
				x2 = r * cos(DEG2RAD(theta_deg));
				y2 = r * sin(DEG2RAD(theta_deg));
			}
		} else {	/* MIDSECTION */
			/* allow for wraparound on map 6 */
			if (Map == 6 && ra_hour < 12.0) {
				ra_hour += 24.0;
			}
			x2 = RA_HOUR2X(ra_hour);
			y2 = DEC_DEG2Y(dec_deg);
		}

		if (first_iter) {
			first_iter = false;
		} else {
			/*
			 * Draw the line if the coords are valid.  Also guard
			 * against a case where a wraparound causes a long
			 * line across the page.
			 */
			if (pos1 != CP_NOTSET && pos2 != CP_NOTSET &&
					fabs(x1 - x2) < 500.0) {
				printf("newpath ");
				printf("%.2f %.2f moveto\n", x1, y1);
				printf("%.2f %.2f lineto stroke\n", x2, y2);
			}
		}

		/* remember this point as being the previous point */
		x1 = x2;
		y1 = y2;
		pos1 = pos2;
	}

	printf("grestore\n");

	Precess_year -= 50.0;	/* restore the correct precession */
}



/*
 * Function:	print_horizon
 *
 * Description:	Draw the horizon.  It is based on the zenith coordinates set by
 *		the -z option.  The algorithm is based on the ecliptic's
 *		algorithm.
 *
 * Returns:	void
 */

void
print_horizon()
{
	float ra_offset_hour;		/* RA where horizon crosses equator*/
	float cos_tilt, sin_tilt;	/* inclination of horizon to equator */
	float ra_hour, dec_deg;		/* for a point on the horizon */
	float ra_rad, dec_rad;		/* for a point on the horizon */
	float lambda_deg, lambda_rad;	/* "horizon longitude" */
	float x1, y1, x2, y2;		/* screen coords of 2 equator points */
	enum CENTERPOS pos1, pos2;	/* screen position of the 2 points */


	printf("%% horizon\n");
	printf("gsave\n");
	printf("1.2 setlinewidth\n");

	/* find the RA where the horizon crosses the equator */
	ra_offset_hour = Zenith_ra_hour + 6.0;
	if (ra_offset_hour >= 24.0) {
		ra_offset_hour -= 24.0;
	}

	cos_tilt = cos(DEG2RAD(90.0 - Zenith_dec_deg));
	sin_tilt = sin(DEG2RAD(90.0 - Zenith_dec_deg));

	/* set x,y of starting point: where horizon crosses equator */
	pos1 = center_xy(ra_offset_hour, 0.0, &x1, &y1);

	/*
	 * For every 1/2 degree around the horizon, working left from the
	 * start point all the way around back to the start, find that point's
	 * RA and declination.  Draw a straight line from the previous point to
	 * this point.
	 */
	for (lambda_deg = 0.5; lambda_deg <= 360.01; lambda_deg += 0.5) {
		/* get x,y of end of this segment */
		lambda_rad = DEG2RAD(lambda_deg);
		ra_rad = atan2(sin(lambda_rad) * cos_tilt, cos(lambda_rad));
		if (ra_rad < 0.0) {
			ra_rad += 2 * PI;
		}
		dec_rad = asin(sin(lambda_rad) * sin_tilt);

		/*
		 * ra_rad is for as if horizon crossed equator at RA 0h.  We
		 * add the offset, to rotate it to the correct place.
		 */
		ra_hour = RAD2HOUR(ra_rad) + ra_offset_hour;
		if (ra_hour >= 24.0) {
			ra_hour -= 24.0;
		}
		dec_deg = RAD2DEG(dec_rad);
		pos2 = center_xy(ra_hour, dec_deg, &x2, &y2);

		/* if prev point and this point in this hemisphere, draw line */
		if (pos1 != CP_NOTSET && pos2 != CP_NOTSET) {
			printf("newpath %.2f %.2f moveto\n", x1, y1);
			printf("%.2f %.2f lineto stroke\n", x2, y2);
		}

		/* copy this point to prev point */
		x1 = x2;
		y1 = y2;
		pos1 = pos2;
	}

	printf("grestore\n");
}



/*
 * Function:	set_star_coords
 *
 * Description:	Store the x, y coordinates for a star.  This is not used for
 *		the circumpolar maps.
 *
 * Returns:	void
 */

void
set_star_coords(

int sidx		/* star index number */
)
{
	float ra_hour, dec_deg;


	/* get the RA and dec from the array of stars */
	ra_hour = Stardata[sidx].ra_hour;
	dec_deg = Stardata[sidx].dec_deg;

	if (CENTER) {
		Stardata[sidx].centerpos = center_xy(ra_hour, dec_deg,
				&Stardata[sidx].x, &Stardata[sidx].y);
		return;
	}

	/* adjust for wraparound on first and last midsection maps */
	if (Map == 6 && ra_hour < 6.0) {
		ra_hour += 24.0;
	} else if (Map == 3 && ra_hour > 18.0) {
		ra_hour -= 24.0;
	}

	Stardata[sidx].x = RA_HOUR2X(ra_hour);
	Stardata[sidx].y = DEC_DEG2Y(dec_deg);
}



/*
 * Function:	print_stars
 *
 * Description:	Draw stars that have positive radius (meaning the ones not
 *		fainter than the magnitude limit), along with name and/or
 *		letter, or whatever was requested to be printed.
 *
 * Returns:	void
 */

void
print_stars()
{
	int sidx;	/* star index number */


	printf("%% stars\n");
	printf("%.2f setgray\n", GETGRAY(0.0));

	/* for every object to be printed, call a subroutine to do it */
	for (sidx = 1; sidx <= Num_objects; sidx++) {
		if (Stardata[sidx].radius > 0.0) {
			if (CIRCUMPOLAR) {
				print_star_polar(sidx);
			} else if (MIDSECTION) {
				print_star_mid(sidx);
			} else {
				print_star_center(sidx);
			}
		}
	}
}



/*
 * Function:	print_star_polar
 *
 * Description:	Draw a star and associated name/letter for a polar map.
 *
 * Returns:	void
 */

void
print_star_polar(

int sidx		/* star index number */
)
{
	float r, theta_deg;	/* polar coords of star (in points, degrees) */
	float ra_hour, dec_deg;	/* right ascension and declination */
	float radius;		/* radius of the star */


	dec_deg = Stardata[sidx].dec_deg;

	if (NORTH && dec_deg < 35.0 || SOUTH && dec_deg > -35.0) {
		/* center of star is too far from pole to be on this map */
		return;
	}

	ra_hour = Stardata[sidx].ra_hour;
	theta_deg = HOUR2DEG(ra_hour);
	if (NORTH) {
		/* RA is clockwise on north map, so need to reverse it */
		theta_deg = -theta_deg;
	}

	r = DEC_DEG2R(dec_deg);		/* get points from the pole */

	radius = Stardata[sidx].radius;

	printf("gsave\n");

	/* move to center of page, rotate so star is on positive Y axis */
	printf("%.2f %.2f translate\n", MID_X, MID_Y);
	printf("%.2f rotate\n", theta_deg);

	/* draw the star on the Y axis, r points above the pole */
	print_star_itself(sidx, 0.0, r);

	/* if name and letter are requested, print them */
	if (Print_star_namelet) {
		print_star_bayer_flam(sidx, 0.0, r);
		print_star_proper(sidx, 0.0, r);
	}

	printf("grestore\n");
}



/*
 * Function:	print_star_mid
 *
 * Description:	Draw a star and associated name/letter for a midsection map.
 *
 * Returns:	void
 */

void
print_star_mid(

int sidx		/* star index number */
)
{
	float x, y;		/* coords in points */


	/* copy for convenience */
	x = Stardata[sidx].x;
	y = Stardata[sidx].y;

	/* if vertically outside of the limits, don't print */
	if (y < Min_y || y > Max_y) {
		return;
	}

	/*
	 * If horizontally beyond the limit on the side that has RA numbers,
	 * don't print; we allow "overflow" on the other side.
	 */
	if ((Map == 3 || Map == 5) && x > Max_x) {
		return;
	}
	if ((Map == 4 || Map == 6) && x < Min_x) {
		return;
	}

	/* draw the actual star */
	print_star_itself(sidx, x, y);

	/* if name and letter are requested, print them */
	if (Print_star_namelet) {
		print_star_bayer_flam(sidx, x, y);
		print_star_proper(sidx, x, y);
	}
}



/*
 * Function:	print_star_center
 *
 * Description:	Draw a star and associated name/letter for a centered map.
 *
 * Returns:	void
 */

void
print_star_center(int sidx)
{
	float x, y;		/* coords in points */


	/* copy for convenience */
	x = Stardata[sidx].x;
	y = Stardata[sidx].y;

	/* if horizontally outside of the limits, don't print */
	if (x < Min_x || x > Max_x) {
		return;
	}

	/* if vertically outside of the limits, don't print */
	if (y < Min_y || y > Max_y) {
		return;
	}

	/* draw the actual star */
	print_star_itself(sidx, x, y);

	/* if name and letter are requested, print them */
	if (Print_star_namelet) {
		print_star_bayer_flam(sidx, x, y);
		print_star_proper(sidx, x, y);
	}
}



/*
 * Function:	print_star_itself
 *
 * Description:	Draw a star.
 *
 * Returns:	void
 */

void
print_star_itself(

int sidx,		/* star index number */
float x,		/* in points */
float y			/* in points */
)
{
	/* print a white circle a little bigger than the star */
	printf("%.2f setgray\n", GETGRAY(1.0));
	printf("newpath %.2f %.2f %.2f 0 360 arc fill closepath %% sidx %d\n",
			x, y, Stardata[sidx].radius + 0.7, sidx);

	/* print the star */
	printf("%.2f setgray\n", GETGRAY(0.0));
	printf("newpath %.2f %.2f %.2f 0 360 arc fill closepath %% sidx %d\n",
			x, y, Stardata[sidx].radius, sidx);
}



/*
 * Function:	print_star_bayer_flam
 *
 * Description:	Print the Bayer letter and/or Flamsteed number for a star, if
 *		the magnitude cutoff allows.
 *
 * Returns:	void
 */

void
print_star_bayer_flam(

int sidx,		/* star index number */
float star_x,		/* x position of star, in points */
float star_y		/* y position of star, in points */
)
{
	float xoff, yoff;	/* point offsets at which to print */
	float shift;		/* amount to alter offsets due to mag limit */
	char desigbuf[20];	/* what we are going to print */
	bool bflag;		/* remember whether we used bayer, when flam */


	/* don't print if fainter than the limit */
	if (Stardata[sidx].mag > Bf_limit) {
		return;
	}

	/*
	 * Populate the buffer with bayer and/or flamsteed as requested.
	 */
	bflag = false;
	switch (Designation) {
	case DES_BAYER:
		strcpy(desigbuf, Stardata[sidx].bayer);
		break;
	case DES_FLAM:
		strcpy(desigbuf, Stardata[sidx].flamsteed);
		break;
	case DES_BAYER_OR_FLAM:
		if (Stardata[sidx].bayer[0] == '\0') {
			strcpy(desigbuf, Stardata[sidx].flamsteed);
		} else {
			strcpy(desigbuf, Stardata[sidx].bayer);
			bflag = true;
		}
		break;
	case DES_BAYER_AND_FLAM:
		sprintf(desigbuf, "%s%s", Stardata[sidx].flamsteed,
				Stardata[sidx].bayer);
		bflag = true;
		break;
	case DES_NEITHER:
		desigbuf[0] = '\0';
		break;
	}

	/* if nothing to print, get out */
	if (desigbuf[0] == '\0') {
		return;
	}

	printf("/Symbol findfont\n");
	printf("10 scalefont setfont\n");

	/* see if default position is being overridden for the relevant map */
	if (Stardata[sidx].bayerpos[star_midx(sidx)].xoff != 0.0 ||
	    Stardata[sidx].bayerpos[star_midx(sidx)].yoff != 0.0) {
		/*
		 * Normally, use the override positions from the file.  But if
		 * the star's radius has been increased due to a fainter
		 * magnitude cutoff, alter these numbers.
		 */
		xoff = Stardata[sidx].bayerpos[star_midx(sidx)].xoff;
		yoff = Stardata[sidx].bayerpos[star_midx(sidx)].yoff;
		if (Mag_limit > MAG_SCALE_LIMIT) {
			shift = (Mag_limit - MAG_SCALE_LIMIT) * 0.8;
			if (xoff > 0.0) {
				xoff += shift;
			} else {
				xoff -= shift;
			}
			if (yoff > 0.0) {
				yoff += shift;
			} else {
				yoff -= shift;
			}
		}

		/* move to the resulting starting place */
		printf("%.2f %.2f moveto\n", star_x + xoff, star_y + yoff);

		/*
		 * For cases where a Flamsteed number is present, even though
		 * this is not the default way to print the map and we make no
		 * guarentees against collisions, we will try to allow for the
		 * fact that the string may be a different length than was
		 * anticipated by the bayerpos file.  And so strings that start
		 * on the left side need to start at a different place, to end
		 * just before the star.
		 */
		if ( ! isalpha(desigbuf[0]) &&
		    Stardata[sidx].bayerpos[star_midx(sidx)].xoff < 0.0) {

			if (bflag) {
				/*
				 * There are both bayer and flamsteed.  Move
				 * left by the width of the flamsteed.
				 */
				printf("(%s) (%s) stringwidth pop neg 0 rmoveto show\n",
					desigbuf, Stardata[sidx].flamsteed);
			} else {
				/*
				 * There is only flamsteed.  Move left by its
				 * width, and right by the width of the bayer
				 * (which could be zero).
				 */
				printf("(%s) (%s) stringwidth pop neg 0 rmoveto (%s) stringwidth pop 0 rmoveto show\n",
					desigbuf, Stardata[sidx].flamsteed,
					Stardata[sidx].bayer);
			}
		} else {
			/* no flamsteed, just print at the normal place */
			printf("(%s) show\n", desigbuf);
		}
	} else {
		/* use the default rules */
		if (proper_print(sidx)[0] != '\0') {
			/* proper name or other info is present */
			/* put bayer on the left */
			printf("%.2f %.2f moveto\n",
				star_x - Stardata[sidx].radius - 2.0,
				star_y - 3.0);
			printf("(%s) dup stringwidth pop neg 0 rmoveto show\n",
				desigbuf);
		} else {
			/* proper name or other info is not present */
			/* put bayer on the right */
			printf("%.2f %.2f moveto\n",
				star_x + Stardata[sidx].radius + 2.0,
				star_y - 3.0);
			printf("(%s) show\n", desigbuf);
		}
	}
}



/*
 * Function:	print_star_proper
 *
 * Description:	Print the proper name for a star, if the magnitude cutoff
 *		allows.  This may actually be the magnitude or star number, if
 *		the user requested.
 *
 * Returns:	void
 */

void
print_star_proper(

int sidx,		/* star index number */
float star_x,		/* x position of star, in points */
float star_y		/* y position of star, in points */
)
{
	float xoff, yoff;	/* point offsets at which to print */
	float shift;		/* amount to alter offsets due to mag limit */
	char *propinfo;		/* proper name or other info to print */


	/* get what is to be printed */
	propinfo = proper_print(sidx);

	/* if nothing to print, return */
	if (propinfo[0] == '\0') {
		return;
	}

	/* for magnitude/starnumber use italic, to distinguish from flamsteed */
	if (Print_mag || Print_starnum) {
		printf("/Times-Italic findfont\n");
	} else {
		printf("/Times-Roman findfont\n");
	}
	printf("10 scalefont setfont\n");

	/* see if default position is being overridden for the relevant map */
	if (Stardata[sidx].properpos[star_midx(sidx)].xoff != 0.0 ||
	    Stardata[sidx].properpos[star_midx(sidx)].yoff != 0.0) {
		/*
		 * Normally, use the override positions from the file.  But if
		 * the star's radius has been increased due to a fainter
		 * magnitude cutoff, alter these numbers.
		 */
		xoff = Stardata[sidx].properpos[star_midx(sidx)].xoff;
		yoff = Stardata[sidx].properpos[star_midx(sidx)].yoff;
		if (Mag_limit > MAG_SCALE_LIMIT) {
			shift = (Mag_limit - MAG_SCALE_LIMIT) * 0.8;
			if (xoff > 0.0) {
				xoff += shift;
			} else {
				xoff -= shift;
			}
			if (yoff > 0.0) {
				yoff += shift;
			} else {
				yoff -= shift;
			}
		}
		printf("%.2f %.2f moveto\n", star_x + xoff, star_y + yoff);
	} else {
		/* use the default rule; print on the right */
		printf("%.2f %.2f moveto\n",
			star_x + Stardata[sidx].radius + 2.0,
			star_y - 3.0);
	}

	printf("(%s) show\n", propinfo);
}



/*
 * Function:	proper_print
 *
 * Description:	Find what string should be printed as the proper name of an
 *		object.  Usually it is the real proper name, or \0 if there
 *		isn't one.  But based on the type of map and screen width and
 *		-s option, sometimes we print nothing.  And the command line
 *		can override what should be printed.
 *
 * Returns:	pointer to a static buffer containing the answer
 */

char *
proper_print(

int sidx
)
{
	static char propbuf[MAX_PROPNAME + 1];
	char *prop_ptr;


	/* default to printing the proper name */
	prop_ptr = Stardata[sidx].propname;

	/* if this is an object from the -a option, always print the name */
	if (sidx > Num_stars) {
		strcpy(propbuf, prop_ptr);
		return propbuf;
	}

	/* never print star names if too dim for the limiting magnitude */
	if (Stardata[sidx].mag > Prop_limit) {
		return "";
	}

	/* we would like to print the name, but check whether we shouldn't */
	if (CENTER) {
		/*
		 * For centered maps, if a width limit was set, discard the
		 * proper name if the map covers too wide an area to print the
		 * name without colliding with things.
		 */
		if (Stardata[sidx].width_limit != 0 &&
				Cen_wid_deg > Stardata[sidx].width_limit) {
			prop_ptr = "";
		}
	} else {
		/*
		 * For standard maps, if a width limit was set, discard the
		 * proper name always.
		 */
		if (Stardata[sidx].width_limit != 0) {
			prop_ptr = "";
		}
	}

	if (Print_mag) {
		/* print magnitude times 10 instead of proper name */
		sprintf(propbuf, "%.0f", Stardata[sidx].mag * 10);
	} else if (Print_starnum) {
		/* print star number instead of proper name */
		sprintf(propbuf, "%d", sidx);
	} else {
		/* print proper name, might be \0 */
		strcpy(propbuf, prop_ptr);
	}

	return propbuf;
}



/*
 * Function:	star_midx
 *
 * Description:	Find what map index should be used for the given star.  Map
 *		index 0 is used for maps 1 and 2, index 1 is used for maps
 *		3 and 4, 2 for 5 and 6.
 *
 * Returns:	map index, 0 to 2
 */

int
star_midx(

int sidx		/* star index number */
)
{
	return best_midx(Stardata[sidx].ra_hour, Stardata[sidx].dec_deg);
}



/*
 * Function:	best_midx
 *
 * Description:	Find what map index is best to use for the given coordinates.
 *		If we are drawing a standard map (1 to 6), there is a
 *		definite answer, given by the MIDX macro.  But for centered
 *		maps, it's not obvious in the regions of the sky where the
 *		standard maps overlap.  We take our best shot at it.
 *
 * Returns:	map index, 0 to 2
 */

int
best_midx(

float ra_hour,		/* right ascension */
float dec_deg		/* declination */
)
{
	/* for fixed maps, the answer is obvious */
	if ( ! CENTER) {
		return MIDX;
	}

	/*
	 * For centered, we take our best shot at which one to use.  Use polar
	 * only for the part of that which doesn't overlap the midsection maps.
	 * That's because the way the name and letters are placed on the
	 * midsection maps will probably look better than the way they are on
	 * the polar maps.  The centered maps are displayed with north at the
	 * top, like midsection.
	 */
	if (dec_deg > 65.0 || dec_deg < -65.0) {
		return 0;
	}

	/*
	 * Split the overlap evenly between maps 3/4 and 5/6, to get the best
	 * placement of names/letters.
	 */
	if (ra_hour > 1.5 && ra_hour < 13.5) {
		return 1;
	}
	return 2;
}



/*
 * Function:	star_radius
 *
 * Description:	Find the radius of a star if it is to be printed.
 *
 * Returns:	radius, or zero if too faint to print
 */

float
star_radius(

float mag		/* magnitude of the star */
)
{
	float radius;		/* the answer */


	/* if the star is too faint, set to 0, meaning don't print it */
	if (mag > Mag_limit) {
		return 0.0;
	}

	/* optionally round it off to the nearest integer */
	mag = quantize(mag);

	/*
	 * If the magnitude cutoff is brighter than MAG_SCALE_LIMIT, use a
	 * fixed linear formula to decide the star's radius.  Otherwise, use a
	 * sliding scale so that the faintest stars never get too small.
	 */
	if (Mag_limit <= MAG_SCALE_LIMIT) {
		radius = -0.88 * mag + 5.36;
	} else {
		radius = -0.88 * (mag + MAG_SCALE_LIMIT - Mag_limit) + 5.36;
	}

	/* but we can't allow it to get too small */
	if (radius <= 0.5) {
		radius = 0.5;
	}

	return radius;
}



/*
 * Function:	quantize
 *
 * Description:	If "Quantize" is set, round a number to the nearest integer.
 *
 * Returns:	the resulting number
 */

float
quantize(

float f		/* the number to be rounded (if desired) */
)
{
	int sign;	/*  -1 if negative, else 1 */

	if ( ! Quantize) {
		return f;	/* quantizing is disabled */
	}

	if (f < 0.0) {
		f = -f;
		sign = -1;
	} else {
		sign = 1;
	}

	f = (int)(f + 0.5);

	return sign * f;
}



/*
 * Function:	print_boundaries
 *
 * Description:	Draw all constellation boundaries.  It reads the data defining
 *		the boundaries from BOUNDARY_FILE.
 *
 * Returns:	void
 */

void
print_boundaries()
{
	FILE *bd_p;		/* for reading boundary data file */


	bd_p = fopen(BOUNDARY_FILE, "r");
	if (bd_p == NULL) {
		error("can't read %s", BOUNDARY_FILE);
	}

	printf("%% print_boundaries\n");
	printf("gsave\n");

	if (Black_and_white_map) {
		printf("%.2f setgray\n", GETGRAY(GRAY_BOUND));
	} else {
		if (White_background) {
			printf("1 0.6 0.6 setrgbcolor\n");
		} else {
			printf("0.6 0 0 setrgbcolor\n");
		}
	}

	if (CIRCUMPOLAR) {
		print_boundaries_polar(bd_p);
	} else if (MIDSECTION) {
		print_boundaries_mid(bd_p);
	} else {
		print_boundaries_center(bd_p);
	}

	printf("grestore\n");

	fclose(bd_p);
}



/*
 * Function:	print_boundaries_polar
 *
 * Description:	Draw all constellation boundaries for a polar map.
 *
 * Returns:	void
 */

void
print_boundaries_polar(

FILE *bd_p		/* boundary data file */
)
{
	float r, theta_rad;	/* position in points, radians */
	float ra_hour, dec_deg;	/* for a point on a constellation boundary */
	char buf[80];		/* holds one line from the file */
	char curcon[10];	/* current constellation's abbreviation */
	char newcon[10];	/* new constellation's abbreviation */
	bool print;		/* should we print this constellation? */


	printf("gsave\n");

	/* move origin to the center of the map */
	printf("%.2f %.2f translate\n", MID_X, MID_Y);

	/* loop once for every line in the coord file */
	curcon[0] = '\0';	/* no current constellation yet */
	while (fgets(buf, sizeof (buf), bd_p) != NULL) {

		/* read a line representing a point; apply precession */
		sscanf(buf, "%f %f %s", &ra_hour, &dec_deg, newcon);
		precess(&ra_hour, &dec_deg);

		/* get RA in radians, but offset so zero will point up */
		theta_rad = DEG2RAD(HOUR2DEG(ra_hour) + (NORTH ? -90.0 : 90.0));
		if (NORTH) {
			theta_rad = -theta_rad;	/* wraps opposite way */
		}

		r = DEC_DEG2R(dec_deg);

		if (strcmp(curcon, newcon) != 0) {

			/* starting a new constellation */

			/* end previous constellation if there is one */
			if (curcon[0] != '\0') {
				if (print) {
					printf("closepath stroke\n");
				}
			}

			/* new one becomes the current one */
			strcpy(curcon, newcon);

			/*
			 * Print only constellations in the correct hemisphere.
			 * Also exclude Ursa Minor and Octans; their boundaries
			 * have problems due to the poles.  We can skip them,
			 * since their boundaries (and all constellations'
			 * boundaries) get drawn twice, once by each bordering
			 * constellation.
			 */
			print = (NORTH && dec_deg > 0.0 ||
				 SOUTH && dec_deg < 0.0) &&
				strcmp(curcon, "UMI") != 0 &&
				strcmp(curcon, "OCT") != 0;

			if (print) {
				/* first point in new constellation */
				printf("newpath %% %s\n", newcon);
				printf("%.2f %.2f moveto\n",
					r * cos(theta_rad), r * sin(theta_rad));
			}

		} else if (print) {

			/* another point in the current constellation */

			printf("%.2f %.2f lineto\n",
					r * cos(theta_rad), r * sin(theta_rad));
		}
	}

	/* end the last constellation */
	if (print) {
		printf("closepath stroke\n");
	}

	printf("grestore\n");
}



/*
 * Function:	print_boundaries_mid
 *
 * Description:	Draw all constellation boundaries for a midsection map.
 *
 * Returns:	void
 */

void
print_boundaries_mid(

FILE *bd_p		/* boundary data file */
)
{
	float ra_hour, dec_deg;	/* for a point on a constellation boundary */
	float x, y;		/* coordinates for a point */
	float prev_ra_hour;	/* for previous point */
	char buf[80];		/* holds one line from the file */
	char curcon[10];	/* current constellation's abbreviation */
	char newcon[10];	/* new constellation's abbreviation */


	/* loop once for every line in the coord file */
	curcon[0] = '\0';	/* no current constellation */
	while (fgets(buf, sizeof (buf), bd_p) != NULL) {

		/* read a line representing a point; apply precession */
		sscanf(buf, "%f %f %s", &ra_hour, &dec_deg, newcon);
		precess(&ra_hour, &dec_deg);

		/* deal with RA wraparound */
		if (Map == 6 && ra_hour < 6.0) {
			ra_hour += 24.0;
		} else if (Map == 3 && ra_hour > 18.0) {
			ra_hour -= 24.0;
		}

		x = RA_HOUR2X(ra_hour);
		y = DEC_DEG2Y(dec_deg);

		if (strcmp(curcon, newcon) != 0) {

			/* starting a new constellation */

			/* end previous constellation if there is one */
			if (curcon[0] != '\0') {
				printf("closepath stroke\n");
			}

			/* new one becomes the current one */
			strcpy(curcon, newcon);

			/* first point in new constellation */
			printf("newpath %% %s\n", newcon);
			printf("%.2f %.2f moveto\n", x, y);

			prev_ra_hour = ra_hour;

		} else {

			/* another point in the current constellation */

			/* deal with RA wraparound */
			if (ra_hour - prev_ra_hour > 12.0) {
				ra_hour -= 24.0;
				x = RA_HOUR2X(ra_hour);
			} else if (ra_hour - prev_ra_hour < -12.0) {
				ra_hour += 24.0;
				x = RA_HOUR2X(ra_hour);
			}

			printf("%.2f %.2f lineto\n", x, y);
		}

		prev_ra_hour = ra_hour;
	}

	/* end the last constellation */
	printf("closepath stroke\n");
}



/*
 * Function:	print_boundaries_center
 *
 * Description:	Draw all constellation boundaries for a centered map.
 *
 * Returns:	void
 */

void
print_boundaries_center(

FILE *bd_p		/* boundary data file */
)
{
	/*
	 * The "1" variables are for the previous point on the boundary.
	 * The "2" variables are for the current point on the boundary.
	 * The "first" variables are for the first point on the boundary.
	 */
	float x1, x2, firstx;			/* x coord */
	float y1, y2, firsty;			/* y coord */
	enum CENTERPOS pos1, pos2, firstpos;	/* on screen position */

	float ra_hour, dec_deg;	/* for the current point */
	char buf[80];		/* holds one line from the file */
	char curcon[10];	/* current constellation's abbreviation */
	char newcon[10];	/* new constellation's abbreviation */
	bool print;		/* should we print this constellation? */


	/* loop once for every line in the coord file */
	curcon[0] = '\0';	/* no current constellation */
	while (fgets(buf, sizeof (buf), bd_p) != NULL) {

		/* read a line representing a point; apply precession */
		sscanf(buf, "%f %f %s", &ra_hour, &dec_deg, newcon);
		precess(&ra_hour, &dec_deg);

		/* find position of this point */
		pos2 = center_xy(ra_hour, dec_deg, &x2, &y2);

		if (strcmp(curcon, newcon) != 0) {

			/* starting a new constellation */

			/* end previous constellation if there is one */
			if (curcon[0] != '\0' && print &&
			    pos1 != CP_NOTSET && firstpos != CP_NOTSET) {
				printf("newpath %.2f %.2f moveto\n", x1, y1);
				printf("%.2f %.2f lineto stroke\n",
						firstx, firsty);
			}

			/* new one becomes the current one */
			strcpy(curcon, newcon);

			/*
			 * Exclude Ursa Minor and Octans; their boundaries
			 * have problems due to the poles.  We can skip them,
			 * since their boundaries (and all constellations'
			 * boundaries) get drawn twice, once by each bordering
			 * constellation.
			 */
			print = strcmp(curcon, "UMI") != 0 &&
				strcmp(curcon, "OCT") != 0;

			if (print) {
				printf("%% %s\n", newcon);
			}

			/* remember this point as the first one */
			firstpos = pos2;
			firstx = x2;
			firsty = y2;

		} else {

			/* another point in the current constellation */

			if (pos1 != CP_NOTSET && pos2 != CP_NOTSET && print) {
				printf("newpath %.2f %.2f moveto\n", x1, y1);
				printf("%.2f %.2f lineto stroke\n", x2, y2);
			}
		}

		/* remember current point as previous */
		pos1 = pos2;
		x1 = x2;
		y1 = y2;
	}

	/* end last constellation */
	if (pos1 != CP_NOTSET && firstpos!= CP_NOTSET && print) {
		printf("newpath %.2f %.2f moveto\n", x1,y1);
		printf("%.2f %.2f lineto stroke\n", firstx, firsty);
	}
}



/*
 * Function:	print_const_names
 *
 * Description:	Print all constellation names.  It reads the data defining
 *		their positions from CONST_NAMES_FILE.
 *
 * Returns:	void
 */

void
print_const_names()
{
	FILE *cn_p;		/* to read constellation name position file */
	char buf[80];		/* hold a line from the file */
	char *con;		/* set to where the name begins in the line */
	char *sf_p;		/* point at suffix for "split flags" */
	bool split;		/* true if name should be split for this map */
	float ra_hour, dec_deg;	/* where center of name should be */
	int midx;		/* map index: which set of file's nums to use */


	cn_p = fopen(CONST_NAMES_FILE, "r");
	if (cn_p == NULL) {
		error("can't read %s", CONST_NAMES_FILE);
	}

	printf("gsave\n");
	printf("%.2f setgray\n", GETGRAY(0.0));
	printf("/Times-Bold findfont\n");
	printf("9 scalefont setfont\n");

	/* loop once per line in the file (constellation) */
	while (fgets(buf, sizeof (buf), cn_p) != NULL) {

		/* constellation name follows the '=' */
		con = strchr(buf, '=') + 1;
		con[strlen(con) - 1] = '\0';	/* zap the newline */

		if (CENTER) {
			/*
			 * Get an estimate of where this constellation is by
			 * reading from the polar set of data.  If it's
			 * actually only in the mid area, this will be the
			 * initial version from the Cartes du Ciel program, not
			 * the version I optiimized; but it's good for ballpark.
			 * Then we call best_midx to see where we really
			 * should look.
			 */
			sscanf(buf, "%f %f %*f %*f %*f %*f", &ra_hour,&dec_deg);
			midx = best_midx(ra_hour, dec_deg);
		} else {
			/* non-center case; we know for sure which to use */
			midx = MIDX;
		}

		/* after it there may be a '/' followed by map index numbers */
		sf_p = strchr(con, '/');
		if (sf_p != NULL) {
			/* split const name if this map's index num is found */
			split = (strchr(sf_p, midx + '0') != NULL);
			*sf_p = '\0';	/* zap the / */
		} else {
			split = false;	/* do not split */
		}

		/* get the correct version of the coords for the const name */
		switch (midx) {
		case 0:
			sscanf(buf, "%f %f %*f %*f %*f %*f", &ra_hour,&dec_deg);
			break;
		case 1:
			sscanf(buf, "%*f %*f %f %f %*f %*f", &ra_hour,&dec_deg);
			break;
		case 2:
			sscanf(buf, "%*f %*f %*f %*f %f %f", &ra_hour,&dec_deg);
			break;
		}

		/* apply precession, if any */
		precess(&ra_hour, &dec_deg);

		if (CIRCUMPOLAR) {
			print_const_names_polar(ra_hour, dec_deg, con, split);
		} else if (MIDSECTION) {
			print_const_names_mid(ra_hour, dec_deg, con, split);
		} else {	/* CENTER */
			print_const_names_center(ra_hour, dec_deg, con, split);
		}
	}
	printf("grestore\n");
	fclose(cn_p);
}



/*
 * Function:	print_const_names_polar
 *
 * Description:	Print all constellation names for a polar map.
 *
 * Returns:	void
 */

void
print_const_names_polar(

float ra_hour,		/* for center of name */
float dec_deg,		/* for center of name */
char *con,		/* the constellation name */
bool split		/* for two word names, put on separate lines? */
)
{
	float theta;
	float r;


	/* if too far from the pole, forget it */
	if (NORTH && dec_deg < 25.0 || SOUTH && dec_deg > -25.0) {
		return;
	}

	/* get polar coordiates */
	theta = HOUR2DEG(ra_hour);
	if (NORTH) {
		theta = -theta;
	}
	r = DEC_DEG2R(dec_deg);

	printf("gsave\n");

	/* move origin to center of screen */
	printf("%.2f %.2f translate\n", MID_X, MID_Y);

	/* rotate to put name on positive Y axis */
	printf("%.2f rotate\n", theta);

	/* print the name */
	print_const_names_common(con, split, 0.0, r);

	printf("grestore\n");
}



/*
 * Function:	print_const_names_mid
 *
 * Description:	Print all constellation names for a midsection map.
 *
 * Returns:	void
 */

void
print_const_names_mid(

float ra_hour,		/* for center of name */
float dec_deg,		/* for center of name */
char *con,		/* the constellation name */
bool split		/* for two word names, put on separate lines? */
)
{
	float x, y;		/* screen coordinates */


	/* deal with RA wraparound */
	if (Map == 6 && ra_hour < 6.0) {
		ra_hour += 24.0;
	}

	x = RA_HOUR2X(ra_hour);
	y = DEC_DEG2Y(dec_deg);

	print_const_names_common(con, split, x, y);
}



/*
 * Function:	print_const_names_center
 *
 * Description:	Print all constellation names for a centered map.
 *
 * Returns:	void
 */

void
print_const_names_center(

float ra_hour,		/* for center of name */
float dec_deg,		/* for center of name */
char *con,		/* the constellation name */
bool split		/* for two word names, put on separate lines? */
)
{
	float x, y;		/* screen coordinates */


	/*
	 * If the coordinates can't be set (wrong hemisphere), get out.
	 * Otherwise print, because even if they are CP_OFFMAP, part of the name
	 * may still be on the map, and we'd like to see that.
	 */
	if (center_xy(ra_hour, dec_deg, &x, &y) == CP_NOTSET) {
		return;
	}

	print_const_names_common(con, split, x, y);
}



/*
 * Function:	print_const_names_common
 *
 * Description:	Do the actual printing of a constellation name.
 *
 * Returns:	void
 */

void
print_const_names_common(

char *con,		/* the constellation name */
bool split,		/* for two word names, put on separate lines? */
float x,		/* x screen coordinate of center of name */
float y			/* y screen coordinate of center of name */
)
{
	char *word2;	/* set to second word of a split two word name */


	/* center the names at the position given */

	if (split) {
		/* a two word name that is to be split onto two lines */
		word2 = strchr(con, ' ') + 1;
		word2[-1] = '\0';	/* zap the blank */
		printf("%.2f %.2f moveto\n", x, y);
		printf("(%s) dup stringwidth pop 2 div neg 0 rmoveto show\n",
				con);
		printf("%.2f %.2f moveto\n", x, y - 9.0);
		printf("(%s) dup stringwidth pop 2 div neg 0 rmoveto show\n",
				word2);
	} else {
		printf("%.2f %.2f moveto\n", x, y);
		printf("(%s) dup stringwidth pop 2 div neg 0 rmoveto show\n",
				con);
	}
}



/*
 * Function:	print_const_outlines
 *
 * Description:	Draw the lines joining the stars in all constellations.  It
 *		reads the data of what stars to join from CONST_LINES_FILE.
 *
 * Returns:	void
 */
#define GAP	2.0

void
print_const_outlines()
{
	FILE *cl_p;		/* to read constellation lines file */
	char buf[80];		/* hold a line from the file */
	int sidx1, sidx2;	/* the numbers of two stars to be joined */
	float x1, y1, x2, y2;	/* screen coords of two stars to be joined */
	float dec1_deg, dec2_deg;	/* dec of the two stars */
	float r_deg;		/* distance from pole of a star */
	float theta_deg;	/* polar coord angle */
	float r1, r2;		/* radii of the stars */
	float deltax, deltay;	/* coordinate differences between two stars */
	float length;		/* dist between centers of the stars */


	cl_p = fopen(CONST_LINES_FILE, "r");
	if (cl_p == NULL) {
		error("can't read %s", CONST_LINES_FILE);
	}

	printf("gsave\n");
	printf("%.2f setgray\n", GETGRAY(GRAY_OUTLINE));
	if (CIRCUMPOLAR) {
		/* move to center of page, point 0 hours upwards */
		printf("%.2f %.2f translate\n", MID_X, MID_Y);
		printf("90 rotate\n");
	}

	printf("%.2f setlinewidth\n", THIN);

	/* loop once per line in the file (a line between two stars) */
	while (fgets(buf, sizeof (buf), cl_p) != NULL) {

		/* get numbers of two stars (indices into Stardata) */
		if (sscanf(buf, "%d %d", &sidx1, &sidx2) != 2) {
			/* not a line with two numbers (should be blank line) */
			continue;
		}

		/*
		 * Find the x, y coords of the two stars.
		 */

		if (CIRCUMPOLAR) {

			dec1_deg = Stardata[sidx1].dec_deg;
			dec2_deg = Stardata[sidx2].dec_deg;

			/* if both stars are off the map, don't draw line */
			if (NORTH && dec1_deg < 35.0 && dec2_deg < 35.0) {
				continue;
			}
			if (SOUTH && dec1_deg > -35.0 && dec2_deg > -35.0) {
				continue;
			}

			/* get star 1 coordinates */
			theta_deg = HOUR2DEG(Stardata[sidx1].ra_hour);
			if (NORTH) {
				theta_deg = -theta_deg;
			}
			r_deg = DEC_DEG2R(dec1_deg);
			x1 = r_deg * cos(DEG2RAD(theta_deg));
			y1 = r_deg * sin(DEG2RAD(theta_deg));

			/* get star 2 coordinates */
			theta_deg = HOUR2DEG(Stardata[sidx2].ra_hour);
			if (NORTH) {
				theta_deg = -theta_deg;
			}
			r_deg = DEC_DEG2R(dec2_deg);
			x2 = r_deg * cos(DEG2RAD(theta_deg));
			y2 = r_deg * sin(DEG2RAD(theta_deg));

		} else if (MIDSECTION) {

			/* the coordinates were already stored */
			x1 = Stardata[sidx1].x;
			y1 = Stardata[sidx1].y;
			x2 = Stardata[sidx2].x;
			y2 = Stardata[sidx2].y;

			/*
			 * If both points are off the screen, skip it.  But
			 * consider the X of the edge to be extended in one
			 * direction or the other, to allow overlap of maps
			 * 3 and 4, and 5 and 6.
			 */
			if (Map == 3 || Map == 5) {
				if ((x1 < Min_x - 100 || x1 > Max_x ||
				     y1 < Min_y || y1 > Max_y) &&
				    (x2 < Min_x - 100 || x2 > Max_x ||
				     y2 < Min_y || y2 > Max_y)) {
					continue;
				}
			} else {	/* map 4 or 6 */
				if ((x1 < Min_x || x1 > Max_x + 100 ||
				     y1 < Min_y || y1 > Max_y) &&
				    (x2 < Min_x || x2 > Max_x + 100 ||
				     y2 < Min_y || y2 > Max_y)) {
					continue;
				}
			}

		} else {	/* centered map */

			/* if either has x,y not set, forget it */
			if (Stardata[sidx1].centerpos == CP_NOTSET ||
			    Stardata[sidx2].centerpos == CP_NOTSET) {
				continue;
			}

			/* if both are off the map, forget it */
			if (Stardata[sidx1].centerpos == CP_OFFMAP &&
			    Stardata[sidx2].centerpos == CP_OFFMAP) {
				continue;
			}

			/* the coordinates were already stored */
			x1 = Stardata[sidx1].x;
			y1 = Stardata[sidx1].y;
			x2 = Stardata[sidx2].x;
			y2 = Stardata[sidx2].y;
		}

		r1 = Stardata[sidx1].radius;
		r2 = Stardata[sidx2].radius;

		/* get x and y distances between stars */
		deltax = x2 - x1;
		deltay = y2 - y1;

		/* distance between centers of the stars */
		length = sqrt(deltax * deltax + deltay * deltay);

		/*
		 * Subtract stars' radii and gap at each end; if remaining line
		 * is too short, forget it.
		 */
		if (length - (r1 + r2 + 2 * GAP) < 1.0) {
			continue;
		}

		/* draw the line */
		printf("newpath %.2f %.2f moveto %.2f %.2f lineto stroke %%stars %d and %d\n",
				x1 + (r1 + GAP) * deltax / length,
				y1 + (r1 + GAP) * deltay / length,
				x2 - (r2 + GAP) * deltax / length,
				y2 - (r2 + GAP) * deltay / length,
				sidx1, sidx2);
	}

	fclose(cl_p);
	printf("grestore\n");
}



/*
 * Function:	print_misc
 *
 * Description:	Draw a few miscellaneous objects.
 *
 * Returns:	void
 */
/* Large Magellanic Cloud */
#define LMC_RA_HOUR		5.42
#define LMC_DEC_DEG		-69.0
#define LMC_R_DEG		3.0
/* Small Magellanic Cloud */
#define SMC_RA_HOUR		0.87
#define SMC_DEC_DEG		-72.7
#define SMC_R_DEG		1.3
/* Pleiades */
#define PLEIADES_RA_HOUR	3.69
#define PLEIADES_DEC_DEG	23.5

void
print_misc(int call)
{
	if (CIRCUMPOLAR) {
		print_misc_polar(call);
	} else if (MIDSECTION) {
		print_misc_mid(call);
	} else {	/* CENTER */
		print_misc_center(call);
	}
}



/*
 * Function:	print_misc_polar
 *
 * Description:	Draw a few miscellaneous objects for the polar maps.
 *
 * Returns:	void
 */

void
print_misc_polar(

int call		/* which call to here */
)
{
	float ra_hour, dec_deg;		/* location of an object */
	float r, theta_deg;		/* its polar coordinates */


	/* none of the objects ever is on the north map, even with precession */
	if (NORTH) {
		return;
	}

	printf("gsave\n");
	printf("%.2f %.2f translate\n", MID_X, MID_Y);

	/* Large Magellanic Cloud */
	ra_hour = LMC_RA_HOUR;
	dec_deg = LMC_DEC_DEG;
	precess(&ra_hour, &dec_deg);	/* apply precession */
	r = DEC_DEG2R(dec_deg);
	theta_deg = HOUR2DEG(ra_hour);
	printf("%.2f rotate\n", theta_deg);
	if (call == 1) {
		printf("%.2f setgray %% LMC\n", GETGRAY(GRAY_CLOUD));
		/*
		 * Approximate it as circular.  We misuse DEC_DEG2R in a funny
		 * way to convert its radius from degrees to points.
		 */
		printf("newpath 0 %.2f %.2f 0 360 arc fill closepath\n",
			r, DEC_DEG2R(90.0 - LMC_R_DEG));
		printf("%.2f setgray\n", GETGRAY(0.0));
	} else if (Print_star_namelet) {
		printf("/Times-Italic findfont\n");
		printf("10 scalefont setfont\n");
		printf("-10 %.2f moveto (LMC) show\n", r - 23.0);
	}
	printf("%.2f rotate\n", -theta_deg);	/* put it back where it was */

	/* Small Magellanic Cloud */
	ra_hour = SMC_RA_HOUR;
	dec_deg = SMC_DEC_DEG;
	precess(&ra_hour, &dec_deg);	/* apply precession */
	r = DEC_DEG2R(dec_deg);
	theta_deg = HOUR2DEG(ra_hour);
	printf("%.2f rotate\n", theta_deg);
	if (call == 1) {
		printf("%.2f setgray %% SMC\n", GETGRAY(GRAY_CLOUD));
		/* same explanation as for LMC */
		printf("newpath 0 %.2f %.2f 0 360 arc fill closepath\n",
			r, DEC_DEG2R(90.0 - SMC_R_DEG));
		printf("%.2f setgray\n", GETGRAY(0.0));
	} else if (Print_star_namelet) {
		printf("/Times-Italic findfont\n");
		printf("10 scalefont setfont\n");
		printf("-25 %.2f moveto (SMC) show\n", r - 12.0);
	}

	printf("grestore\n");

	/* the Pleiades are never on a polar map, even with precession */
}



/*
 * Function:	print_misc_mid
 *
 * Description:	Draw a few miscellaneous objects for the midsection maps.
 *
 * Returns:	void
 */

void
print_misc_mid(

int call		/* which call to here */
)
{
	float ra_hour, dec_deg;		/* location of an object */
	float x, y;			/* its screen coordinates */


	/* Large Magellanic Cloud */
	ra_hour = LMC_RA_HOUR;
	dec_deg = LMC_DEC_DEG;
	precess(&ra_hour, &dec_deg);	/* apply precession */
	if (dec_deg > -70) {	/* part of it may get onto this map */
		x = RA_HOUR2X(ra_hour);
		y = DEC_DEG2Y(dec_deg);
		if (call == 1) {
			if (x > MIN_X - 100 && x < MAX_X + 100) {
				printf("%.2f setgray %% LMC\n", GETGRAY(GRAY_CLOUD));
				/*
				 * Approximate it as circular.  Since it is far
				 * from the equator, on this map we really
				 * should stretch it into an ellipse, like
				 * everything else is stretched.  But just make
				 * it a somewhat bigger circle.
				 */
				printf("newpath %.2f %.2f %.2f 0 360 arc fill closepath\n",
					x, y, 6.0 * LMC_R_DEG);
				printf("%.2f setgray\n", GETGRAY(0.0));
			}
		} else if (Print_star_namelet && y >= Min_y) {
			printf("/Times-Italic findfont\n");
			printf("10 scalefont setfont\n");
			printf("%.2f %.2f moveto (LMC) show\n", x + 8.0, y);
		}
	}

	/* Small Magellanic Cloud */
	ra_hour = SMC_RA_HOUR;
	dec_deg = SMC_DEC_DEG;
	precess(&ra_hour, &dec_deg);	/* apply precession */
	if (dec_deg > -70) {	/* part of it may get onto this map */
		x = RA_HOUR2X(ra_hour);
		y = DEC_DEG2Y(dec_deg);
		if (call == 1) {
			if (x > MIN_X - 100 && x < MAX_X + 100) {
				printf("%.2f setgray %% SMC\n", GETGRAY(GRAY_CLOUD));
				/* same explanation as for LMC */
				printf("newpath %.2f %.2f %.2f 0 360 arc fill closepath\n",
					x, y, 6.0 * SMC_R_DEG);
				printf("%.2f setgray\n", GETGRAY(0.0));
			}
		} else if (Print_star_namelet && y >= Min_y) {
			printf("/Times-Italic findfont\n");
			printf("10 scalefont setfont\n");
			printf("%.2f %.2f moveto (SMC) show\n", x + 8.0, y);
		}
	}

	/* the name of the Pleiades (the stars are drawn like other stars) */
	if (call == 1 && Print_star_namelet) {
		ra_hour = PLEIADES_RA_HOUR;
		dec_deg = PLEIADES_DEC_DEG;
		precess(&ra_hour, &dec_deg);	/* apply precession */
		x = RA_HOUR2X(ra_hour);
		if (x < MIN_X - 100 || x > MAX_X + 100) {
			return;
		}
		y = DEC_DEG2Y(dec_deg);
		printf("/Times-Italic findfont\n");
		printf("10 scalefont setfont\n");
		printf("%.2f %.2f moveto (Pleiades) show\n", x, y);
	}
}



/*
 * Function:	print_misc_center
 *
 * Description:	Draw a few miscellaneous objects for a centered map.
 *
 * Returns:	void
 */

void
print_misc_center(

int call		/* which call to here */
)
{
	float ra_hour, dec_deg;		/* location of an object */
	float x, y;			/* its screen coordinates */


	/* Large Magellanic Cloud */
	ra_hour = LMC_RA_HOUR;
	dec_deg = LMC_DEC_DEG;
	precess(&ra_hour, &dec_deg);	/* apply precession */
	if (center_xy(ra_hour, dec_deg, &x, &y) != CP_NOTSET) {
		if (call == 1) {
			printf("%.2f setgray %% LMC\n", GETGRAY(GRAY_CLOUD));
			/* approximate it as circular */
			printf("newpath %.2f %.2f %.2f 0 360 arc fill closepath\n",
				x, y, 8.0 * LMC_R_DEG / SCALE);
			printf("%.2f setgray\n", GETGRAY(0.0));
		} else if (Print_star_namelet && y >= Min_y) {
			printf("/Times-Italic findfont\n");
			printf("10 scalefont setfont\n");
			printf("%.2f %.2f moveto (LMC) show\n",
				x + 8.0 * LMC_R_DEG / SCALE, y);
		}
	}

	/* Small Magellanic Cloud */
	ra_hour = SMC_RA_HOUR;
	dec_deg = SMC_DEC_DEG;
	precess(&ra_hour, &dec_deg);	/* apply precession */
	if (center_xy(ra_hour, dec_deg, &x, &y) != CP_NOTSET) {
		if (call == 1) {
			printf("%.2f setgray %% SMC\n", GETGRAY(GRAY_CLOUD));
			/* approximate it as circular */
			printf("newpath %.2f %.2f %.2f 0 360 arc fill closepath\n",
				x, y, 8.0 * SMC_R_DEG / SCALE);
			printf("%.2f setgray\n", GETGRAY(0.0));
		} else if (Print_star_namelet && y >= Min_y) {
			printf("/Times-Italic findfont\n");
			printf("10 scalefont setfont\n");
			printf("%.2f %.2f moveto (SMC) show\n",
				x + 8.0 * SMC_R_DEG / SCALE, y);
		}
	}

	/* the name of the Pleiades (the stars are drawn like other stars) */
	if (call == 1 && Print_star_namelet) {
		ra_hour = PLEIADES_RA_HOUR;
		dec_deg = PLEIADES_DEC_DEG;
		precess(&ra_hour, &dec_deg);	/* apply precession */
		if (center_xy(ra_hour, dec_deg, &x, &y) != CP_NOTSET) {
			printf("%.2f setgray\n", GETGRAY(0.0));
			printf("/Times-Italic findfont\n");
			printf("10 scalefont setfont\n");
			printf("%.2f %.2f moveto (Pleiades) show\n", x, y);
		}
	}
}



/*
 * Function:	print_view_fields
 *
 * Description:	Draw circles representing view fields.
 *
 * Returns:	void
 */

void
print_view_fields()
{
	int vf_num;		/* viewfield number */


	printf("gsave %%%% WHY\n");
	printf("%.2f setgray\n", GETGRAY(GRAY_OUTLINE));

	/* loop through each viewfield and draw its circle */
	for (vf_num = 0; vf_num < Num_view_fields; vf_num++) {
		print_one_view_field(vf_num);
	}

	printf("grestore\n");
}



/*
 * Function:	print_one_view_field
 *
 * Description:	Draw a circle representing a view field.
 *
 * Returns:	void
 */

void
print_one_view_field(

int vf_num		/* viewfield number */
)
{
	float alpha_rad;	/* angular diameter of view field */
	float cos_alpha2;	/* cos(alpha/2.0) */
	float sin_alpha2;	/* sin(alpha/2.0) */
	float ra_rad;		/* RA of center of view field, radians */
	float cos_ra;		/* cos(ra_rad) */
	float sin_ra;		/* sin(ra_rad) */
	float dec_rad;		/* dec of center of view field, radians */
	float cos_dec;		/* cos(dec_rad) */
	float sin_dec;		/* sin(dec_rad) */
	int theta_deg;		/* angle around the view field circle */
	float theta_rad;
	float x0, y0, z0;	/* point on circle before rotating */
	float x1, y1, z1;	/* point on circle after rotating for dec */
	float x2, y2, z2;	/* point on circle after rotating for RA */
	enum CENTERPOS pos1, pos2;	/* screen position of the 2 points */
	float sx1, sy1, sx2, sy2;	/* screen coords of 2 points on circle*/
	float pt_ra_rad, pt_dec_rad;	/* final RA and dec of a point, rad */
	float cos_pt_ra_rad;		/* cos(pt_ra_rad) */
	float cos_pt_dec_rad;		/* cos(pt_dec_rad) */
	float pt_ra_hour, pt_dec_deg;	/* final RA and dec of a point, deg */


	/*
	 * Assume the sky is a unit sphere, where
	 *	positive X axis hits vernal equinox
	 *	positive Y axis hits equator at R.A. 6 hours
	 *	positive Z axis hits north pole
	 * Initially, pretend that the view field circle is to be centered on
	 * the vernal equinox.  We will find the coordinates of points at
	 * intervals around the circle.  For each point we will rotate the sky
	 * such that the center of the view field is at the correct RA and dec,
	 * and see where our point ends up.  We will connect the points with
	 * straight lines, approximating a circle.
	 *
	 * If alpha is the (angular) diameter of the view circle, draw any
	 * radius of the sphere that forms a alpha/2 angle with the X axis.  It
	 * intersects the sphere at a point of the view field circle.  The
	 * center (in space, not on the sphere) of the circle is at
	 * (cos(alpha/2), 0, 0), and the radius of the circle is sin(alpha/2).
	 * If theta is any angle,
	 *	(cos(alpha/2), sin(alpha/2)*cos(theta), sin(alpha/2)*sin(theta))
	 * is a point on the circle.
	 */

	alpha_rad = DEG2RAD(Vf_diam_deg[vf_num]);
	cos_alpha2 = cos(alpha_rad / 2.0);
	sin_alpha2 = sin(alpha_rad / 2.0);

	ra_rad = HOUR2RAD(Vf_ra_hour[vf_num]);
	cos_ra = cos(ra_rad);
	sin_ra = sin(ra_rad);

	dec_rad = DEG2RAD(Vf_dec_deg[vf_num]);
	cos_dec = cos(dec_rad);
	sin_dec = sin(dec_rad);

	/* in each loop, find next point on circle and draw line from previous*/
	for (theta_deg = 0; theta_deg <= 360; theta_deg += 2) {

		/* get (x, y, z) of this point */
		theta_rad = DEG2RAD(theta_deg);
		x0 = cos_alpha2;
		y0 = sin_alpha2 * cos(theta_rad);
		z0 = sin_alpha2 * sin(theta_rad);

		/*
		 * Rotate clockwise (looking towards the origin along
		 * the positive Y axis) which would move the center point to
		 * its correct declination.
		 */
		x1 = x0 * cos_dec - z0 * sin_dec;
		y1 = y0;
		z1 = x0 * sin_dec + z0 * cos_dec;

		/*
		 * Rotate counterclockwise (looking towards the origin along
		 * the positive Z axis) which would move the center point from
		 * vernal equinox to its correct RA.
		 */
		x2 = x1 * cos_ra - y1 * sin_ra;
		y2 = x1 * sin_ra + y1 * cos_ra;
		z2 = z1;

		/*
		 * Convert the new x2, y2, z2 for this point to R.A. and dec,
		 * guarding against roundoff error giving out of range values.
		 */
		if (z2 > 1.0) {
			pt_dec_rad = PI / 2.0;
		} else if (z2 < -1.0) {
			pt_dec_rad = -PI / 2.0;
		} else {
			pt_dec_rad = asin(z2);
		}
		cos_pt_dec_rad = cos(pt_dec_rad);
		if (cos_pt_dec_rad == 0.0) {
			pt_ra_rad = 0.0;	/* R.A. is arbitrary at poles */
		} else {
			cos_pt_ra_rad = x2 / cos_pt_dec_rad;
			if (cos_pt_ra_rad > 1.0) {
				pt_ra_rad = 0.0;
			} else if (cos_pt_ra_rad < -1.0) {
				pt_ra_rad = PI;
			} else {
				pt_ra_rad = acos(cos_pt_ra_rad);
			}
		}
		if (y2 < 0.0) {
			pt_ra_rad = -pt_ra_rad + 2.0 * PI;
		}

		pt_ra_hour = RAD2HOUR(pt_ra_rad);
		pt_dec_deg = RAD2DEG(pt_dec_rad);

		/* first loop iteration, just find and save screen coords */
		if (theta_deg == 0) {
			pos1 = center_xy(pt_ra_hour, pt_dec_deg, &sx1, &sy1);
			continue;
		}

		/*
		 * Later iterations, find this point and draw a line if both
		 * points are in this hemisphere.
		 */
		pos2 = center_xy(pt_ra_hour, pt_dec_deg, &sx2, &sy2);

		if (pos1 != CP_NOTSET && pos2 != CP_NOTSET) {
			printf("newpath %.2f %.2f moveto\n", sx1, sy1);
			printf("%.2f %.2f lineto stroke\n", sx2, sy2);
		}

		/* copy this point to prev point */
		sx1 = sx2;
		sy1 = sy2;
		pos1 = pos2;
	}
}



/*
 * Function:	whiteout
 *
 * Description:	Obliterate anything outside the limits of the map we want to
 *		print.
 *
 * Returns:	void
 */

void
whiteout()
{
	printf("gsave\n");
	printf("%.2f setgray\n", GETGRAY(1.0));

	if (CIRCUMPOLAR) {
		whiteout_polar();
	} else if (MIDSECTION) {
		whiteout_mid();
	} else {	/* CENTER */
		whiteout_center();
	}

	printf("grestore\n");
}



/*
 * Function:	whiteout_polar
 *
 * Description:	Obliterate anything outside a polar map.
 *
 * Returns:	void
 */

void
whiteout_polar()
{
	/* draw a giant white donut around the region we want to print */
	printf("gsave\n");
	printf("%.2f %.2f translate\n", MID_X, MID_Y);
	printf("newpath 0 0 %.2f 0 360 arc\n", DEC_DEG2R(35));
	printf("0 0 %.2f 0 360 arc eofill\n", DEC_DEG2R(0));
	printf("grestore\n");
}



/*
 * Function:	whiteout_mid
 *
 * Description:	Obliterate anything outside a midsection map.
 *
 * Returns:	void
 */

void
whiteout_mid()
{
	/* draw big white rectangles next to the good region */

	/* bottom */
	printf("newpath\n");
	printf("%.2f %.2f moveto\n", Min_x - MARGIN, Min_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Min_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Min_y - MARGIN);
	printf("%.2f %.2f lineto\n", Min_x - MARGIN, Min_y - MARGIN);
	printf("closepath fill stroke\n");

	/* top */
	printf("newpath\n");
	printf("%.2f %.2f moveto\n", Min_x - MARGIN, Max_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Max_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Max_y + MARGIN);
	printf("%.2f %.2f lineto\n", Min_x - MARGIN, Max_y + MARGIN);
	printf("closepath fill stroke\n");

	/* left side */
	if (Map == 4 || Map == 6) {
		printf("newpath\n");
		printf("%.2f %.2f moveto\n", Min_x - 0.4, Min_y - MARGIN);
		printf("%.2f %.2f lineto\n", Min_x - 0.4, Min_y + MARGIN);
		printf("%.2f %.2f lineto\n", Min_x - MARGIN, Min_y + MARGIN);
		printf("%.2f %.2f lineto\n", Min_x - MARGIN, Min_y - MARGIN);
		printf("closepath fill stroke\n");
	}

	/* right side */
	if (Map == 3 || Map == 5) {
		printf("newpath\n");
		printf("%.2f %.2f moveto\n", Max_x + 0.4, Min_y - MARGIN);
		printf("%.2f %.2f lineto\n", Max_x + 0.4, Min_y + MARGIN);
		printf("%.2f %.2f lineto\n", Max_x + MARGIN, Min_y + MARGIN);
		printf("%.2f %.2f lineto\n", Max_x + MARGIN, Min_y - MARGIN);
		printf("closepath fill stroke\n");
	}
}



/*
 * Function:	whiteout_center
 *
 * Description:	Obliterate anything outside a centered map.
 *
 * Returns:	void
 */

void
whiteout_center()
{
	/* draw big white rectangles next to the good region */

	/* bottom */
	printf("newpath\n");
	printf("%.2f %.2f moveto\n", Min_x - MARGIN, Min_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Min_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Min_y - MARGIN);
	printf("%.2f %.2f lineto\n", Min_x - MARGIN, Min_y - MARGIN);
	printf("closepath fill stroke\n");

	/* top */
	printf("newpath\n");
	printf("%.2f %.2f moveto\n", Min_x - MARGIN, Max_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Max_y);
	printf("%.2f %.2f lineto\n", Min_x + MARGIN, Max_y + MARGIN);
	printf("%.2f %.2f lineto\n", Min_x - MARGIN, Max_y + MARGIN);
	printf("closepath fill stroke\n");

	/* left side */
	printf("newpath\n");
	printf("%.2f %.2f moveto\n", Min_x, Min_y - MARGIN);
	printf("%.2f %.2f lineto\n", Min_x, Min_y + MARGIN);
	printf("%.2f %.2f lineto\n", Min_x - MARGIN, Min_y + MARGIN);
	printf("%.2f %.2f lineto\n", Min_x - MARGIN, Min_y - MARGIN);
	printf("closepath fill stroke\n");

	/* right side */
	printf("newpath\n");
	printf("%.2f %.2f moveto\n", Max_x, Min_y - MARGIN);
	printf("%.2f %.2f lineto\n", Max_x, Min_y + MARGIN);
	printf("%.2f %.2f lineto\n", Max_x + MARGIN, Min_y + MARGIN);
	printf("%.2f %.2f lineto\n", Max_x + MARGIN, Min_y - MARGIN);
	printf("closepath fill stroke\n");
}



/*
 * Function:	cross_left
 *
 * Description:	If the line segment (x1, y1) to (x2, y2) crosses the left
 *		screen boundary x = Min_x, set *y_ptr to where that happens.
 *		Actually we require that exactly one point intersect the
 *		boundary and least one point lie in the interior of the map.
 *		If it crosses within the screen limits Min_y to Max_y,
 *		return true.
 *
 * Returns:	true or false as stated above
 */

bool
cross_left(

float x1, float y1,	/* the first point */
float x2, float y2,	/* the second point */
float *y_ptr		/* where to store the Y intercept */
)
{
	if (x1 <= Min_x && x2 <= Min_x) {
		return false;	/* both ends left of or on boundary */
	}
	if (x1 > Min_x && x2 > Min_x) {
		return false;	/* both ends right of boundary */
	}

	/* find where segment crosses boundary line */
	*y_ptr = y1 + (y2 - y1) * (Min_x - x1) / (x2 - x1);

	/* if above or below window, it doesn't count */
	if (*y_ptr < Min_y || *y_ptr > Max_y) {
		return false;
	}

	return true;
}



/*
 * Function:	cross_right
 *
 * Description:	If the line segment (x1, y1) to (x2, y2) crosses the right
 *		screen boundary x = Max_x, set *y_ptr to where that happens.
 *		Actually we require that exactly one point intersect the
 *		boundary and least one point lie in the interior of the map.
 *		If it crosses within the screen limits Min_y to Max_y,
 *		return true.
 *
 * Returns:	true or false as stated above
 */

bool
cross_right(

float x1, float y1,	/* the first point */
float x2, float y2,	/* the second point */
float *y_ptr		/* where to store the Y intercept */
)
{
	if (x1 < Max_x && x2 < Max_x) {
		return false;	/* both ends left of boundary */
	}
	if (x1 >= Max_x && x2 >= Max_x) {
		return false;	/* both ends right of or on boundary */
	}

	/* find where segment crosses boundary line */
	*y_ptr = y1 + (y2 - y1) * (Max_x - x1) / (x2 - x1);

	/* if above or below window, it doesn't count */
	if (*y_ptr < Min_y || *y_ptr > Max_y) {
		return false;
	}

	return true;
}



/*
 * Function:	cross_top
 *
 * Description:	If the line segment (x1, y1) to (x2, y2) crosses the top
 *		screen boundary y = Max_y, set *x_ptr to where that happens.
 *		Actually we require that exactly one point intersect the
 *		boundary and least one point lie in the interior of the map.
 *		If it crosses within the screen limits Min_x to Max_x,
 *		return true.
 *
 * Returns:	true or false as stated above
 */

bool
cross_top(

float x1, float y1,	/* the first point */
float x2, float y2,	/* the second point */
float *x_ptr		/* where to store the X intercept */
)
{
	if (y1 < Max_y && y2 < Max_y) {
		return false;	/* both ends below boundary */
	}
	if (y1 >= Max_y && y2 >= Max_y) {
		return false;	/* both ends above or on boundary */
	}

	/* find where segment crosses boundary line */
	*x_ptr = x2 - (x2 - x1) * (y2 - Max_y) / (y2 - y1);

	/* if left or right of window, it doesn't count */
	if (*x_ptr < Min_x || *x_ptr > Max_x) {
		return false;
	}

	return true;
}



/*
 * Function:	cross_bottom
 *
 * Description:	If the line segment (x1, y1) to (x2, y2) crosses the bottom
 *		screen boundary y = Min_y, set *x_ptr to where that happens.
 *		Actually we require that exactly one point intersect the
 *		boundary and least one point lie in the interior of the map.
 *		If it crosses within the screen limits Min_x to Max_x,
 *		return true.
 *
 * Returns:	true or false as stated above
 */

bool
cross_bottom(

float x1, float y1,	/* the first point */
float x2, float y2,	/* the second point */
float *x_ptr		/* where to store the X intercept */
)
{
	if (y1 <= Min_y && y2 <= Min_y) {
		return false;	/* both ends below or on boundary */
	}
	if (y1 > Min_y && y2 > Min_y) {
		return false;	/* both ends above boundary */
	}

	/* find where segment crosses boundary line */
	*x_ptr = x2 - (x2 - x1) * (y2 - Min_y) / (y2 - y1);

	/* if left or right of window, it doesn't count */
	if (*x_ptr < Min_x || *x_ptr > Max_x) {
		return false;
	}

	return true;
}



/*
 * Function:	ra_float2hourmin
 *
 * Description:	Convert right ascension from floating point hours to integer
 *		hours and minutes (to the closest minute).  It assumes
 *		nonnegative input.
 *
 * Returns:	void
 */

void
ra_float2hourmin(

float ra_hour,			/* the input RA */
int *hour_ptr, int *min_ptr	/* where to put the output hours and minutes */
)
{
	ra_hour += 1.0 / 120.0;	/* add 1/2 minute, to round to nearest minute */

	*hour_ptr = ra_hour;
	*min_ptr = (ra_hour - *hour_ptr) * 60.0;
}



/*
 * Function:	dec_float2deg
 *
 * Description:	Convert declination from floating point degrees to integer
 *		degrees (to the closest degree).  It handles negative input.
 *
 * Returns:	void
 */

void
dec_float2deg(

float dec_deg,			/* the input declination */
int *deg_ptr			/* where to put the output */
)
{
	int sign;		/* -1 for negative, else 1 */


	/* make positive so that roundoff will always work right */
	if (dec_deg < 0) {
		sign = -1;
		dec_deg = -dec_deg;
	} else {
		sign = 1;
	}

	dec_deg += 0.5;		/* add 1/2 degree, to round to nearest degree */

	*deg_ptr = dec_deg;
	*deg_ptr *= sign;
}



/*
 * Function:	print_ra_left
 *
 * Description:	This function is given a segment (x1, y1) to (x2, y2) of a
 *		right ascension line.  If the segment crosses the left map
 *		boundary, it prints the right ascension in the margin.
 *
 * Returns:	void
 */

void
print_ra_left(

float x1, float y1,		/* the first point */
float x2, float y2,		/* the second point */
float ra_hour			/* the right ascension to be printed */
)
{
	float x;		/* where to start printing hours */
	float y;		/* where the segment crosses the left edge */
	int hour, min;		/* RA in integers */


	/* find y; if the segment doesn't cross the boundary, get out */
	if (cross_left(x1, y1, x2, y2, &y) == false) {
		return;
	}

	/* get integer values to nearest minute */
	ra_float2hourmin(ra_hour, &hour, &min);

	/* set x according to how many digits in the hour number */
	x = hour <= 9 ? Min_x - 14.0 : Min_x - 20.0;

	if (min == 0) {
		/* print only the hours */
		printf("%.2f %2f moveto (%dh) show\n", x, y - 4, hour);
	} else {
		/* print the hours and minutes */
		printf("%.2f %2f moveto (%dh) show\n", x, y + 1, hour);
		x = min <= 9 ? Min_x - 17 : Min_x - 23;
		printf("%.2f %2f moveto (%dm) show\n", x, y - 9, min);
	}
}



/* we never print RA along the right boundary, so there is no print_ra_right */



/*
 * Function:	print_ra_top
 *
 * Description:	This function is given a segment (x1, y1) to (x2, y2) of a
 *		right ascension line.  If the segment crosses the top map
 *		boundary, it prints the right ascension in the margin.
 *
 * Returns:	void
 */

void
print_ra_top(

float x1, float y1,		/* the first point */
float x2, float y2,		/* the second point */
float ra_hour			/* the right ascension to be printed */
)
{
	float x;		/* where the segment crosses the top edge */
	int hour, min;		/* RA in integers */


	/* find x; if the segment doesn't cross the boundary, get out */
	if (cross_top(x1, y1, x2, y2, &x) == false) {
		return;
	}

	/* get integer values to nearest minute */
	ra_float2hourmin(ra_hour, &hour, &min);

	if (min == 0) {
		/* print only the hours */
		printf("%.2f %2f moveto (%dh) show\n", x - 6, Max_y + 3, hour);
	} else {
		/* print the hours and minutes */
		printf("%.2f %2f moveto (%dh) show\n", x - 6, Max_y + 13, hour);
		printf("%.2f %2f moveto (%dm) show\n", x - 6, Max_y + 3, min);
	}
}



/*
 * Function:	print_ra_bottom
 *
 * Description:	This function is given a segment (x1, y1) to (x2, y2) of a
 *		right ascension line.  If the segment crosses the bottom map
 *		boundary, it prints the right ascension in the margin.
 *
 * Returns:	void
 */

void
print_ra_bottom(

float x1, float y1,		/* the first point */
float x2, float y2,		/* the second point */
float ra_hour			/* the right ascension to be printed */
)
{
	float x;		/* where the segment crosses the bottom edge */
	int hour, min;		/* RA in integers */


	/* find x; if the segment doesn't cross the boundary, get out */
	if (cross_bottom(x1, y1, x2, y2, &x) == false) {
		return;
	}

	/* get integer values to nearest minute */
	ra_float2hourmin(ra_hour, &hour, &min);

	if (min == 0) {
		/* print only the hours */
		printf("%.2f %2f moveto (%dh) show\n", x - 6, Min_y - 11, hour);
	} else {
		/* print the hours and minutes */
		printf("%.2f %2f moveto (%dh) show\n", x - 6, Min_y - 11, hour);
		printf("%.2f %2f moveto (%dm) show\n", x - 6, Min_y - 21, min);
	}
}



/* we never print dec along the left boundary, so there is no print_dec_left */



/*
 * Function:	print_dec_right
 *
 * Description:	This function is given a segment (x1, y1) to (x2, y2) of a
 *		declination line.  If the segment crosses the right map
 *		boundary, it prints the declination in the margin.
 *
 * Returns:	void
 */

void
print_dec_right(

float x1, float y1,		/* the first point */
float x2, float y2,		/* the second point */
float dec_deg			/* the declination to be printed */
)
{
	float y;		/* intersection point; later, where to print */
	int deg;		/* declination as integer */
	char buf[10];		/* hold sign and number */
	float shift;		/* offset of degree symbol (circle) */
	float x;		/* for center of degree symbol */


	/* find y; if the segment doesn't cross the boundary, get out */
	if (cross_right(x1, y1, x2, y2, &y) == false) {
		return;
	}

	/* get integer value to nearest degree */
	dec_float2deg(dec_deg, &deg);

	/* depending on number, put sign in buf, get offset of degree symbol */
	if (deg > 0) {
		sprintf(buf, "%c", '+');
		shift = deg >= 10 ? 22.0 : 16.0;
	} else if (deg < 0) {
		sprintf(buf, "%c", '-');
		shift = deg <= -10 ? 19.5 : 13.5;
	} else if (deg == 0) {
		buf[0] = '\0';
		shift = 9.0;
	}

	/* put (absolute value of) number in buf */
	sprintf(buf + strlen(buf), "%d", abs(deg));

	/* print along right side */
	x = Max_x + shift;
	y -= 3.0;
	printf("%.2f %2f moveto (%s) show\n", Max_x + 1, y, buf);
	printf("0.4 setlinewidth\n");
	printf("newpath %.2f %.2f %.2f 0 360 arc stroke closepath\n",
		x, y + 7.0, 2.0);
}



/*
 * Function:	print_dec_top
 *
 * Description:	This function is given a segment (x1, y1) to (x2, y2) of a
 *		declination line.  If the segment crosses the top map
 *		boundary, it prints the declination in the margin.
 *
 * Returns:	void
 */

void
print_dec_top(

float x1, float y1,		/* the first point */
float x2, float y2,		/* the second point */
float dec_deg			/* the declination to be printed */
)
{
	float x;		/* intersection point; later, where to print */
	int deg;		/* declination as integer */
	char buf[10];		/* hold sign and number */
	float shift;		/* offset of degree symbol (circle) */
	float y;		/* baseline for printing */


	/* find x; if the segment doesn't cross the boundary, get out */
	if (cross_top(x1, y1, x2, y2, &x) == false) {
		return;
	}

	/* get integer value to nearest degree */
	dec_float2deg(dec_deg, &deg);

	/* depending on number, put sign in buf, get offset of degree symbol */
	if (deg > 0) {
		sprintf(buf, "%c", '+');
		shift = deg >= 10 ? 21.0 : 17.0;
	} else if (deg < 0) {
		sprintf(buf, "%c", '-');
		shift = deg >= 10 ? 15.0 : 12.0;
	} else if (deg == 0) {
		buf[0] = '\0';
		shift = 8.0;
	}

	/* put (absolute value of) number in buf */
	sprintf(buf + strlen(buf), "%d", abs(deg));

	/* print along top */
	x -= 12.5;
	y = Max_y + 3.0;
	printf("%.2f %2f moveto (%s) show\n", x, y, buf);
	printf("0.4 setlinewidth\n");
	printf("newpath %.2f %.2f %.2f 0 360 arc stroke closepath\n",
		x + shift, y + 7, 2.0);
}



/*
 * Function:	print_dec_bottom
 *
 * Description:	This function is given a segment (x1, y1) to (x2, y2) of a
 *		declination line.  If the segment crosses the bottom map
 *		boundary, it prints the declination in the margin.
 *
 * Returns:	void
 */

void
print_dec_bottom(

float x1, float y1,		/* the first point */
float x2, float y2,		/* the second point */
float dec_deg			/* the declination to be printed */
)
{
	float x;		/* intersection point; later, where to print */
	int deg;		/* declination as integer */
	char buf[10];		/* hold sign and number */
	float shift;		/* offset of degree symbol (circle) */
	float y;		/* baseline for printing */


	/* find x; if the segment doesn't cross the boundary, get out */
	if (cross_bottom(x1, y1, x2, y2, &x) == false) {
		return;
	}

	/* get integer value to nearest degree */
	dec_float2deg(dec_deg, &deg);

	/* depending on number, put sign in buf, get offset of degree symbol */
	if (deg > 0) {
		sprintf(buf, "%c", '+');
		shift = deg >= 10 ? 27.0 : 23.0;
	} else if (deg < 0) {
		sprintf(buf, "%c", '-');
		shift = deg >= 10 ? 21.0 : 18.0;
	} else if (deg == 0) {
		buf[0] = '\0';
		shift = 14.0;
	}

	/* put (absolute value of) number in buf */
	sprintf(buf + strlen(buf), "%d", abs(deg));

	/* print along bottom */
	x -= 10.0;
	y = Min_y - 11.0;
	printf("%.2f %2f moveto (%s) show\n", x, y, buf);
	printf("0.4 setlinewidth\n");
	printf("newpath %.2f %.2f %.2f 0 360 arc stroke closepath\n",
		x + shift, y + 7, 2.0);
}



/*
 * Function:	precess
 *
 * Description:	This function applies precession of the equinoxes to the given
 *		coordinates, based on the year.
 *
 * Returns:	void
 */

void
precess(

float *ra_hour_ptr,		/* right ascension to be precessed */
float *dec_deg_ptr		/* declination to be precessed */
)
{
	float year_interval;	/* how many years from now we are going to */
	float ra_rad;		/* right ascension in radians */
	float dec_rad;		/* declination in radians */
	float x0, y0, z0;	/* starting coords: sky is a unit sphere where:
				 * positive X axis hits vernal equinox,
				 * positive Y axis hits equator at R.A. 6 hours,
				 * positive Z axis hits north pole */
	float x1, y1, z1;	/* after rotation */
	float x2, y2, z2;	/* after another rotation */
	float x3, y3, z3;	/* after another rotation */
	float cos_dec_rad;
	float cos_ra_rad;
	float p_rad;		/* how far along the precession cycle we are */


	if (Precess_year == EPOCH) {
		/* no need to precess; coords are specified for the epoch */
		return;
	}

	year_interval = Precess_year - EPOCH;
	p_rad = year_interval / 25772.0 * 2.0 * PI;

	ra_rad = HOUR2RAD(*ra_hour_ptr);
	dec_rad = DEG2RAD(*dec_deg_ptr);

	/* convert R.A and dec to x, y, z */
	x0 = cos(dec_rad) * cos(ra_rad);
	y0 = cos(dec_rad) * sin(ra_rad);
	z0 = sin(dec_rad);

	/*
	 * Rotate clockwise (looking towards the origin along the positive X
	 * axis) to bring the ecliptic to the equator.
	 */
	x1 = x0;
	y1 = y0 * Cos_epsilon - z0 * (-Sin_epsilon);
	z1 = y0 * (-Sin_epsilon) + z0 * Cos_epsilon;

	/*
	 * Rotate clockwise (looking towards the origin along the positive Z
	 * axis) to move the node along the ecliptic.
	 */
	x2 = x1 * cos(p_rad) - y1 * sin(p_rad);
	y2 = x1 * sin(p_rad) + y1 * cos(p_rad);
	z2 = z1;

	/*
	 * Rotate counterclockwise (looking towards the origin along the
	 * positive X axis) to re-tilt the ecliptic.
	 */
	x3 = x2;
	y3 = y2 * Cos_epsilon - z2 * Sin_epsilon;
	z3 = y2 * Sin_epsilon + z2 * Cos_epsilon;

	/* convert the new x3, y3, z3 to R.A. and dec */
	dec_rad = asin(z3);
	cos_dec_rad = cos(dec_rad);
	if (cos_dec_rad == 0.0) {
		ra_rad = 0.0;	/* R.A. is arbitrary at poles */
	} else {
		cos_ra_rad = x3 / cos_dec_rad;
		/* guard against roundoff error giving out of range value */
		if (cos_ra_rad > 1.0) {
			ra_rad = 0.0;
		} else if (cos_ra_rad < -1.0) {
			ra_rad = PI;
		} else {
			ra_rad = acos(cos_ra_rad);
		}
	}
	if (y3 < 0.0) {
		ra_rad = -ra_rad + 2.0 * PI;
	}

	*ra_hour_ptr = RAD2HOUR(ra_rad);
	*dec_deg_ptr = RAD2DEG(dec_rad);
}



/*
 * Function:	center_xy
 *
 * Description:	Given right ascension and declination, this function finds the
 *		x and y screeen coordinates for a point on a centered map.
 *
 * Returns:	CP_ONMAP, CP_OFFMAP, or CP_NOTSET
 */

enum CENTERPOS
center_xy(float ra_hour, float dec_deg, float *out_x_ptr, float *out_y_ptr)
{
	float ra_rad;		/* right ascension in radians */
	float dec_rad;		/* declination in radians */
	float x0, y0, z0;	/* coords: sky is a unit sphere where:
				 * positive X axis hits vernal equinox,
				 * positive Y axis hits equator at R.A. 6 hours,
				 * positive Z axis hits north pole */
	float x1, y1, z1;	/* after two rotations */
	float half_width_rad;	/* half width the screen is to cover */
	float ratio;		/* to map between screen points & angular dist*/
	float x, y, r, dist;	/* for mapping from sky to screen */


	/* change input point's coords to radians */
	ra_rad = HOUR2RAD(ra_hour);
	dec_rad = DEG2RAD(dec_deg);

	/* rotate around Z axis (changing RA) such that center is at 6 hours */
	ra_rad += PI / 2.0 - HOUR2RAD(Cen_ra_hour);
	if (ra_rad < 0.0) {
		ra_rad += 2.0 * PI;
	}
	if (ra_rad >= 2.0 * PI) {
		ra_rad -= 2.0 * PI;
	}

	/* convert the new R.A. and (old) dec to x, y, z */
	x0 = cos(dec_rad) * cos(ra_rad);
	y0 = cos(dec_rad) * sin(ra_rad);
	z0 = sin(dec_rad);

	/* rotate clockwise (looking at x axis) to bring center to equator */
	x1 = x0;
	y1 = y0 * cos(-DEG2RAD(Cen_dec_deg)) - z0 * sin(-DEG2RAD(Cen_dec_deg));
	z1 = y0 * sin(-DEG2RAD(Cen_dec_deg)) + z0 * cos(-DEG2RAD(Cen_dec_deg));

	if (y1 <= 0.0) {
		/*
		 * We never display anything in this hemisphere.
		 */
		return CP_NOTSET;
	}

	/*
	 * The screen will be centered on the new R.A. 6 hours, declination 0.
	 * We will treat R.A. as horizontal and dec as vertical.  Now, how to
	 * map our point from the sphere of the sky onto a plane?
	 *
	 * 1. We could do this by projecting the point straight onto the XZ
	 * plane effectively setting its Y value to zero.  The x and z become
	 * the output x and y.  This has the effect of preserving the distances
	 * along circles centered on the point the map is centered on, but
	 * compressing the radial distances, not at all at the center of the
	 * map, but more and more as we move out from there.
	 *
	 * 2. We could instead maintain all the radial distances, by taking the
	 * great (semi)circles that pass through the point the map is centered
	 * on, and unbending them into straight lines on the y = 1 plane.  This
	 * stretches distances along circles centered on the center point, not
	 * at all at the center, but more and more as we move out from the
	 * center.
	 * 
	 * 3. We will take the middle ground, and average the two results.  So
	 * there will be some compressing of radial distances, and some
	 * stretching of the circular distances.  All of this scales such that
	 * the width of the screen covers the requested amount, and therefore
	 * the height covers more.
	 */

	/* half the width of the screen in radians */
	half_width_rad = DEG2RAD(Cen_wid_deg / 2.0);

	/*
	 * Find a ratio for mapping between screen points and angular distance.
	 * Method 1 would be ratio = (Max_x - MID_X) / sin(half_width_rad).
	 * Method 2 would be ratio = (Max_x - MID_X) / half_width_rad.
	 * Our method (3) is sort of an average.
	 */
	ratio = (Max_x - Min_x) / (sin(half_width_rad) + half_width_rad);

	/* set x and y for method 1 */
	x = x1;
	y = z1;

	/* distance from map center for method 1 */
	r = sqrt(x * x + y * y);

	/* distance from map center for method 2 */
	dist = acos(y1);

	/* change x and y to be the values for method 2 */
	if (r != 0) {
		x *= dist / r;
		y *= dist / r;
	}

	/* the output x and y are the average result */
	*out_x_ptr = MID_X + ratio * (x + x1) / 2.0;
	*out_y_ptr = MID_Y + ratio * (y + z1) / 2.0;

	/* if off the screen, return failure */
	if (*out_x_ptr < Min_x || *out_x_ptr > Max_x ||
	    *out_y_ptr < Min_y || *out_y_ptr > Max_y) {
		return CP_OFFMAP;
	}

	return CP_ONMAP;
}



/*
 * Function:	hms_str2float
 *
 * Description:	This function converts a string representing hours, minutes,
 *		and seconds to a floating point number in hours.  These
 *		formats are allowed:
 *
 * 			HOURS
 * 			HOURS h
 * 			HOURS h MINUTES
 * 			HOURS h MINUTES m
 * 			HOURS h MINUTES m SECONDS
 * 			HOURS h MINUTES m SECONDS s
 *
 *		where HOURS, MINUTES, and SECONDS are numbers having no sign
 *		and there are not actually any blanks between tokens.  Only the
 *		last number given may contain a decimal point.  Whatever
 *		numbers are given must be in these ranges:
 *
 *			0 <= hours < 24
 *			0 <= minutes < 60
 *			0 <= seconds < 60
 *
 *		The sum must be less than 24.
 *
 * Returns:	0 if success, -1 if failure.
 */

int
hms_str2float(

char *str,		/* the string input */
float *ans_ptr		/* answer filled in, valid only on success */
)
{
	char *beg_ptr;		/* point at first char of a number */
	char *end_ptr;		/* point after last char of a number */
	char *c_ptr;		/* point at a char */
	float temp;		/* each number as parsed */


	/* only these characters are allowed in the string */
	if (str[0] == '\0' || strspn(str, "0123456789.hms") != strlen(str)) {
		 return -1;
	}

	/* if a decimal point anywhere, it must be only in the last number */
	c_ptr = strchr(str, '.');
	if (c_ptr != NULL) {
		c_ptr++;
		c_ptr += strspn(c_ptr, "0123456789");
		if (*c_ptr != '\0' && *(c_ptr + 1) != '\0') {
			return -1;
		}
	}

	beg_ptr = str;		/* start of hours number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 'h' && *end_ptr != '\0') {
		return -1;	/* can't parse hours or junk after it */
	}
	if (temp >= 24.0) {
		return -1;
	}
	*ans_ptr = temp;
	if (*end_ptr == '\0' || *end_ptr == 'h' && *(end_ptr + 1) == '\0') {
		return 0;	/* nothing, or just "h", after hours */
	}

	beg_ptr = end_ptr + 1;	/* start of minutes number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 'm' && *end_ptr != '\0') {
		return -1;	/* can't parse minutes or junk after it */
	}
	if (temp >= 60.0) {
		 return -1;
	}
	*ans_ptr += temp / 60.0;
	if (*end_ptr == '\0' || *end_ptr == 'm' && *(end_ptr + 1) == '\0') {
		return 0;	/* nothing, or just "m", after minutes */
	}

	beg_ptr = end_ptr + 1;	/* start of seconds number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 's' && *end_ptr != '\0') {
		return -1;	/* can't parse seconds or junk after it */
	}
	if (temp >= 60.0) {
		return -1;
	}
	*ans_ptr += temp / 3600.0;
	if (*end_ptr == '\0' || *end_ptr == 's' && *(end_ptr + 1) == '\0') {
		return 0;	/* nothing, or just "s", after seconds */
	}

	return -1;		/* extra junk after the end */
}



/*
 * Function:	dms_str2float
 *
 * Description:	This function converts a string representing degrees, minutes,
 *		and seconds to a floating point number in degrees.  These
 *		formats are allowed:
 *
 * 			DEGREES
 * 			DEGREES o
 * 			DEGREES o ARCMINUTES
 * 			DEGREES o ARCMINUTES '
 * 			DEGREES o ARCMINUTES ' ARCSECONDS
 * 			DEGREES o ARCMINUTES ' ARCSECONDS "
 *
 *		where DEGREES, ARCMINUTES, and ARCSECONDS are numbers,
 *		ARCMINUTES and ARCSECONDS having no sign, and DEGREES
 *		optionally having a + or - sign, and there are not actually any
 *		blanks between tokens.  Only the last number given may contain
 *		a decimal point.  If DEGREES has a sign, it is assumed to apply
 *		to all the numbers.  Whatever numbers are given must be in
 *		these ranges (disregarding the sign):
 *
 *			0 <= degrees < 360
 *			0 <= arcminutes < 60
 *			0 <= arcseconds < 60
 *
 *		The sum must be less than 360.
 *
 * Returns:	0 if success, -1 if failure.
 */

int
dms_str2float(

char *str,		/* the string input */
float *ans_ptr		/* answer filled in, valid only on success */
)
{
	char *beg_ptr;		/* point at first char of a number */
	char *end_ptr;		/* point after last char of a number */
	char *c_ptr;		/* point at a char */
	float temp;		/* each number as parsed */
	int sign;		/* -1 if negative input, else 1 */


	/* set sign if present, and skip by it */
	if (str[0] == '-') {
		sign = -1;
		str++;
	} else if (str[0] == '+') {
		sign = 1;
		str++;
	} else {
		sign = 1;
	}

	/* only these characters are allowed in the rest of the string */
	if (str[0] == '\0' || strspn(str, "0123456789.o'\"") != strlen(str)) {
		 return -1;
	}

	/* if a decimal point anywhere, it must be only in the last number */
	c_ptr = strchr(str, '.');
	if (c_ptr != NULL) {
		c_ptr++;
		c_ptr += strspn(c_ptr, "0123456789");
		if (*c_ptr != '\0' && *(c_ptr + 1) != '\0') {
			return -1;
		}
	}

	beg_ptr = str;		/* start of degrees number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != 'o' && *end_ptr != '\0') {
		return -1;	/* can't parse hours or junk after it */
	}
	if (temp > 360.0) {
		 return -1;
	}
	*ans_ptr = temp;
	if (*end_ptr == '\0' || *end_ptr == 'o' && *(end_ptr + 1) == '\0') {
		*ans_ptr *= sign;
		return 0;	/* nothing, or just "o", after degrees */
	}

	beg_ptr = end_ptr + 1;	/* start of arcminutes number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != '\'' && *end_ptr != '\0') {
		return -1;	/* can't parse arcminutes or junk after it */
	}
	if (temp >= 60.0) {
		return -1;
	}
	*ans_ptr += temp / 60.0;
	if (*end_ptr == '\0' || *end_ptr == '\'' && *(end_ptr + 1) == '\0') {
		*ans_ptr *= sign;
		return 0;	/* nothing, or just "'", after arcminutes */
	}

	beg_ptr = end_ptr + 1;	/* start of arcseconds number */
	errno = 0;
	temp = strtof(beg_ptr, &end_ptr);
	if (errno != 0 || end_ptr == beg_ptr ||
			*end_ptr != '"' && *end_ptr != '\0') {
		return -1;	/* can't parse arcseconds or junk after it */
	}
	if (temp >= 60.0) {
		return -1;
	}
	*ans_ptr += temp / 3600.0;
	if (*end_ptr == '\0' || *end_ptr == '"' && *(end_ptr + 1) == '\0') {
		*ans_ptr *= sign;
		return 0;	/* nothing, or just "\"", after arcseconds */
	}

	return -1;
}



/*
 * Function:	error
 *
 * Description:	Print an error message and exit.  It is called with parameters
 *		like printf.  It adds some additional output and prints to
 *		stderr.
 *
 * Returns:	does not return; exits with 1
 */

void
error(char *format, ...)
{
	va_list arglist;


	fprintf(stderr, "%s: error: ", Prog);

	va_start(arglist, format);
	vfprintf(stderr, format, arglist);
	va_end(arglist);

	fprintf(stderr, ".\n");

	exit(1);
}

